#include "stdafx.h"
#include "resource.h"
#include "MetafileExplorerDoc.h"
#include "ImageView.h"
#include "QueryTree.h"
#include "IMGmsg.h"
#include "QueryImage.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

IMPLEMENT_DYNCREATE(CImageView, CScrollView)

BEGIN_MESSAGE_MAP(CImageView, CScrollView)
    ON_REGISTERED_MESSAGE(IMG_INVALIDATE, OnInvalidate)
    ON_REGISTERED_MESSAGE(IMG_QUERY_RECT, OnQueryRect)
    ON_WM_SIZE()
END_MESSAGE_MAP()

CImageView::CImageView()
   {
   }

CImageView::~CImageView()
   {
   }

BOOL CImageView::PreCreateWindow(CREATESTRUCT& cs)
{
        // TODO: Modify the Window class or styles here by modifying
        //  the CREATESTRUCT cs

        return CScrollView::PreCreateWindow(cs);
}

void CImageView::OnInitialUpdate()
{
        CScrollView::OnInitialUpdate();
        CRect r;
        GetClientRect(&r);
        SetScrollSizes(MM_TEXT, CSize(r.Width(), r.Height()));

        GetClientRect(&r);
        GetDocument()->SendViewMessage(IMG_SEND_RECT, &r);
}


// CImageView diagnostics

#ifdef _DEBUG
void CImageView::AssertValid() const
{
        CScrollView::AssertValid();
}

void CImageView::Dump(CDumpContext& dc) const
{
        CScrollView::Dump(dc);
}

CMetafileExplorerDoc* CImageView::GetDocument() const // non-debug version is inline
{
        ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CMetafileExplorerDoc)));
        return (CMetafileExplorerDoc*)m_pDocument;
}
#endif //_DEBUG


/****************************************************************************
*                             CImageView::OnDraw
* Inputs:
*       CDC * pDC:
* Result: void
*       
* Effect: 
*       Draws in the view
****************************************************************************/

void CImageView::OnDraw(CDC * pDC)
    {
     HENHMETAFILE meta = GetDocument()->GetMetaFile();
     CRect r;
     r.SetRectEmpty();
     GetDocument()->SendViewMessage(QRY_RECT, &r);
     if(r.IsRectNull())
        GetClientRect(&r);

     SetScrollSizes(MM_TEXT, CSize(max(0, r.Width()), max(0, r.Height())));
     ::EnumEnhMetaFile(pDC->m_hDC, meta, DrawEnum, this, &r);
    } // CImageView::OnDraw


/****************************************************************************
*                            CImageView::DrawEnum
* Inputs:
*       HDC dc: DC into which to draw
*       HANDLETABLE * handles: A handle table created by ::EnumEnhMetaFile
*       CONST ENHMETARECORD * record: The actual record to draw
*       int count: The number of handles in the handle table
*       LPARAM p: (LPARAM)(CImageView *)
* Result: int
*       1, always
* Effect: 
*       Draws the metafile record if it should be drawn
****************************************************************************/

/* static */ int CALLBACK CImageView::DrawEnum(HDC dc, HANDLETABLE * handles, CONST ENHMETARECORD * record, int count, LPARAM p)
    {
     CImageView * me = (CImageView *)p;
     if(me->GetDocument()->QueryShow(record))
        { /* show it */
         ::PlayEnhMetaFileRecord(dc, handles, record, count);
        } /* show it */
     return 1;
    } // CImageView::DrawEnum

/****************************************************************************
*                             CImageView::OnUpdate
* Inputs:
*       CView * pSender:
*       LPARAM lHint:
*       LPARAM pHint:
* Result: void
*       
* Effect: 
*       Updates the view
****************************************************************************/

void CImageView::OnUpdate(CView* pSender, LPARAM lHint, CObject * pHint)
    {
     if(lHint != 0)
        SendMessage((UINT)lHint, (WPARAM)pHint);
    }

/****************************************************************************
*                          CImageView::OnInvalidate
* Inputs:
*       WPARAM: unused
*       LPARAM: unused
* Result: LRESULT
*       Logically void, 0, always
* Effect: 
*       Invalidates the image
****************************************************************************/

LRESULT CImageView::OnInvalidate(WPARAM, LPARAM)
    {
     Invalidate();
     return 0;
    } // CImageView::OnInvalidate

/****************************************************************************
*                             CImageView::OnSize
* Inputs:
*       UINT nType:
*       int cx:
*       int cy:
* Result: void
*       
* Effect: 
*       Handles sizing
****************************************************************************/

void CImageView::OnSize(UINT nType, int cx, int cy)
    {
     CScrollView::OnSize(nType, cx, cy);

     CRect r;
     GetClientRect(&r);
     GetDocument()->SendViewMessage(IMG_SEND_RECT, &r);
    }

/****************************************************************************
*                           CImageView::OnQueryRect
* Inputs:
*       WPARAM: (WPARAM)(CRect *) Rectangle to fill in
*       LPARAM: unused
* Result: LRESULT
*       Logically void, 0, always
* Effect: 
*       Retrieves the client rect
****************************************************************************/

LRESULT CImageView::OnQueryRect(WPARAM wParam, LPARAM)
    {
     CRect * r = (CRect *)wParam;
     GetClientRect(r);
     return 0;
    } // CImageView::OnQueryRect
