// MetafileExplorerWMFDoc.cpp : implementation of the CMetafileExplorerWMFDoc class
//

#include "stdafx.h"
//#include "MetafileExplorer.h"

#include "MetafileExplorerWMFDoc.h"
#include "PlaceableHeader.h"
#include "QueryTree.h"
#include "QueryImage.h"
#include "resource.h"
#include "metaheader.h"
#include "shlwapi.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

// CMetafileExplorerWMFDoc

IMPLEMENT_DYNCREATE(CMetafileExplorerWMFDoc, CMetafileExplorerDoc)

BEGIN_MESSAGE_MAP(CMetafileExplorerWMFDoc, CMetafileExplorerDoc)
//    ON_COMMAND(ID_FILE_SAVE, OnFileSave)
    ON_UPDATE_COMMAND_UI(ID_FILE_SAVE_AS, OnUpdateFileSaveAs)
    ON_UPDATE_COMMAND_UI(ID_FILE_SAVE, OnUpdateFileSave)
    ON_COMMAND(ID_FILE_SAVE, OnFileSave)
    ON_COMMAND(ID_FILE_SAVE_AS, OnFileSaveAs)
END_MESSAGE_MAP()

#ifdef _DEBUG
void CMetafileExplorerWMFDoc::AssertValid() const
{
        CMetafileExplorerDoc::AssertValid();
}

void CMetafileExplorerWMFDoc::Dump(CDumpContext& dc) const
{
        CMetafileExplorerDoc::Dump(dc);
}
#endif //_DEBUG


// CMetafileExplorerDoc construction/destruction

CMetafileExplorerWMFDoc::CMetafileExplorerWMFDoc() : CMetafileExplorerDoc()
   {
   }

CMetafileExplorerWMFDoc::~CMetafileExplorerWMFDoc()
   {
   }

/****************************************************************************
*                    CMetafileExplorerWMFDoc::OnOpenDocument
* Inputs:
*       LPCTSTR filename: File name
* Result: BOOL
*       TRUE if successful
*       FALSE if error
* Effect: 
*       Opens the document by creating a CMetaFile to hold it
****************************************************************************/

BOOL CMetafileExplorerWMFDoc::OnOpenDocument(LPCTSTR filename)
    {
     CFile f;
     if(!f.Open(filename, CFile::modeRead))
        return FALSE;

     ULONGLONG size = f.GetLength();

     // We will assume we have no metafiles > 4.2GB in length...

     CByteArray data;
     data.SetSize((UINT_PTR)size);
     f.Read(data.GetData(), (UINT)size);
     f.Close();

     METAFILEPICT mfp = {MM_TEXT};
     LPBYTE bits = data.GetData();

     ASSERT(sizeof(PLACEABLEMETAHEADER) == 22);

     PLACEABLEMETAHEADER * hdr = (PLACEABLEMETAHEADER*)bits;
     if(hdr->Key != 0x9AC6CDD7)
        { /* not placeable header */
         ASSERT(FALSE); // bad format? // TODO: something smarter here
         return FALSE;
        } /* not placeable header */
     
     bits = bits + sizeof(PLACEABLEMETAHEADER);
     HENHMETAFILE enh = ::SetWinMetaFileBits((UINT)size, bits, NULL, &mfp);
     if(enh == NULL)
        return FALSE;
         
     meta = enh;
     ProcessMetafile(filename);

     return meta != NULL;
    } // CMetafileExplorerWMFDoc::OnOpenDocument

/****************************************************************************
*                 CMetafileExplorerWMFDoc::OnUpdateFileSaveAs
* Inputs:
*       CCmdUI * pCmdUI:
* Result: void
*       
* Effect: 
*       Enables File Save As
****************************************************************************/

void CMetafileExplorerWMFDoc::OnUpdateFileSaveAs(CCmdUI *pCmdUI)
    {
     pCmdUI->Enable(TRUE);
    }

/****************************************************************************
*                  CMetafileExplorerWMFDoc::OnUpdateFileSave
* Inputs:
*       CCmdUI * pCmdUI
* Result: void
*       
* Effect: 
*       Enables FileSaveAs
****************************************************************************/

void CMetafileExplorerWMFDoc::OnUpdateFileSave(CCmdUI *pCmdUI)
    {
     pCmdUI->Enable(TRUE);
    }


/****************************************************************************
*                   CMetafileExplorerWMFDoc::OnSaveDocument
* Inputs:
*       LPCTSTR PathName: Name of file to save
* Result: BOOL
*       
* Effect: 
*       Saves the file as a .emf file
****************************************************************************/

BOOL CMetafileExplorerWMFDoc::OnSaveDocument(LPCTSTR PathName)
    {
     ENHMETAHEADER * hdr = NULL;
     UpdateAllViews(NULL, QRY_HEADER, (CObject*)&hdr);
     if(hdr == NULL)
        { /* failed */
         ASSERT(FALSE);  // failed to get header
        } /* failed */

     CString description;
     description = AfxGetApp()->m_pszAppName;

     CStringW pictureW;
     pictureW = GetDescription(hdr);
         
     CString picture(pictureW); // in case an ANSI app...

     //----------------------------------------------------------------
     // Desired string, where  is a '\0'
     //
     //   APPNAMEPICTURE
     //   |<--->| |<--->|
     //      |       |
     //      |       picture.GetLength()
     //      description.GetLength()
     //----------------------------------------------------------------
     int len = description.GetLength();
     LPTSTR p = description.GetBuffer(len + picture.GetLength() + 3);
     LPTSTR p1 = &p[len + 2];

     //----------------------------------------------------------------
     //   APPNAME............
     //   ^       ^
     //   |       |
     //   p       p1
     //----------------------------------------------------------------
     _tcscpy(p1, picture); // safe because buffer length has been computed properly!

     //----------------------------------------------------------------
     //   APPNAMEPICTURE.
     //   ^       ^       ^
     //   |       |       |
     //   p       p1      &p1[picture.GetLength()]
     //
     //----------------------------------------------------------------
     p1[picture.GetLength() + 1] = _T('\0');

     //----------------------------------------------------------------
     //   APPNAMEPICTURE
     //   ^       ^       ^
     //   |       |       |
     //   p       p1      &p1[picture.GetLength() + 1]
     //
     //----------------------------------------------------------------
     
     CRect r(0,0,0,0);

     r = *(RECT*)&hdr->rclBounds;

     // Now convert the pixel units to 0.01mm units
     
     PixelsTo01mm(r);
     
     HDC refDC = ::GetDC(NULL);
     HDC metadc = CreateEnhMetaFile(refDC, PathName, &r, p1);
     if(metadc == NULL)
        { /* failed */
         DWORD err = ::GetLastError();
         ::ReleaseDC(NULL, refDC);
         description.ReleaseBuffer();
         return FALSE;
        } /* failed */

     else
        { /* play it, sam */
         ::SetViewportExtEx(metadc, hdr->szlDevice.cx, hdr->szlDevice.cy, NULL);
         ::SetMapMode(metadc, MM_ANISOTROPIC);
         ::SetWindowOrgEx(metadc, hdr->rclBounds.left, hdr->rclBounds.top, NULL);
         ::SetWindowExtEx(metadc, hdr->rclBounds.right, hdr->rclBounds.bottom, NULL);
         
         ::PlayEnhMetaFile(metadc, meta, &r);
         description.ReleaseBuffer();
        } /* play it, sam */

     HENHMETAFILE mf = ::CloseEnhMetaFile(metadc);
     ::DeleteEnhMetaFile(mf);

     ::ReleaseDC(NULL, metadc);

     return TRUE;
    }

/****************************************************************************
*                     CMetafileExplorerWMFDoc::OnFileSave
* Result: void
*       
* Effect: 
*       Does a Save
****************************************************************************/

void CMetafileExplorerWMFDoc::OnFileSave()
    {
     if(EMFName.IsEmpty())
        DoSave(NULL);
     else
        DoSave(EMFName);
    }

/****************************************************************************
*                    CMetafileExplorerWMFDoc::OnFileSaveAs
* Result: void
*       
* Effect: 
*       Does a Save As
****************************************************************************/

void CMetafileExplorerWMFDoc::OnFileSaveAs()
    {
     DoSave(NULL);
    }

/****************************************************************************
*                       CMetafileExplorerWMFDoc::DoSave
* Inputs:
*       LPCTSTR name: File name to use
* Result: void
*       
* Effect: 
*       Saves the file
****************************************************************************/

void CMetafileExplorerWMFDoc::DoSave(LPCTSTR name)
    {
     CString newname;

     if(name != NULL)
        newname = name;
     
     if(newname.IsEmpty())
        { /* prompt for name */
         // Note that because we are writing an EMF file, and the default document
         // string for this class is WMF files, we cannot use CDocument::GetDocumentString
         // or use the default OnSave/OnSaveAs handlers, or any other default mechanisms
         
         CString filter;
         filter.LoadString(IDS_EMF_FILTER);
         
         CString ext = _T(".emf"); // TODO: Should use my subroutine that extracts this from the filter...

         CString DefaultName;
         DefaultName = GetPathName();

         LPTSTR p = DefaultName.GetBuffer(MAX_PATH);
         PathRenameExtension(p, ext); 
         DefaultName.ReleaseBuffer();

         CFileDialog dlg(FALSE, ext, DefaultName,
                         OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT | OFN_PATHMUSTEXIST,
                         filter);
         if(dlg.DoModal() != IDOK)
            return;
         newname = dlg.GetPathName();
        } /* prompt for name */

     if(OnSaveDocument(newname))
        EMFName = newname;
    } // CMetafileExplorerWMFDoc::DoSave
