// MetafileTreeView.cpp : implementation of the CMetafileTreeView class
//

#include "stdafx.h"
#include "resource.h"

#include "MetafileExplorerDoc.h"
#include "MetafileTreeView.h"

#include "TreeDecode.h"
#include "EMFmsg.h"
#include "QueryTree.h"
#include "SelectionChanged.h"
#include "MoveBy.h"
#include "StateIcon.h"
#include "metaheader.h"
#include <shlwapi.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#endif


// CMetafileTreeView

IMPLEMENT_DYNCREATE(CMetafileTreeView, CTreeView)

BEGIN_MESSAGE_MAP(CMetafileTreeView, CTreeView)
    ON_NOTIFY_REFLECT(TVN_SELCHANGED, OnTvnSelchanged)
    ON_REGISTERED_MESSAGE(EMF_FILE, OnAddFileName)
    ON_REGISTERED_MESSAGE(EMF_RECORD, OnAddRecord)
    ON_REGISTERED_MESSAGE(QRY_NAME, OnQueryName)
    ON_REGISTERED_MESSAGE(QRY_RECT, OnQueryRect)
    ON_REGISTERED_MESSAGE(QRY_SHOWREQUEST, OnQueryShowRequest)
    ON_REGISTERED_MESSAGE(QRY_HEADER, OnQueryHeader)
    ON_REGISTERED_MESSAGE(QRY_ENABLE, OnQueryEnable)
    ON_REGISTERED_MESSAGE(UPD_MOVE_UP, OnMoveUp)
    ON_REGISTERED_MESSAGE(UPD_MOVE_DOWN, OnMoveDown)
    ON_COMMAND(ID_FILE_SAVETREETEXT, OnFileSavetreetext)
END_MESSAGE_MAP()

// CMetafileTreeView construction/destruction

CMetafileTreeView::CMetafileTreeView()
{
        // TODO: add construction code here

}

CMetafileTreeView::~CMetafileTreeView()
{
}

BOOL CMetafileTreeView::PreCreateWindow(CREATESTRUCT& cs)
{
        // TODO: Modify the Window class or styles here by modifying
        //  the CREATESTRUCT cs

        return CTreeView::PreCreateWindow(cs);
}

/****************************************************************************
*                   CMetafileTreeView::OnInitialUpdate
* Result: void
*       
* Effect: 
*       Handles initial update
****************************************************************************/

void CMetafileTreeView::OnInitialUpdate()
   {
    CTreeView::OnInitialUpdate();
    CTreeCtrl & ctl = GetTreeCtrl();
    GetDocument()->updateControls();
    ctl.ModifyStyle(0, TVS_FULLROWSELECT | TVS_HASBUTTONS | TVS_HASLINES | TVS_LINESATROOT | TVS_SHOWSELALWAYS);

    imagelist.Create(IDB_STATE, 0, 0, RGB(255,255,255));
    ctl.SetImageList(&imagelist, TVSIL_STATE);
   }

// CMetafileTreeView diagnostics

#ifdef _DEBUG
void CMetafileTreeView::AssertValid() const
{
        CTreeView::AssertValid();
}

void CMetafileTreeView::Dump(CDumpContext& dc) const
{
        CTreeView::Dump(dc);
}

CMetafileExplorerDoc* CMetafileTreeView::GetDocument() const // non-debug version is inline
{
        ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CMetafileExplorerDoc)));
        return (CMetafileExplorerDoc*)m_pDocument;
}
#endif //_DEBUG


/****************************************************************************
*                                EMR_ decodes
****************************************************************************/

#define EMRNAME(x) { _T(#x), x}

static const struct {
   LPCTSTR name;
   UINT id;
} EMRdecode[] = {
    EMRNAME(EMR_HEADER),
    EMRNAME(EMR_POLYBEZIER),
    EMRNAME(EMR_POLYGON),
    EMRNAME(EMR_POLYLINE),
    EMRNAME(EMR_POLYBEZIERTO),
    EMRNAME(EMR_POLYLINETO),
    EMRNAME(EMR_POLYPOLYLINE),
    EMRNAME(EMR_POLYPOLYGON),
    EMRNAME(EMR_SETWINDOWEXTEX),
    EMRNAME(EMR_SETWINDOWORGEX),
    EMRNAME(EMR_SETVIEWPORTEXTEX),
    EMRNAME(EMR_SETVIEWPORTORGEX),
    EMRNAME(EMR_SETBRUSHORGEX),
    EMRNAME(EMR_EOF),
    EMRNAME(EMR_SETPIXELV),
    EMRNAME(EMR_SETMAPPERFLAGS),
    EMRNAME(EMR_SETMAPMODE),
    EMRNAME(EMR_SETBKMODE),
    EMRNAME(EMR_SETPOLYFILLMODE),
    EMRNAME(EMR_SETROP2),
    EMRNAME(EMR_SETSTRETCHBLTMODE),
    EMRNAME(EMR_SETTEXTALIGN),
    EMRNAME(EMR_SETCOLORADJUSTMENT),
    EMRNAME(EMR_SETTEXTCOLOR),
    EMRNAME(EMR_SETBKCOLOR),
    EMRNAME(EMR_OFFSETCLIPRGN),
    EMRNAME(EMR_MOVETOEX),
    EMRNAME(EMR_SETMETARGN),
    EMRNAME(EMR_EXCLUDECLIPRECT),
    EMRNAME(EMR_INTERSECTCLIPRECT),
    EMRNAME(EMR_SCALEVIEWPORTEXTEX),
    EMRNAME(EMR_SCALEWINDOWEXTEX),
    EMRNAME(EMR_SAVEDC),
    EMRNAME(EMR_RESTOREDC),
    EMRNAME(EMR_SETWORLDTRANSFORM),
    EMRNAME(EMR_MODIFYWORLDTRANSFORM),
    EMRNAME(EMR_SELECTOBJECT),
    EMRNAME(EMR_CREATEPEN),
    EMRNAME(EMR_CREATEBRUSHINDIRECT),
    EMRNAME(EMR_DELETEOBJECT),
    EMRNAME(EMR_ANGLEARC),
    EMRNAME(EMR_ELLIPSE),
    EMRNAME(EMR_RECTANGLE),
    EMRNAME(EMR_ROUNDRECT),
    EMRNAME(EMR_ARC),
    EMRNAME(EMR_CHORD),
    EMRNAME(EMR_PIE),
    EMRNAME(EMR_SELECTPALETTE),
    EMRNAME(EMR_CREATEPALETTE),
    EMRNAME(EMR_SETPALETTEENTRIES),
    EMRNAME(EMR_RESIZEPALETTE),
    EMRNAME(EMR_REALIZEPALETTE),
    EMRNAME(EMR_EXTFLOODFILL),
    EMRNAME(EMR_LINETO),
    EMRNAME(EMR_ARCTO),
    EMRNAME(EMR_POLYDRAW),
    EMRNAME(EMR_SETARCDIRECTION),
    EMRNAME(EMR_SETMITERLIMIT),
    EMRNAME(EMR_BEGINPATH),
    EMRNAME(EMR_ENDPATH),
    EMRNAME(EMR_CLOSEFIGURE),
    EMRNAME(EMR_FILLPATH),
    EMRNAME(EMR_STROKEANDFILLPATH),
    EMRNAME(EMR_STROKEPATH),
    EMRNAME(EMR_FLATTENPATH),
    EMRNAME(EMR_WIDENPATH),
    EMRNAME(EMR_SELECTCLIPPATH),
    EMRNAME(EMR_ABORTPATH),

    EMRNAME(EMR_GDICOMMENT),
    EMRNAME(EMR_FILLRGN),
    EMRNAME(EMR_FRAMERGN),
    EMRNAME(EMR_INVERTRGN),
    EMRNAME(EMR_PAINTRGN),
    EMRNAME(EMR_EXTSELECTCLIPRGN),
    EMRNAME(EMR_BITBLT),
    EMRNAME(EMR_STRETCHBLT),
    EMRNAME(EMR_MASKBLT),
    EMRNAME(EMR_PLGBLT),
    EMRNAME(EMR_SETDIBITSTODEVICE),
    EMRNAME(EMR_STRETCHDIBITS),
    EMRNAME(EMR_EXTCREATEFONTINDIRECTW),
    EMRNAME(EMR_EXTTEXTOUTA),
    EMRNAME(EMR_EXTTEXTOUTW),
    EMRNAME(EMR_POLYBEZIER16),
    EMRNAME(EMR_POLYGON16),
    EMRNAME(EMR_POLYLINE16),
    EMRNAME(EMR_POLYBEZIERTO16),
    EMRNAME(EMR_POLYLINETO16),
    EMRNAME(EMR_POLYPOLYLINE16),
    EMRNAME(EMR_POLYPOLYGON16),
    EMRNAME(EMR_POLYDRAW16),
    EMRNAME(EMR_CREATEMONOBRUSH),
    EMRNAME(EMR_CREATEDIBPATTERNBRUSHPT),
    EMRNAME(EMR_EXTCREATEPEN),
    EMRNAME(EMR_POLYTEXTOUTA),
    EMRNAME(EMR_POLYTEXTOUTW),

    EMRNAME(EMR_SETICMMODE),
    EMRNAME(EMR_CREATECOLORSPACE),
    EMRNAME(EMR_SETCOLORSPACE),
    EMRNAME(EMR_DELETECOLORSPACE),
    EMRNAME(EMR_GLSRECORD),
    EMRNAME(EMR_GLSBOUNDEDRECORD),
    EMRNAME(EMR_PIXELFORMAT),

    EMRNAME(EMR_RESERVED_105),
    EMRNAME(EMR_RESERVED_106),
    EMRNAME(EMR_RESERVED_107),
    EMRNAME(EMR_RESERVED_108),
    EMRNAME(EMR_RESERVED_109),
    EMRNAME(EMR_RESERVED_110),
    EMRNAME(EMR_COLORCORRECTPALETTE),
    EMRNAME(EMR_SETICMPROFILEA),
    EMRNAME(EMR_SETICMPROFILEW),
    EMRNAME(EMR_ALPHABLEND),
    EMRNAME(EMR_SETLAYOUT),
    EMRNAME(EMR_TRANSPARENTBLT),
    EMRNAME(EMR_RESERVED_117),
    EMRNAME(EMR_GRADIENTFILL),
    EMRNAME(EMR_RESERVED_119),
    EMRNAME(EMR_RESERVED_120),
    EMRNAME(EMR_COLORMATCHTOTARGETW),
    EMRNAME(EMR_CREATECOLORSPACEW),
    { NULL, 0}// EOT
};

#undef EMRNAME

// CMetafileTreeView message handlers

/****************************************************************************
*                      CMetafileTreeView::OnAddFileName
* Inputs:
*       WPARAM: (WPARAM)(LPCTSTR): file name
*       LPARAM: unused
* Result: LRESULT
*       Logically void, 0, always
* Effect: 
*       Adds the file name to the control
****************************************************************************/

LRESULT CMetafileTreeView::OnAddFileName(WPARAM wParam, LPARAM)
    {
     CTreeCtrl & ctl = GetTreeCtrl();
     LPCTSTR name = (LPCTSTR) wParam;
     ASSERT(name != NULL);
     
     HTREEITEM item = ctl.InsertItem(name);
     return 0;
    } // CMetafileTreeView::OnAddFileName

/****************************************************************************
*                     CMetafileTreeView::DecodeRecord
* Inputs:
*       HTREEITEM item: item that should be decoded
*       CONST ENHMETARECORD * rec: Meta record
* Result: void
*       
* Effect: 
*       Decodes the record
****************************************************************************/

void CMetafileTreeView::DecodeRecord(HTREEITEM item, CONST ENHMETARECORD * rec)
    {
     CTreeCtrl & ctl = GetTreeCtrl();
     TreeDecode d;
     d.Decode(rec, &ctl, (LPVOID)item);
    } // CMetafileTreeView::DecodeRecord

/****************************************************************************
*                        CMetafileTreeView::OnAddRecord
* Inputs:
*       WPARAM: (WPARAM)(CONST ENHMETARECORD *) rec: Meta record
* Result: void
*       
* Effect: 
*       Adds the metarecord to the display
****************************************************************************/

LRESULT CMetafileTreeView::OnAddRecord(WPARAM wParam, LPARAM)
   {
    CONST ENHMETARECORD * rec = (CONST ENHMETARECORD *)wParam;

     for(int i = 0; EMRdecode[i].name != NULL; i++)
        { /* decode it */
         if(rec->iType == EMRdecode[i].id)
            { /* found it */
             CTreeCtrl & ctl = GetTreeCtrl();
             HTREEITEM root = ctl.GetRootItem();
             HTREEITEM item = ctl.InsertItem(EMRdecode[i].name, root);
             ctl.SetItemData(item, (LPARAM)rec);
             DecodeRecord(item, rec);
            } /* found it */
        } /* decode it */
     return 0;
    } // CMetafileExplorer::OnAddRecord

/****************************************************************************
*                  CMetafileTreeView::OnQueryShowRequest
* Inputs:
*       WPARAM: (WPARAM)(QueryShowInfo *)
*       LPARAM: unused
* Result: LRESULT
*       Logically void, 0, always
* Effect: 
*       Fills in the info->show flag value
****************************************************************************/

LRESULT CMetafileTreeView::OnQueryShowRequest(WPARAM wParam, LPARAM)
    {
     QueryShowInfo * info = (QueryShowInfo *)wParam;
     
     CTreeCtrl & ctl = GetTreeCtrl();
     //================================================================
     //  +--filename   <== A
     //     |
     //     +--- r1    <== B
     //     |
     //     +--- r2    <== C  [selected]
     //     |
     //     +--- r3    <== D
     //     |
     //     :
     //     |
     //     +--- rn   <==  E
     //================================================================

     HTREEITEM root = ctl.GetRootItem();
     HTREEITEM sel = ctl.GetSelectedItem();
     if(sel == NULL)
        { /* No selection */
         info->show = FALSE;
         return 0;
        } /* No selection */
     
     if( root == sel)
        { /* A: root selected */
         info->show = FALSE;
         return 0;
        } /* A: root selected */

     sel = GetMetaRecord(sel); // move up to record-level node

     // This is based on the fact that if there is an item selected, we would
     // encounter it.  Note that we want to stop up-to-and-INCLUDING
     // the selected element
     HTREEITEM next = ctl.GetNextSiblingItem(sel);
     if(next == NULL)
        { /* E: at end */
         info->show = TRUE;
         return 0;
        } /* E: at end */
     
     ENHMETARECORD * nextrec = (ENHMETARECORD *)ctl.GetItemData(next);
     if(info->record >= nextrec)
        { /* D: after selection */
         info->show = FALSE;
         return 0;
        } /* D: after selection */

     info->show = TRUE;
     return 0;
    } // CMetafileTreeView::OnQueryShowRequest

/****************************************************************************
*                    CMetafileTreeView::GetMetaRecord
* Inputs:
*       HTREEITEM item: An item in the tree
* Result: HTREEITEM
*       A new item, the metarecord above the item if the selection was
*       a subitem, the item itself it is a metarecord, NULL if at the root
****************************************************************************/

HTREEITEM CMetafileTreeView::GetMetaRecord(HTREEITEM item)
    {
     CTreeCtrl & ctl = GetTreeCtrl();
     HTREEITEM root = ctl.GetRootItem();
     if(item == root)
        return NULL;   // not possible to get metarecord of root

     while(TRUE)
        { /* scan up in tree */
         HTREEITEM parent = ctl.GetParentItem(item);
         if(parent == root)
            return item;  // it is a valid metarecord item
         item = parent;
        } /* scan up in tree */
    } // CMetafileTreeView::GetMetaRecord

/****************************************************************************
*                         CMetafileTreeView::OnMoveUp
* Inputs:
*       WPARAM: UPDF_BY_LINE or UPDF_BY_DRAW
*       LPARAM: unused
* Result: LRESULT
*       Logically void, 0, always
* Effect: 
*       Moves the selection up, if possible
****************************************************************************/

LRESULT CMetafileTreeView::OnMoveUp(WPARAM wParam, LPARAM)
    {
     CTreeCtrl & ctl = GetTreeCtrl();
     HTREEITEM root = ctl.GetRootItem();
     HTREEITEM sel = ctl.GetSelectedItem();
     HTREEITEM target = NULL;
     if(root == sel)
        { /* selection at root */
         // Should not actually be here, control should have been disabled
         GetDocument()->updateControls();
         return 0;
        } /* selection at root */
     
     // Get the metarecord associated with the selection
     HTREEITEM parent = GetMetaRecord(sel);

     HTREEITEM newsel = ctl.GetPrevSiblingItem(parent);
     if(newsel == NULL)
        { /* was at first item */
         // If at the first item, move the selection upwards
         // to the filename, the root element
         newsel = ctl.GetParentItem(parent);
        } /* was first item */
     else
        { /* internal item */
         if(wParam == UPDF_BY_DRAW)
            { /* line up */
             while(TRUE)
                { /* scan */
                 if(IsDrawable(newsel))
                    break;
                 newsel = ctl.GetPrevSiblingItem(newsel);
                 if(newsel == NULL)
                    break;
                } /* scan */
            } /* line up */
        } /* internal item */

     if(newsel != NULL)
        { /* has selection */
         ctl.SelectItem(newsel);
         ctl.EnsureVisible(newsel);
        } /* has selection */
     GetDocument()->Invalidate();
     GetDocument()->updateControls();

     return 0;
    } // CMetafileTreeView::OnMoveUp

/****************************************************************************
*                    CMetafileTreeView::DownToDrawable
* Inputs:
*       HTREEITEM item: Nominally the item to select
*       WPARAM wParam: Type of drawing
* Result: void
*       
* Effect: 
*       Moves the selection down based on the type of request
****************************************************************************/

void CMetafileTreeView::DownToDrawable(HTREEITEM item, WPARAM wParam)
    {
     CTreeCtrl & ctl = GetTreeCtrl();
     if(wParam == UPDF_BY_DRAW)
        { /* keep going */
         while(TRUE)
            { /* scan */
             if(IsDrawable(item))
                break;
             item = ctl.GetNextSiblingItem(item);
            } /* scan */
        } /* keep going */

     if(item != NULL)
        { /* has selection */
         ctl.SelectItem(item);
         ctl.EnsureVisible(item);
        } /* has selection */

     GetDocument()->Invalidate();
     GetDocument()->updateControls();
     
    } // CMetafileTreeView::DownToDrawable

/****************************************************************************
*                        CMetafileTreeView::OnMoveDown
* Inputs:
*       WPARAM: (WPARAM) UPDF_BY_LINE or UPDF_BY_DRAW
* Result: LRESULT
*       Logically void, 0, always
* Effect: 
*       Moves the selection down, if possible
****************************************************************************/

LRESULT CMetafileTreeView::OnMoveDown(WPARAM wParam, LPARAM)
    {
     CTreeCtrl & ctl = GetTreeCtrl();
     HTREEITEM root = ctl.GetRootItem();
     HTREEITEM sel = ctl.GetSelectedItem();
     if(root == sel)
        { /* selection at root */
         HTREEITEM target = ctl.GetChildItem(root);
         DownToDrawable(target, wParam);
         return 0;
        } /* selection at root */

     // Get the metarecord associated with the selection
     HTREEITEM parent = GetMetaRecord(sel);

     HTREEITEM newsel = ctl.GetNextSiblingItem(parent);
     DownToDrawable(newsel, wParam);
     return 0;
    } // CMetafileTreeView::OnDown

/****************************************************************************
*                      CMetafileTreeView::IsDrawable
* Inputs:
*       HTREEITEM item: Item in question
* Result: BOOL
*       TRUE if item is drawable
*       FALSE if item is not drawable
****************************************************************************/

BOOL CMetafileTreeView::IsDrawable(HTREEITEM item)
    {
     CTreeCtrl & ctl = GetTreeCtrl();

     UINT state = ctl.GetItemState(item, TVIS_STATEIMAGEMASK);

     return state == INDEXTOSTATEIMAGEMASK(StateIcon::Drawing);
    } // CMetafileTreeView::IsDrawable

/****************************************************************************
*                    CMetafileTreeView::OnQueryEnable
* Inputs:
*       WPARAM: (WPARAM)(QueryEnableInfo)
*       LPARAM: unused
* Result: LRESULT
*      Logically void, 0, always
* Effect: 
*       Fills in the fields of the QueryEnableInfo managed by this component
****************************************************************************/

LRESULT CMetafileTreeView::OnQueryEnable(WPARAM wParam, LPARAM)
    {
     QueryEnableInfo * info = (QueryEnableInfo *)wParam;
     CTreeCtrl & ctl = GetTreeCtrl();
     if(ctl.GetCount() == 0)
        { /* empty */
         info->SelectionEmpty = TRUE;
         info->SelectionUp = FALSE;
         info->SelectionDown = FALSE;
         return 0;
        } /* empty */

     info->SelectionEmpty = FALSE;

     HTREEITEM sel = ctl.GetSelectedItem();
     if(sel == NULL)
        { /* no selection */
         info->SelectionUp = FALSE;
         info->SelectionDown = FALSE;
         info->SelectionUpDraw = FALSE;
         info->SelectionDownDraw = FALSE;
         return 0;
        } /* no selection */

     info->SelectionEmpty = FALSE;
     
     HTREEITEM root = ctl.GetRootItem();
     if(root == sel) 
        { /* now at root */
         info->SelectionUp = FALSE;
         info->SelectionUpDraw = FALSE;
         info->SelectionDown = TRUE;
         info->SelectionDownDraw = TRUE;
         return 0;
        } /* now at root */

     info->SelectionUp = TRUE;

     sel = GetMetaRecord(sel);
     if(ctl.GetNextSiblingItem(sel) == NULL)
        info->SelectionDown = FALSE;
     else
        info->SelectionDown = TRUE;

     // Now see if there is a drawing element above or below...

     info->SelectionUpDraw = FALSE; // assume false...
     for(HTREEITEM item = ctl.GetPrevSiblingItem(sel); ; item = ctl.GetPrevSiblingItem(item))
        { /* see if drawable up */
         if(item == NULL)
            { /* hit top */
             info->SelectionUpDraw = TRUE; // we can always go back to the root
             break;
            } /* hit top */
         if(IsDrawable(item))
            { /* hit drawable */
             info->SelectionUpDraw = TRUE;
             break;
            } /* hit drawable */
        } /* see if drawable up */

     info->SelectionDownDraw = FALSE; // assume false
     for(HTREEITEM item = ctl.GetNextSiblingItem(sel); item != NULL; item = ctl.GetNextSiblingItem(item))
        { /* see if drawable down */
         if(IsDrawable(item))
            { /* hit drawable */
             info->SelectionDownDraw = TRUE;
             break;
            } /* hit drawable */
        } /* see if drawable down */
     return 0;
    } // CMetafileTreeView::OnQueryEnable

/****************************************************************************
*                        CMetafileTreeView::GetMetaHeader
*       
* Result: ENHMETAHEADER *
*       Pointer to metaheader, or NULL if error
****************************************************************************/

ENHMETAHEADER * CMetafileTreeView::GetMetaHeader()
    {
     CTreeCtrl & ctl = GetTreeCtrl();
     HTREEITEM root = ctl.GetRootItem();
     if(root == NULL)
        return NULL;  // no root, we should not be here
     HTREEITEM first = ctl.GetChildItem(root);
     if(first == NULL)
        return NULL; // no first child, we should not be here
     
     // Ideally, this should be an EMR_HEADER object
     ENHMETAHEADER * hdr = (ENHMETAHEADER *)ctl.GetItemData(first);
     if(hdr->iType != EMR_HEADER)
        return NULL;  // not a header record, erroneous

     return hdr; // is a header record, return it
    } // CMetafileTreeView::GetMetaHeader

/****************************************************************************
*                     CMetafileTreeView::OnQueryRect
* Inputs:
*       WPARAM: (WPARAM)(CRect *): Rectangle to set.  
*       LPARAM: unused
* Result: LRESULT
*       Logically void, 0, always
* Effect: 
*       Fills in the rectangle based on the header, if the rectangle is the NULL rectangle
*       Otherwise, leaves it alone
* Notes:
*       If there is a problem, the rect value is not changed
****************************************************************************/

LRESULT CMetafileTreeView::OnQueryRect(WPARAM wParam, LPARAM)
    {
     CRect * r = (CRect *)wParam;
     
     // If the rectangle has already been filled in, do not change it
     if(!r->IsRectNull())
        return 0;

     ENHMETAHEADER * hdr = NULL;
     hdr = GetMetaHeader();
     if(hdr == NULL)
        return 0;

     *r = *(RECT*)&hdr->rclBounds;  // use the bounds value given

     return 0;
    } // CMetafileTreeView::OnQueryRect

/****************************************************************************
*                     CMetafileTreeView::OnQueryName
* Inputs:
*       WPARAM: (WPARAM)(CStringW *) Place to put name
*       LPARAM: unused
* Result: LRESULT
*       Logically void, 0, always
* Effect: 
*       Fills in the name from the metaheader
****************************************************************************/

LRESULT CMetafileTreeView::OnQueryName(WPARAM wParam, LPARAM)
    {
     CString * s = (CString *)wParam;

     ENHMETAHEADER * hdr = GetMetaHeader();
     if(hdr == NULL)
        return 0;  // not a header record, erroneous

     *s = GetDescription(hdr);

     return 0;
    } // CMetafileTreeView::OnQueryName

/****************************************************************************
*                       CMetafileTreeView::OnUpdate
* Inputs:
*       CView * pSender:
*       LPARAM lHint:
*       CObject * pHint: type depends on lHint
* Result: void
*       
* Effect: 
*       Whatever needs to be done according to the lHint/pHint combination
****************************************************************************/

void CMetafileTreeView::OnUpdate(CView* pSender, LPARAM lHint, CObject * pHint)
    {
     if(lHint == 0 && pHint == NULL)
        { /* use default */
         CTreeView::OnUpdate(pSender, lHint, pHint);
         return;
        } /* use default */

     SendMessage((UINT)lHint, (WPARAM)pHint);
    }

/****************************************************************************
*                   CMetafileTreeView::OnTvnSelchanged
* Inputs:
*       NMHDR * pNMHDR:
*       LRESULT * pResult:
* Result: void
*       
* Effect: 
*       Forces an update if appropriate
****************************************************************************/

void CMetafileTreeView::OnTvnSelchanged(NMHDR *pNMHDR, LRESULT *pResult)
    {
     LPNMTREEVIEW pNMTreeView = reinterpret_cast<LPNMTREEVIEW>(pNMHDR);
     GetDocument()->SendViewMessage(UPD_SELECTIONCHANGED);
     *pResult = 0;
    }

/****************************************************************************
*                      CMetafileTreeView::OnQueryHeader
* Inputs:
*       WPARAM: (WPARAM)(ENHMETAHEADER **)
*       LPARAM: unused
* Result: LRESULT
*       Logically void, 0, always
* Effect: 
*       Retrieves the ENHMETAHEADER
****************************************************************************/

LRESULT CMetafileTreeView::OnQueryHeader(WPARAM wParam, LPARAM)
    {
     ENHMETAHEADER ** hdr = (ENHMETAHEADER**)wParam;
     
     ENHMETAHEADER * hd = GetMetaHeader();
     *hdr = hd;

     return 0;
    } // CMetafileTreeView::OnQueryHeader

/****************************************************************************
*                         CMetafileTreeView::ShowText
* Inputs:
*       CStdioFile & f: File to write to
*       HTREEITEM item: Item to display
*       UINT indent: Indentation depth
* Result: void
*       
* Effect: 
*       Displays tree text to a file
****************************************************************************/

void CMetafileTreeView::ShowText(CStdioFile & f, HTREEITEM item, UINT indent)
    {
     CString indent1 = CString(_T(' '), indent);
     CString indenter = indent1 + indent1 + indent1;

     CTreeCtrl & ctl = GetTreeCtrl();

     while(item != NULL)
        { /* show siblings */
         CString text = ctl.GetItemText(item);
         f.WriteString(indenter);
         f.WriteString(text);
         f.WriteString(_T("\r\n"));
         HTREEITEM child = ctl.GetChildItem(item);
         if(child != NULL)
            { /* show children */
             ShowText(f, child, indent + 1);
            } /* show children */
         item = ctl.GetNextSiblingItem(item);
        } /* show siblings */
     
    } // CMetafileTreeView::ShowText

/****************************************************************************
*                     CMetafileTreeView::OnSsavetreetext
* Result: void
*       
* Effect: 
*       Opens a file into which the tree text can be saved
****************************************************************************/

void CMetafileTreeView::OnFileSavetreetext()
    {
     CString filter;
     filter.LoadString(IDS_TREE_TEXT_FILTER);
     // TODO: extract extension from filter pattern
     CString ext = _T(".txt");

     CString DefaultName = GetDocument()->GetPathName();
     LPTSTR p = DefaultName.GetBuffer(MAX_PATH);
     PathRenameExtension(p, ext);
     DefaultName.ReleaseBuffer();
     
     CFileDialog dlg(FALSE, ext, DefaultName,
                     OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT | OFN_PATHMUSTEXIST,
                     filter);
     if(dlg.DoModal() != IDOK)
        return;

     CStdioFile f;
     if(!f.Open(dlg.GetPathName(), CFile::modeWrite | CFile::modeCreate))
        { /* failed */
         DWORD err = ::GetLastError();
         // TODO: report error message
         return;
        } /* failed */
     CTreeCtrl & ctl = GetTreeCtrl();
     HTREEITEM item = ctl.GetRootItem();
     
     ShowText(f, item, 0);

     f.Close();
    }
