#include "stdafx.h"
#include "TreeDecode.h"

/****************************************************************************
*                                GET_CTL_ITEM
* Effect: 
*       Declares two variables
*               CTreeCtrl * ctl = (CTreeCtrl *)p1;
*               HTREEITEM item  = (HTREEITEM)p2;
****************************************************************************/

#define GET_CTL_ITEM()      CTreeCtrl * ctl = (CTreeCtrl *)p1;  HTREEITEM item = (HTREEITEM)p2;

/****************************************************************************
*                                   AddItem
* Inputs:
*       CTreeCtrl * ctl: Control type
*       HTREEITEM item: Item in tree control under which fields are added
*       class classtype: Name of the class of the type from which a field is being extracted
*       LPVOID str: A pointer to a type which will be interpreted as classtype
*       class type: Type of the field in the structure
*       fieldname field: The name of the field in the structure.  This must be an integer-typed field
*       const CString & fmt: A formatting string to use
* Result: void
*       
* Effect: 
*       Adds the field name to the tree under the HTREEITEM item, along with its formatted value
****************************************************************************/

#define AddItem(ctl, item, classtype, str, type, field, fmt) { \
             CString t;                                      \
             t.Format(_T("%s %s: ") fmt, _T(#type), _T(#field), (type)((classtype *)str)->field); \
             ctl->InsertItem(t, item); \
}

/****************************************************************************
*                                  AddItemEx
* Inputs:
*       CTreeCtrl * ctl: Control type
*       HTREEITEM item: Item in tree control under which fields are added
*       HTREEITEM var: Variable into which to store the HTREEITEM of the item which is added
*       class classtype: Name of the class of the type from which a field is being extracted
*       LPVOID str: A pointer to a type which will be interpreted as classtype
*       class type: Type of the field in the structure.  This must be an integer-type field
*       fieldname field: The name of the field in the structure
*       const CString & fmt: A formatting string to use
* Result: void
*       
* Effect: 
*       Adds the field name to the tree under the HTREEITEM item, along with
*       its formatted value
*       The HTREEITEM of the newly-added item is saved in var
****************************************************************************/

#define AddItemEx(ctl, item, var, emrtype, str, type, field, fmt) { \
             CString t;                                      \
             t.Format(_T("%s %s: ") fmt, _T(#type), _T(#field), (type)((emrtype *)str)->field); \
             var = ctl->InsertItem(t, item); \
}

/****************************************************************************
*                               AddStringItemW
* Inputs:
*       
* Result: 
*       
* Effect: 
*       
****************************************************************************/

#define AddStringItemW(ctl, item, classtype, str, type, field, size) { \
             CString t;                                      \
             t.Format(_T("%s %s[%s]: L\"%ls\""), _T(#type), _T(#field), _T(#size), (type*)((classtype *)str)->field); \
             ctl->InsertItem(t, item); \
}

/****************************************************************************
*                                  AddFormattedItem
* Inputs:
*       CTreeCtrl * ctl: Control type
*       HTREEITEM item: Item in tree control under which fields are added
*       class classtype: Name of the class of the type from which a field is being extracted
*       LPVOID str: A pointer to a type which will be interpreted as classtype
*       class type: Type of the field in the structure
*       fieldname field: The name of the field in the structure
*       CString (*cvt)(type): The function to be used to format the field
* Result: void
*       
* Effect: 
*       Adds the field name to the tree under the HTREEITEM item, along with
*       its formatted value
****************************************************************************/

#define AddFormattedItem(ctl, item, classtype, str, type, field, cvt) { \
             CString t;                                              \
             t.Format(_T("%s %s: %s"), _T(#type), _T(#field), cvt(((classtype *)str)->field)); \
             ctl->InsertItem(t, item); \
}

/****************************************************************************
*                                  AddFormattedItemEx
* Inputs:
*       CTreeCtrl * ctl: Control type
*       HTREEITEM item: Item in tree control under which fields are added
*       HTREEITEM var: Variable into which to store the HTREEITEM of the item which is added
*       class classtype: Name of the class of the type from which a field is being extracted
*       LPVOID str: A pointer to a type which will be interpreted as classtype
*       class type: Type of the field in the structure
*       fieldname field: The name of the field in the structure
*       CString (*cvt)(type): The function to be used to format the field
* Result: void
*       
* Effect: 
*       Adds the field name to the tree under the HTREEITEM item, along with
*       its formatted value
*       The HTREEITEM of the newly-added item is saved in var
****************************************************************************/

#define AddFormattedItemEx(ctl, item, var, classtype, str, type, field, cvt) {                           \
                       CString t;                                                                        \
                       t.Format(_T("%s %s: %s"), _T(#type), _T(#field), cvt(((classtype *)str)->field)); \
                       var = ctl->InsertItem(t, item);                                                   \
}

/****************************************************************************
*                          AddFormattedItemIndirect
* Inputs:
*       CTreeCtrl * ctl: Control type
*       HTREEITEM item: Item in tree control under which fields are added
*       class classtype: Name of the class of the type from which a field is being extracted
*       LPVOID str: A pointer to a type which will be interpreted as classtype
*       class type: Type of the field in the structure; this is a pointer to the value
*       fieldname field: The name of the field in the structure
*       CString (*cvt)(type): The function to be used to format the field
* Effect: 
*       Adds the field name to the tree under the HTREEITEM item, along with
*       its formatted value
****************************************************************************/

#define AddFormattedItemIndirect(ctl, item, classtype, str, type, field, cvt) {                          \
             CString t;                                                                                  \
             t.Format(_T("%s %s: %s"), _T(#type), _T(#field), cvt(&((classtype *)str)->field));          \
             ctl->InsertItem(t, item);                                                                   \
}

/****************************************************************************
*                                  AddFlags
* Inputs:
*       CString & s: String which is the cumulative flag string, thus far
*       CString & sep: A separator.  Initially empty, it is modified if
*                       a value is added
*       type & options: 'type' is any integer type (DWORD, WORD, ULONG, BYTE, etc.)
*                       and this is the set of bit flags
*       name: The name of the field to be tested.  This bit is removed from the options
* Result: void
*       
* Effect: 
*       If the flag 'name' is present in the value 'options', adds the flag to
*       the string s, updating sep if it will be required to add the next element
****************************************************************************/

#define AddFlag(s, sep, options, name) { \
             if(options & name)          \
                { /* options */          \
                 s += sep + _T(#name);   \
                 sep = _T(" | ");        \
                 options &= ~name;       \
                } /* options */          \
}

/****************************************************************************
*                                    FLAG0
* Inputs:
*       type flag: 'type' is any integer type (DWORD, UINT, BYTE, etc.) the flag
*               value to test; if it is 0, this simply returns the literal string "0"
* Result: void
*       
* Effect: 
*       Returns "0" if the flag is 0.
* Notes:
*       This is the first test done in a flag-formating function
****************************************************************************/

#define FLAG0(flag) {                      \
                     if((flag) == 0)       \
                         return _T("0");   \
                    }

/****************************************************************************
*                                 AddResiduum
* Inputs:
*       CString & s: String which is thus far
*       const CString & sep: A separator
*       type options: 'type' is any integer type; this is what is left after AddFlag
*               values have removed their bits
* Result: void
*       
* Effect: 
*       Formats the remaining bits (if any) as hex and adds them to the string s
****************************************************************************/

#define AddResiduum(s, sep, options) {           \
           if((options) != 0)                    \
              { /* residuum */                   \
               CString t;                        \
               t.Format(_T("0x%x"), (options));  \
               s += sep + t;                     \
              } /* residuum */                   \
           }

/****************************************************************************
*                                 STRINGCASE
* Inputs:
*       type x: type is any integer type; x is the name of a value in an integer
*               field.  The value x must be an exact value found in the field
*               being decoded
* Effect: 
*       If the case matches the switch value, returns the string
****************************************************************************/
                               
#define STRINGCASE(x) case x: return _T(#x)

/****************************************************************************
*                                STRINGDEFAULT
* Inputs:
*       const CString & fmt: A formatting string, typically "%d" or "0x%x"
*       type v: type is any integer type, the value to be formatted
* Effect: 
*       The value is formatted according to the code and returned
****************************************************************************/

#define STRINGDEFAULT(fmt, v) default: { CString s; s.Format(fmt, v); return s; }

/****************************************************************************
*                             TreeDecode::AddVal
* Inputs:
*       CString & target:
*       CString & sep:
*       const CString & val:
* Result: void
*       
* Effect: 
*       Updates the fields
****************************************************************************/

void TreeDecode::AddVal(CString & target, CString & sep, const CString & val)
    {
     if(val.IsEmpty())
        return;
     target += sep + val;
     sep = _T(" | ");
    } // TreeDecode::AddVal

/****************************************************************************
*                           TreeDecode::GetLastItem
* Inputs:
*       CTreeCtrl * ctl:
*       HTREEITEM item: Item to start search at
* Result: HTREEITEM
*       last item in list
****************************************************************************/

HTREEITEM TreeDecode::GetLastItem(CTreeCtrl * ctl, HTREEITEM item)
    {
     while(TRUE)
        { /* get next */
         HTREEITEM next = ctl->GetNextSiblingItem(item);
         if(next == NULL)
            return item;
        } /* get next */
    } // TreeDecode::GetLastItem

/****************************************************************************
*                            TreeDecode::GetWinVer
* Result: DWORD
*       Windows version, e.g., 0x0500 for Windows XP
****************************************************************************/

DWORD TreeDecode::GetWinVer()
    {
     OSVERSIONINFO ver = { sizeof(OSVERSIONINFO) };
     ::GetVersionEx(&ver);
     return MAKEWORD(ver.dwMinorVersion, ver.dwMajorVersion);
    } // TreeDecode::GetWinVer

/****************************************************************************
*                            TreeDecode::SetState
* Inputs:
*       CTreeCtrl * ctl: Control to set
*       HTREEITEM item: Item to set
*       StateIcon::States state: State to set
* Result: void
*       
* Effect: 
*       Sets the icon to indicate the state
****************************************************************************/

void TreeDecode::SetState(CTreeCtrl * ctl, HTREEITEM item, StateIcon::States state)
    {
     ctl->SetItemState(item, INDEXTOSTATEIMAGEMASK((UINT)state), TVIS_STATEIMAGEMASK);
    } // TreeDecode::SetState

/****************************************************************************
*                              TreeDecode::cvtAC
* Inputs:
*       DWORD dwAction:
* Result: CString
*       
****************************************************************************/

CString TreeDecode::cvtAC(DWORD dwAction)
    {
     switch(dwAction)
        { /* action */
         STRINGCASE(CS_ENABLE);
         STRINGCASE(CS_DISABLE);
         STRINGCASE(CS_DELETE_TRANSFORM);
         STRINGDEFAULT(_T("%d"), dwAction);
        } /* action */
    } // TreeDecode::cvtAC

/****************************************************************************
*                             TreeDecode::cvtAD
* Inputs:
*       DWORD mode:
* Result: CString
*       
****************************************************************************/

CString TreeDecode::cvtAD(DWORD mode)
    {
     switch(mode)
        { /* mode */
         STRINGCASE(AD_CLOCKWISE);
         STRINGCASE(AD_COUNTERCLOCKWISE);
         STRINGDEFAULT(_T("%d"), mode);
        } /* mode */
    } // TreeDecode::cvtAD

/****************************************************************************
*                             TreeDecode::cvtBIC
* Inputs:
*       DWORD mode: compression mode
* Result: CString
*       
****************************************************************************/

CString TreeDecode::cvtBIC(DWORD mode)
    {
     switch(mode)
        { /* mode */
         STRINGCASE(BI_RGB);
         STRINGCASE(BI_RLE8);
         STRINGCASE(BI_RLE4);
         STRINGCASE(BI_BITFIELDS);
         STRINGCASE(BI_JPEG);
         STRINGCASE(BI_PNG);
         STRINGDEFAULT(_T("%d"), mode);
        } /* mode */
    } // TreeDecode::cvtBIC

/****************************************************************************
*                             TreeDecode::cvtBKM
* Inputs:
*       UINT iMode:
* Result: CString
*       
****************************************************************************/

CString TreeDecode::cvtBKM(UINT iMode)
    {
     switch(iMode)
        { /* iMode */
         STRINGCASE(TRANSPARENT);
         STRINGCASE(OPAQUE);
         STRINGDEFAULT(_T("%d"), iMode);
        } /* iMode */
    } // TreeDecode::cvtBKM

/****************************************************************************
*                              TreeDecode::cvtBS
* Inputs:
*       UINT brush:
* Result: CString
*
****************************************************************************/

CString TreeDecode::cvtBS(UINT brush)
    {
     switch(brush)
        { /* brush */
         STRINGCASE(BS_DIBPATTERN);
         STRINGCASE(BS_DIBPATTERNPT);
         STRINGCASE(BS_HATCHED);
         STRINGCASE(BS_HOLLOW);
         STRINGCASE(BS_PATTERN);
         STRINGCASE(BS_SOLID);
         STRINGDEFAULT(_T("0x%x"), brush);
        } /* brush */
    } // TreeDecode::cvtBS

/****************************************************************************
*                             TreeDecode::cvtC16
* Inputs:
*       COLOR16 c:
* Result: CString
*       
****************************************************************************/

CString TreeDecode::cvtC16(COLOR16 c)
    {
     CString s;
     s.Format(_T("<%d,%d>"), HIBYTE(c), LOBYTE(c));
     return s;
    } // TreeDecode::cvtC16

/****************************************************************************
*                             TreeDecode::cvtCAF
* Inputs:
*       WORD caf:
* Result: CString
*       
****************************************************************************/

CString TreeDecode::cvtCAF(WORD caf)
    {
     FLAG0(caf);
     CString s;
     CString sep;
     AddFlag(s, sep, caf, CA_NEGATIVE);
     AddFlag(s, sep, caf, CA_LOG_FILTER);
     AddResiduum(s, sep, caf);
     return s;
    } // TreeDecode::cvtCAF

/****************************************************************************
*                             TreeDecode::cvtCCSF
* Inputs:
*       DWORD flags: EMRCREATECOLORSPACEW.dwFlags
* Result: CString
*       
****************************************************************************/

CString TreeDecode::cvtCCSF(DWORD flags)
    {
     switch(flags)
        { /* flags */
         STRINGCASE(CREATECOLORSPACE_EMBEDED);
         STRINGDEFAULT(_T("%d"), flags);
        } /* flags */
    } // TreeDecode::cvtCCSF

/****************************************************************************
*                             TreeDecode::cvtCGPS
* Inputs:
*       DWORD ps:
* Result: CString
*       
****************************************************************************/

CString TreeDecode::cvtCGPS(DWORD ps)
    {
     switch(ps)
        { /* ps */
         STRINGCASE(PS_GEOMETRIC);
         STRINGCASE(PS_COSMETIC);
         STRINGDEFAULT(_T("0x%x"), ps);
        } /* ps */
    } // TreeDecode::cvtCGPS

/****************************************************************************
*                             TreeDecode::cvtCMTT
* Inputs:
*       DWORD flags: EMRCREATECOLORMATCHTOTARGET.dwFlags
* Result: CString
*       
****************************************************************************/

CString TreeDecode::cvtCMTT(DWORD flags)
    {
     switch(flags)
        { /* flags */
         STRINGCASE(COLORMATCHTOTARGET_EMBEDED);
         STRINGDEFAULT(_T("%d"), flags);
        } /* flags */
    } // TreeDecode::cvtCMTT

/****************************************************************************
*                              TreeDecode::cvtCR
* Inputs:
*       COLORREF cr:
* Result: CString
*       Decoded COLORREF
****************************************************************************/

CString TreeDecode::cvtCR(COLORREF cr)
    {
     CString s;
     s.Format(_T("RGB(%d, %d, %d)"), GetRValue(cr), GetGValue(cr), GetBValue(cr));
     return s;
    } // TreeDecode::cvtCR

/****************************************************************************
*                             TreeDecode::cvtCST
* Inputs:
*       LCSCSTYPE cst: Colorspace type
* Result: CString
*       
****************************************************************************/

CString TreeDecode::cvtCST(LCSCSTYPE cst)
    {
     switch(cst)
        { /* cst */
         STRINGCASE(LCS_CALIBRATED_RGB);
         STRINGCASE(LCS_sRGB);
         STRINGCASE(LCS_WINDOWS_COLOR_SPACE);
         STRINGDEFAULT(_T("%d"), cst);
        } /* cst */
    } // TreeDecode::cvtCST

/****************************************************************************
*                              TreeDecode::cvtDBC
* Inputs:
*       DWORD c
* Result: CString
*       
****************************************************************************/

CString TreeDecode::cvtDBC(DWORD c)
   {
    switch(c)
       { /* c */
        STRINGCASE(DIB_PAL_COLORS);
        STRINGCASE(DIB_RGB_COLORS);
        STRINGDEFAULT(_T("0x%x"), c);
       } /* c */
   } // TreeDecode::cvtDBC

/****************************************************************************
*                              TreeDecode::cvtDT
* Inputs:
*       BYTE dt: Drawtype
* Result: CString
*       
****************************************************************************/

CString TreeDecode::cvtDT(BYTE dt)
    {
     switch(dt)
        { /* dt */
         STRINGCASE(PT_MOVETO);
         STRINGCASE(PT_LINETO);
         STRINGCASE(PT_BEZIERTO);
         STRINGDEFAULT(_T("%02x"), dt);
        } /* dt */
    } // TreeDecode::cvtDT

/****************************************************************************
*                              TreeDecode::cvtEC
* Inputs:
*       DWORD ps:
* Result: CString
*       
****************************************************************************/

CString TreeDecode::cvtEC(DWORD ps)
    {
     switch(ps)
        { /* ps */
         STRINGCASE(PS_ENDCAP_FLAT);
         STRINGCASE(PS_ENDCAP_ROUND);
         STRINGCASE(PS_ENDCAP_SQUARE);
         STRINGDEFAULT(_T("0x%x"), ps);
        } /* ps */
    } // TreeDecode::cvtEC

/****************************************************************************
*                              TreeDecode::cvtEJ
* Inputs:
*       DWORD ps:
* Result: CString
*       
****************************************************************************/

CString TreeDecode::cvtEJ(DWORD ps)
    {
     switch(ps)
        { /* ps */
         STRINGCASE(PS_JOIN_BEVEL);
         STRINGCASE(PS_JOIN_MITER);
         STRINGCASE(PS_JOIN_ROUND);
         STRINGDEFAULT(_T("0x%x"), ps);
        } /* ps */
    } // TreeDecode::cvtEJ

/****************************************************************************
*                             TreeDecode::cvtEPS
* Inputs:
*       DWORD ps: Extended pen style
* Result: CString
*       
****************************************************************************/

CString TreeDecode::cvtEPS(DWORD ps)
    {
     CString s;
     CString sep;
     CString s1 = cvtCGPS(ps & PS_TYPE_MASK);
     AddVal(s, sep, s1);
     CString s2 = cvtPS(ps & PS_STYLE_MASK);
     AddVal(s, sep, s2);
     
     if( (ps & PS_TYPE_MASK) == PS_GEOMETRIC)
        { /* geometric */
         CString s3 = cvtEC(ps & PS_ENDCAP_MASK);
         AddVal(s, sep, s3);

         CString s4 = cvtEJ(ps & PS_JOIN_MASK);
         AddVal(s, sep, s4);
        } /* geometric */
     return s;
    } // TreeDecode::cvtEPS

/****************************************************************************
*                              TreeDecode::cvtFF
* Inputs:
*       DWORD iMode: Flood fill mode
* Result: CString
*       
****************************************************************************/

CString TreeDecode::cvtFF(DWORD iMode)
    {
     switch(iMode)
        { /* iMode */
         STRINGCASE(FLOODFILLBORDER);
         STRINGCASE(FLOODFILLSURFACE);
         STRINGDEFAULT(_T("%d"), iMode);
        } /* iMode */
    } // TreeDecode::cvtFF

/****************************************************************************
*                              TreeDecode::cvtFW
* Inputs:
*       LONG fw: Font weight
* Result: CString
*       Decoded font weight
****************************************************************************/

CString TreeDecode::cvtFW(LONG fw)
    {
     switch(fw)
        { /* fw */
         STRINGCASE(FW_DONTCARE);
         STRINGCASE(FW_THIN);
         STRINGCASE(FW_EXTRALIGHT);
         STRINGCASE(FW_LIGHT);
         STRINGCASE(FW_NORMAL);
         STRINGCASE(FW_MEDIUM);
         STRINGCASE(FW_SEMIBOLD);
         STRINGCASE(FW_BOLD);
         STRINGCASE(FW_EXTRABOLD);
         STRINGCASE(FW_HEAVY);
         STRINGDEFAULT(_T("%d"), fw);
        } /* fw */
    } // TreeDecode::cvtFW

/****************************************************************************
*                           TreeDecode::cvtFXPT230
* Inputs:
*       FXPT2DOT30 value: Value to display
* Result: CString
*       
* Notes: 
*       11.000000000000000000000000000000
*         
****************************************************************************/

CString TreeDecode::cvtFXPT230(FXPT2DOT30 value)
    {
     CString result;
     for(int i = 0; i < 30; i++)
        { /* format */
         result = (((value >> i) & 1) ? _T("1") : _T("0")) + result;
        } /* format */
     result = _T(".") + result;
     for(int i = 30; i < 32; i++)
        { /* format digit */
         result = (((value >> i) & 1) ? _T("1") : _T("0")) + result;
        } /* format digit */
     return result;
    } // TreeDecode::cvtFXPT230

/****************************************************************************
*                               TreeDecode::cvtGM
* Inputs:
*       DWORD iGraphicsMode: Graphics mode
* Result: CString
*       Decoded graphics mode
****************************************************************************/

/* static */ CString TreeDecode::cvtGM(DWORD iGraphicsMode)
   {
    switch(iGraphicsMode)
       { /* mode */
        STRINGCASE(GM_COMPATIBLE);
        STRINGCASE(GM_ADVANCED);
        STRINGDEFAULT(_T("%d"), iGraphicsMode);
       } /* mode */
   } // TreeDecode::cvtGM

/****************************************************************************
*                              TreeDecode::cvtGF
* Inputs:
*       ULONG ulMode:
* Result: CString
*       
****************************************************************************/

CString TreeDecode::cvtGF(ULONG ulMode)
    {
     switch(ulMode)
        { /* ulMode */
         STRINGCASE(GRADIENT_FILL_RECT_H);
         STRINGCASE(GRADIENT_FILL_RECT_V);
         STRINGCASE(GRADIENT_FILL_TRIANGLE);
         STRINGDEFAULT(_T("%d"), ulMode);
        } /* ulMode */
    } // TreeDecode::cvtGF

/****************************************************************************
*                               TreeDecode::cvtHT
* Inputs:
*       UINT mode: Graphics mode
* Result: CString
*       Decoded graphics mode
****************************************************************************/

/* static */ CString TreeDecode::cvtHT(UINT mode)
   {
    switch(mode)
       { /* mode */
        STRINGCASE(HS_BDIAGONAL);
        STRINGCASE(HS_CROSS);
        STRINGCASE(HS_DIAGCROSS);
        STRINGCASE(HS_FDIAGONAL);
        STRINGCASE(HS_HORIZONTAL);
        STRINGCASE(HS_VERTICAL);
        STRINGDEFAULT(_T("%d"), mode);
       } /* mode */
   } // TreeDecode::cvtGM

/****************************************************************************
*                              TreeDecode::cvtIC
* Inputs:
*       DWORD iMode: ICM mode
* Result: CString
*       
****************************************************************************/

CString TreeDecode::cvtIC(DWORD iMode)
    {
     switch(iMode)
        { /* iMode */
         STRINGCASE(ICM_ON);
         STRINGCASE(ICM_OFF);
         STRINGCASE(ICM_QUERY);
         STRINGCASE(ICM_DONE_OUTSIDEDC);
         STRINGDEFAULT(_T("%d"), iMode);
        } /* iMode */
    } // TreeDecode::cvtIC

/****************************************************************************
*                              TreeDecode::cvtII
* Inputs:
*       WORD ii:
* Result: CString
*       
****************************************************************************/

CString TreeDecode::cvtII(WORD ii)
    {
     switch(ii)
        { /* ii */
         STRINGCASE(ILLUMINANT_DEVICE_DEFAULT);
         STRINGCASE(ILLUMINANT_A);
         STRINGCASE(ILLUMINANT_B);
         STRINGCASE(ILLUMINANT_C);
         STRINGCASE(ILLUMINANT_D50);
         STRINGCASE(ILLUMINANT_D55);
         STRINGCASE(ILLUMINANT_D65);
         STRINGCASE(ILLUMINANT_D75);
         STRINGCASE(ILLUMINANT_F2);
         //STRINGCASE(ILLUMINANT_TUNGSTEN);
         //STRINGCASE(ILLUMINANT_DAYLIGHT);
         //STRINGCASE(ILLUMINANT_FLUORESCENT);
         //STRINGCASE(ILLUMINANT_NTSC);
         STRINGDEFAULT(_T("%d"), ii);
        } /* ii */
    } // TreeDecode::cvtII

/****************************************************************************
*                              TreeDecode::cvtIN
* Inputs:
*       LCSGAMUTMATCH intent: LOGCOLORSPACE.lcsIntent
* Result: CString
*       
****************************************************************************/

CString TreeDecode::cvtIN(LCSGAMUTMATCH intent)
    {
     switch(intent)
        { /* intent */
         STRINGCASE(LCS_GM_ABS_COLORIMETRIC);
         STRINGCASE(LCS_GM_BUSINESS);
         STRINGCASE(LCS_GM_GRAPHICS);
         STRINGCASE(LCS_GM_IMAGES);
         STRINGDEFAULT(_T("%d"), intent);
        } /* intent */
    } // TreeDecode::cvtIN

/****************************************************************************
*                               TreeDecode::cvtLBS
* Inputs:
*       UINT style: brush style
* Result: CString
*       Decoded brush style
****************************************************************************/

/* static */ CString TreeDecode::cvtLBS(UINT style)
   {
    switch(style)
       { /* mode */
        STRINGCASE(BS_DIBPATTERN);
        STRINGCASE(BS_DIBPATTERNPT);
        STRINGCASE(BS_HATCHED);
        STRINGCASE(BS_HOLLOW);
        STRINGCASE(BS_PATTERN);
        STRINGCASE(BS_SOLID);
        STRINGDEFAULT(_T("%d"), style);
       } /* mode */
   } // TreeDecode::cvtLBS

/****************************************************************************
*                              TreeDecode::cvtLY
* Inputs:
*       DWORD iMode:
* Result: CString
*       
****************************************************************************/

CString TreeDecode::cvtLY(DWORD iMode)
    {
     FLAG0(iMode);
     
     CString s;
     CString sep;
     AddFlag(s, sep, iMode, LAYOUT_BITMAPORIENTATIONPRESERVED);
     AddFlag(s, sep, iMode, LAYOUT_RTL);
     AddResiduum(s, sep, iMode);
     return s;
    } // TreeDecode::cvtLY

/****************************************************************************
*                              TreeDecode::cvtMM
* Inputs:
*       DWORD mode: Mapping Mode to decode
* Result: CString
*       
****************************************************************************/

CString TreeDecode::cvtMM(DWORD mode)
    {
     switch(mode)
        { /* mode */
         STRINGCASE(MM_TEXT);
         STRINGCASE(MM_HIMETRIC);
         STRINGCASE(MM_LOMETRIC);
         STRINGCASE(MM_HIENGLISH);
         STRINGCASE(MM_LOENGLISH);
         STRINGCASE(MM_TWIPS);
         STRINGCASE(MM_ISOTROPIC);
         STRINGCASE(MM_ANISOTROPIC);
         STRINGDEFAULT(_T("%d"), mode);
        } /* mode */
    } // TreeDecode::cvtMM

/****************************************************************************
*                             TreeDecode::cvtMWT
* Inputs:
*       DWORD iMode: mode
* Result: CString
*       The decoded mode
****************************************************************************/

CString TreeDecode::cvtMWT(DWORD iMode)
    {
     switch(iMode)
        { /* iMode */
         STRINGCASE(MWT_IDENTITY);
         STRINGCASE(MWT_LEFTMULTIPLY);
         STRINGCASE(MWT_RIGHTMULTIPLY);
         STRINGDEFAULT(_T("%d"), iMode);
        } /* iMode */
    } // TreeDecode::cvtMWT

/****************************************************************************
*                              TreeDecode::cvtOB
* Inputs:
*       DWORD handle: Handle offset
* Result: CString
*       
****************************************************************************/

CString TreeDecode::cvtOB(DWORD handle)
    {
     CString s;
     if(handle & 0x80000000)
        s.Format(_T("ENHMETA_STOCK_OBJECT | %s"), cvtSO(handle & ~ENHMETA_STOCK_OBJECT));
     else
        s.Format(_T("%d"), handle);
     return s;
    } // TreeDecode::cvtOB

/****************************************************************************
*                            TreeDecode::cvtPANAS
* Inputs:
*       BYTE b: Byte value
* Result: CString
*       
****************************************************************************/

CString TreeDecode::cvtPANAS(BYTE b)
    {
     switch(b)
        { /* arm style */
         STRINGCASE(PAN_ANY);
         STRINGCASE(PAN_NO_FIT);
         STRINGCASE(PAN_STRAIGHT_ARMS_HORZ);
         STRINGCASE(PAN_STRAIGHT_ARMS_WEDGE);
         STRINGCASE(PAN_STRAIGHT_ARMS_VERT);
         STRINGCASE(PAN_STRAIGHT_ARMS_SINGLE_SERIF);
         STRINGCASE(PAN_STRAIGHT_ARMS_DOUBLE_SERIF);
         STRINGCASE(PAN_BENT_ARMS_HORZ);
         STRINGCASE(PAN_BENT_ARMS_WEDGE);
         STRINGCASE(PAN_BENT_ARMS_VERT);
         STRINGCASE(PAN_BENT_ARMS_SINGLE_SERIF);
         STRINGCASE(PAN_BENT_ARMS_DOUBLE_SERIF);
         STRINGDEFAULT(_T("%d"), b);
        } /* arm style */
    } // TreeDecode::cvtPANAS

/****************************************************************************
*                            TreeDecode::cvtPANCO
* Inputs:
*       BYTE b: Pan contrast
* Result: CString
*       
****************************************************************************/

CString TreeDecode::cvtPANCO(BYTE b)
    {
     switch(b)
        { /* Contrast */
         STRINGCASE(PAN_ANY);
         STRINGCASE(PAN_NO_FIT);
         STRINGCASE(PAN_CONTRAST_NONE);
         STRINGCASE(PAN_CONTRAST_VERY_LOW);
         STRINGCASE(PAN_CONTRAST_LOW);
         STRINGCASE(PAN_CONTRAST_MEDIUM_LOW);
         STRINGCASE(PAN_CONTRAST_MEDIUM);
         STRINGCASE(PAN_CONTRAST_MEDIUM_HIGH);
         STRINGCASE(PAN_CONTRAST_HIGH);
         STRINGCASE(PAN_CONTRAST_VERY_HIGH);
         STRINGDEFAULT(_T("%d"), b);
        } /* Contrast */
    } // TreeDecode::cvtPANCO

/****************************************************************************
*                            TreeDecode::cvtPANFT
* Inputs:
*       BYTE b: Family type
* Result: CString
*       
****************************************************************************/

CString TreeDecode::cvtPANFT(BYTE b)
    {
     switch(b)
        { /* Family type */
         STRINGCASE(PAN_ANY);
         STRINGCASE(PAN_NO_FIT);
         STRINGCASE(PAN_FAMILY_TEXT_DISPLAY);
         STRINGCASE(PAN_FAMILY_SCRIPT);
         STRINGCASE(PAN_FAMILY_DECORATIVE);
         STRINGCASE(PAN_FAMILY_PICTORIAL);
         STRINGDEFAULT(_T("%d"), b);
        } /* Family type */
    } // TreeDecode::cvtPANFT
    
/****************************************************************************
*                            TreeDecode::cvtPANLF
* Inputs:
*       BYTE b: LetterForm
* Result: CString
*       
****************************************************************************/

CString TreeDecode::cvtPANLF(BYTE b)
    {
     switch(b)
        { /* LetterForm */
         STRINGCASE(PAN_ANY);
         STRINGCASE(PAN_NO_FIT);
         STRINGCASE(PAN_LETT_NORMAL_CONTACT);
         STRINGCASE(PAN_LETT_NORMAL_WEIGHTED);
         STRINGCASE(PAN_LETT_NORMAL_BOXED);
         STRINGCASE(PAN_LETT_NORMAL_FLATTENED);
         STRINGCASE(PAN_LETT_NORMAL_ROUNDED);
         STRINGCASE(PAN_LETT_NORMAL_OFF_CENTER);
         STRINGCASE(PAN_LETT_NORMAL_SQUARE);
         STRINGCASE(PAN_LETT_OBLIQUE_CONTACT);
         STRINGCASE(PAN_LETT_OBLIQUE_WEIGHTED);
         STRINGCASE(PAN_LETT_OBLIQUE_BOXED);
         STRINGCASE(PAN_LETT_OBLIQUE_FLATTENED);
         STRINGCASE(PAN_LETT_OBLIQUE_ROUNDED);
         STRINGCASE(PAN_LETT_OBLIQUE_OFF_CENTER);
         STRINGCASE(PAN_LETT_OBLIQUE_SQUARE);
         STRINGDEFAULT(_T("%d"), b);
        } /* LetterForm */
    } // TreeDecode::cvtPANLF

/****************************************************************************
*                            TreeDecode::cvtPANML
* Inputs:
*       BYTE b: Midline value
* Result: CString
*       
****************************************************************************/

CString TreeDecode::cvtPANML(BYTE b)
    {
     switch(b)
        { /* Midline */
         STRINGCASE(PAN_ANY);
         STRINGCASE(PAN_NO_FIT);
         STRINGCASE(PAN_MIDLINE_STANDARD_TRIMMED);
         STRINGCASE(PAN_MIDLINE_STANDARD_POINTED);
         STRINGCASE(PAN_MIDLINE_STANDARD_SERIFED);
         STRINGCASE(PAN_MIDLINE_HIGH_TRIMMED);
         STRINGCASE(PAN_MIDLINE_HIGH_POINTED);
         STRINGCASE(PAN_MIDLINE_HIGH_SERIFED);
         STRINGCASE(PAN_MIDLINE_CONSTANT_TRIMMED);
         STRINGCASE(PAN_MIDLINE_CONSTANT_POINTED);
         STRINGCASE(PAN_MIDLINE_CONSTANT_SERIFED);
         STRINGCASE(PAN_MIDLINE_LOW_TRIMMED);
         STRINGCASE(PAN_MIDLINE_LOW_POINTED);
         STRINGCASE(PAN_MIDLINE_LOW_SERIFED);
         STRINGDEFAULT(_T("%d"), b);
        } /* Midline */
    } // TreeDecode::cvtPANML

/****************************************************************************
*                            TreeDecode::cvtPANPR
* Inputs:
*       BYTE b: Proportion
* Result: CString
*       
****************************************************************************/

CString TreeDecode::cvtPANPR(BYTE b)
    {
     switch(b)
        { /* Proportion */
         STRINGCASE(PAN_ANY);
         STRINGCASE(PAN_NO_FIT);
         STRINGCASE(PAN_PROP_OLD_STYLE);
         STRINGCASE(PAN_PROP_MODERN);
         STRINGCASE(PAN_PROP_EVEN_WIDTH);
         STRINGCASE(PAN_PROP_EXPANDED);
         STRINGCASE(PAN_PROP_CONDENSED);
         STRINGCASE(PAN_PROP_VERY_EXPANDED);
         STRINGCASE(PAN_PROP_VERY_CONDENSED);
         STRINGCASE(PAN_PROP_MONOSPACED);
         STRINGDEFAULT(_T("%d"), b);
        } /* Proportion */
    } // TreeDecode::cvtPANPR

/****************************************************************************
*                            TreeDecode::cvtPANSS
* Inputs:
*       BYTE b: Serif style
* Result: CString
*       
****************************************************************************/

CString TreeDecode::cvtPANSS(BYTE b)
    {
     switch(b)
        { /* Serif style */
         STRINGCASE(PAN_ANY);
         STRINGCASE(PAN_NO_FIT);
         STRINGCASE(PAN_SERIF_COVE);
         STRINGCASE(PAN_SERIF_OBTUSE_COVE);
         STRINGCASE(PAN_SERIF_SQUARE_COVE);
         STRINGCASE(PAN_SERIF_SQUARE);
         STRINGCASE(PAN_SERIF_THIN);
         STRINGCASE(PAN_SERIF_BONE);
         STRINGCASE(PAN_SERIF_EXAGGERATED);
         STRINGCASE(PAN_SERIF_TRIANGLE);
         STRINGCASE(PAN_SERIF_NORMAL_SANS);
         STRINGCASE(PAN_SERIF_OBTUSE_SANS);
         STRINGCASE(PAN_SERIF_PERP_SANS);
         STRINGCASE(PAN_SERIF_FLARED);
         STRINGCASE(PAN_SERIF_ROUNDED);
         STRINGDEFAULT(_T("%d"), b);
        } /* Serif style */
    } // TreeDecode::cvtPANSS

/****************************************************************************
*                            TreeDecode::cvtPANSV
* Inputs:
*       BYTE b: Stroke variation
* Result: CString
*       
****************************************************************************/

CString TreeDecode::cvtPANSV(BYTE b)
    {
     switch(b)
        { /* stroke variation */
         STRINGCASE(PAN_ANY);
         STRINGCASE(PAN_NO_FIT);
         STRINGCASE(PAN_STROKE_GRADUAL_DIAG);
         STRINGCASE(PAN_STROKE_GRADUAL_TRAN);
         STRINGCASE(PAN_STROKE_GRADUAL_VERT);
         STRINGCASE(PAN_STROKE_GRADUAL_HORZ);
         STRINGCASE(PAN_STROKE_RAPID_VERT);
         STRINGCASE(PAN_STROKE_RAPID_HORZ);
         STRINGCASE(PAN_STROKE_INSTANT_VERT);
         STRINGDEFAULT(_T("%d"), b);
        } /* stroke variation */
    } // TreeDecode::cvtPANSV

/****************************************************************************
*                            TreeDecode::cvtPANWT
* Inputs:
*       BYTE b: Weight
* Result: CString
*       
****************************************************************************/

CString TreeDecode::cvtPANWT(BYTE b)
    {
     switch(b)
        { /* weight */
         STRINGCASE(PAN_ANY);
         STRINGCASE(PAN_NO_FIT);
         STRINGCASE(PAN_WEIGHT_VERY_LIGHT);
         STRINGCASE(PAN_WEIGHT_LIGHT);
         STRINGCASE(PAN_WEIGHT_THIN);
         STRINGCASE(PAN_WEIGHT_BOOK);
         STRINGCASE(PAN_WEIGHT_MEDIUM);
         STRINGCASE(PAN_WEIGHT_DEMI);
         STRINGCASE(PAN_WEIGHT_BOLD);
         STRINGCASE(PAN_WEIGHT_HEAVY);
         STRINGCASE(PAN_WEIGHT_BLACK);
         STRINGCASE(PAN_WEIGHT_NORD);
         STRINGDEFAULT(_T("%d"), b);
        } /* weight */
    } // TreeDecode::cvtPANWT
    
/****************************************************************************
*                            TreeDecode::cvtPANXH
* Inputs:
*       BYTE b:
* Result: CString
*       
****************************************************************************/

CString TreeDecode::cvtPANXH(BYTE b)
    {
     switch(b)
        { /* XHeight */
         STRINGCASE(PAN_ANY);
         STRINGCASE(PAN_NO_FIT);
         STRINGCASE(PAN_XHEIGHT_CONSTANT_SMALL);
         STRINGCASE(PAN_XHEIGHT_CONSTANT_STD);
         STRINGCASE(PAN_XHEIGHT_CONSTANT_LARGE);
         STRINGCASE(PAN_XHEIGHT_DUCKING_SMALL);
         STRINGCASE(PAN_XHEIGHT_DUCKING_STD);
         STRINGCASE(PAN_XHEIGHT_DUCKING_LARGE);
         STRINGDEFAULT(_T("%d"), b);
        } /* XHeight */
    } // TreeDecode::cvtPANXH

/****************************************************************************
*                             TreeDecode::cvtPEF
* Inputs:
*       BYTE flags
* Result: CString
*       
****************************************************************************/

CString TreeDecode::cvtPEF(BYTE flags)
    {
     switch(flags)
        { /* flags */
         STRINGCASE(PC_EXPLICIT);
         STRINGCASE(PC_NOCOLLAPSE);
         STRINGCASE(PC_RESERVED);
         STRINGDEFAULT(_T("%d"), flags);
        } /* flags */
    } // TreeDecode::cvtPEF

/****************************************************************************
*                             TreeDecode::cvtPFF
* Inputs:
*       DWORD dwFlags:
* Result: CString
*       
****************************************************************************/

CString TreeDecode::cvtPFF(DWORD dwFlags)
    {
     FLAG0(dwFlags);

     CString s;
     CString sep;
     AddFlag(s, sep, dwFlags, PFD_DRAW_TO_WINDOW);
     AddFlag(s, sep, dwFlags, PFD_DRAW_TO_BITMAP);
     AddFlag(s, sep, dwFlags, PFD_SUPPORT_GDI);
     AddFlag(s, sep, dwFlags, PFD_SUPPORT_OPENGL);
     AddFlag(s, sep, dwFlags, PFD_GENERIC_ACCELERATED);
     AddFlag(s, sep, dwFlags, PFD_GENERIC_FORMAT);
     AddFlag(s, sep, dwFlags, PFD_NEED_PALETTE);
     AddFlag(s, sep, dwFlags, PFD_NEED_SYSTEM_PALETTE);
     AddFlag(s, sep, dwFlags, PFD_DOUBLEBUFFER);
     AddFlag(s, sep, dwFlags, PFD_STEREO);
     AddFlag(s, sep, dwFlags, PFD_SWAP_LAYER_BUFFERS);
     AddFlag(s, sep, dwFlags, PFD_DEPTH_DONTCARE);
     AddFlag(s, sep, dwFlags, PFD_DOUBLEBUFFER_DONTCARE);
     AddFlag(s, sep, dwFlags, PFD_STEREO_DONTCARE);
     AddFlag(s, sep, dwFlags, PFD_SWAP_COPY);
     AddFlag(s, sep, dwFlags, PFD_SWAP_EXCHANGE);
     AddResiduum(s, sep, dwFlags);
     return s;
    } // TreeDecode::cvtPFF

/****************************************************************************
*                             TreeDecode::cvtPFM
* Inputs:
*       DWORD iMode
* Result: CString
*       
****************************************************************************/

CString TreeDecode::cvtPFM(DWORD iMode)
    {
     switch(iMode)
        { /* mode */
         STRINGCASE(ALTERNATE);
         STRINGCASE(WINDING);
         STRINGDEFAULT(_T("%d"), iMode);
        } /* mode */
    } // TreeDecode::cvtPFM

/****************************************************************************
*                              TreeDecode::cvtPS
* Inputs:
*       UINT ps: Pen style
* Result: CString
*       Printable pen style
****************************************************************************/

CString TreeDecode::cvtPS(UINT ps)
    {
     switch(ps)
        { /* ps */
         STRINGCASE(PS_SOLID);
         STRINGCASE(PS_DASH);
         STRINGCASE(PS_DOT);
         STRINGCASE(PS_DASHDOT);
         STRINGCASE(PS_DASHDOTDOT);
         STRINGCASE(PS_NULL);
         STRINGCASE(PS_INSIDEFRAME);
         STRINGCASE(PS_USERSTYLE);
         STRINGDEFAULT(_T("%d"), ps);
        } /* ps */
    } // TreeDecode::cvtPS

/****************************************************************************
*                              TreeDecode::cvtPT
* Inputs:
*       BYTE pt: Pixel type
* Result: CString
*       
****************************************************************************/

CString TreeDecode::cvtPT(BYTE pt)
    {
     switch(pt)
        { /* pt */
         STRINGCASE(PFD_TYPE_RGBA);
         STRINGCASE(PFD_TYPE_COLORINDEX);
         STRINGDEFAULT(_T("%d"), pt);
        } /* pt */
    } // TreeDecode::cvtPT

/****************************************************************************
*                            TreeDecode::cvtRCmm
* Inputs:
*       RECTL & r: Rectangle
* Result: CString
*       Decoded rectangle for printing, in 0.01mm units
* Effect: 
*       Decodes the rectangle
****************************************************************************/

CString TreeDecode::cvtRCmm(RECTL r)
    {
     CString s;
#define MM01(x) ((x) / 100), ((x) % 100)
     s.Format(_T("{%d.%02dmm, %d.%02dmm, %d.%02dmm, %d.%02dmm}"), 
                MM01(r.left), MM01(r.top), MM01(r.right), MM01(r.bottom));
#undef MM01
     return s;
    } // TreeDecode::cvt(RECTL)

/****************************************************************************
*                             TreeDecode::cvtRDHT
* Inputs:
*       DWORD iType
* Result: CString
*       
****************************************************************************/

CString TreeDecode::cvtRDHT(DWORD iType)
    {
     switch(iType)
        { /* iType */
         STRINGCASE(RDH_RECTANGLES);
         STRINGDEFAULT(_T("%d"), iType);
        } /* iType */
    } // TreeDecode::cvtRDHT

/****************************************************************************
*                             TreeDecode::cvtRGN
* Inputs:
*       DWORD iMode:
* Result: CString
*       
****************************************************************************/

CString TreeDecode::cvtRGN(DWORD iMode)
    {
     switch(iMode)
        { /* iMode */
         STRINGCASE(RGN_AND);
         STRINGCASE(RGN_COPY);
         STRINGCASE(RGN_DIFF);
         STRINGCASE(RGN_OR);
         STRINGCASE(RGN_XOR);
         STRINGDEFAULT(_T("%d"), iMode);
        } /* iMode */
    } // TreeDecode::cvtRGN

/****************************************************************************
*                             TreeDecode::cvtROP2
* Inputs:
*       DWORD rop:
* Result: CString
*       The decoded ROP2
****************************************************************************/

CString TreeDecode::cvtROP2(DWORD rop)
    {
     switch(rop)
        { /* rop */
         STRINGCASE(R2_BLACK);
         STRINGCASE(R2_NOTMERGEPEN);
         STRINGCASE(R2_MASKNOTPEN);
         STRINGCASE(R2_NOTCOPYPEN);
         STRINGCASE(R2_MASKPENNOT);
         STRINGCASE(R2_NOT);
         STRINGCASE(R2_XORPEN);
         STRINGCASE(R2_NOTMASKPEN);
         STRINGCASE(R2_MASKPEN);
         STRINGCASE(R2_NOTXORPEN);
         STRINGCASE(R2_NOP);
         STRINGCASE(R2_MERGENOTPEN);
         STRINGCASE(R2_COPYPEN);
         STRINGCASE(R2_MERGEPENNOT);
         STRINGCASE(R2_MERGEPEN);
         STRINGCASE(R2_WHITE);
         STRINGDEFAULT(_T("0x%08x"), rop);
        } /* rop */
    } // TreeDecode::cvtROP2

/****************************************************************************
*                             TreeDecode::cvtROP
* Inputs:
*       DWORD rop:
* Result: CString
*       The decoded ROP
****************************************************************************/

CString TreeDecode::cvtROP(DWORD rop)
    {
     switch(rop)
        { /* rop */
         STRINGCASE(BLACKNESS);
         STRINGCASE(CAPTUREBLT);
         STRINGCASE(DSTINVERT);
         STRINGCASE(MERGECOPY);
         STRINGCASE(MERGEPAINT);
         STRINGCASE(NOMIRRORBITMAP);
         STRINGCASE(NOTSRCCOPY);
         STRINGCASE(NOTSRCERASE);
         STRINGCASE(PATCOPY);
         STRINGCASE(PATINVERT);
         STRINGCASE(PATPAINT);
         STRINGCASE(SRCAND);
         STRINGCASE(SRCCOPY);
         STRINGCASE(SRCERASE);
         STRINGCASE(SRCINVERT);
         STRINGCASE(SRCPAINT);
         STRINGCASE(WHITENESS);
         STRINGDEFAULT(_T("0x%08x"), rop);
        } /* rop */
    } // TreeDecode::cvtROP

/****************************************************************************
*                             TreeDecode::cvtSBM
* Inputs:
*       DWORD iMode:
* Result: CString
*       
****************************************************************************/

CString TreeDecode::cvtSBM(DWORD iMode)
    {
     switch(iMode)
        { /* iMode */
         STRINGCASE(STRETCH_ANDSCANS);
         STRINGCASE(STRETCH_DELETESCANS);
         STRINGCASE(STRETCH_HALFTONE);
         STRINGCASE(STRETCH_ORSCANS);
         STRINGDEFAULT(_T("0x%x"), iMode);
        } /* iMode */
    } // TreeDecode::cvtSBM

/****************************************************************************
*                             TreeDecode::cvtSig
* Inputs:
*       DWORD sig: Signature
* Result: CString
*       Formatted representation of signature
* Notes:
*       A signature is a 4-character sequence of 8-bit characters encoded
*       as a DWORD
*       For EMF files it is
*       0x464d4520
*       which is
*       ' EMF'
****************************************************************************/

CString TreeDecode::cvtSig(DWORD sig)
    {
     union {
        DWORD D;
        char ch[4];  // not TCHAR!
     } data;
     data.D = sig;
     CString s = CString(data.ch, 4);
     s = _T("'") + s + _T("'");
     return s;
    } // TreeDecode::cvtSig

/****************************************************************************
*                              TreeDecode::cvtSO
* Inputs:
*       DWORD so: Stock object
* Result: CString
*       
****************************************************************************/

CString TreeDecode::cvtSO(DWORD so)
    {
     switch(so)
        { /* so */
         STRINGCASE(WHITE_BRUSH);
         STRINGCASE(LTGRAY_BRUSH);
         STRINGCASE(GRAY_BRUSH);
         STRINGCASE(DKGRAY_BRUSH);
         STRINGCASE(BLACK_BRUSH);
         STRINGCASE(HOLLOW_BRUSH);
         STRINGCASE(WHITE_PEN);
         STRINGCASE(BLACK_PEN);
         STRINGCASE(NULL_PEN);
         STRINGCASE(OEM_FIXED_FONT);
         STRINGCASE(ANSI_FIXED_FONT);
         STRINGCASE(ANSI_VAR_FONT);
         STRINGCASE(SYSTEM_FONT);
         STRINGCASE(DEVICE_DEFAULT_FONT);
         STRINGCASE(DEFAULT_PALETTE);
         STRINGCASE(SYSTEM_FIXED_FONT);
         STRINGCASE(DEFAULT_GUI_FONT);
         STRINGCASE(DC_BRUSH);
         STRINGCASE(DC_PEN);
         STRINGDEFAULT(_T("%d"), so);
        } /* so */
    } // TreeDecode::cvtSO

/****************************************************************************
*                            TreeDecode::cvtString
* Inputs:
*       LPCWSTR s: String to convert
* Result: CString
*       
****************************************************************************/

CString TreeDecode::cvtString(LPCWSTR s)
    {
     CStringW result = s;
     result.Replace(L"\\", L"\\\\");
     result.Replace(L"\"", L"\\");
     result.Replace(L"\a", L"\\a");
     result.Replace(L"\b", L"\\b");
     result.Replace(L"\f", L"\\f");
     result.Replace(L"\r", L"\\r");
     result.Replace(L"\n", L"\\n");
     result.Replace(L"\t", L"\\t");
     result.Replace(L"\v", L"\\v");
     result = L"\"" + result + L"\"";
     return CString(result); // just in case this is an ANSI app...
    } // TreeDecode::cvtString

/****************************************************************************
*                            TreeDecode::cvtString
* Inputs:
*       LPCSTR s: 8-bit character string to decode
* Result: CString
*       The converted characters in CString format (CStringA or CStringT)
****************************************************************************/

CString TreeDecode::cvtString(LPCSTR s)
    {
     CStringW w = CStringW(s);
     return cvtString(w);
    } // TreeDecode::cvtString

/****************************************************************************
*                              TreeDecode::cvtTA
* Inputs:
*       DWORD iMode:
* Result: CString
*       
* Effect: 
*       Decodes Text Align modes
****************************************************************************/

CString TreeDecode::cvtTA(DWORD iMode)
    {
     CString sep;
     CString s;
     CString s1 = cvtTA1(iMode & (TA_BASELINE | TA_BOTTOM | TA_TOP | VTA_BASELINE | VTA_CENTER));
     AddVal(s, sep, s1);

     CString s2 = cvtTA2(iMode & (TA_CENTER | TA_LEFT | TA_RIGHT));
     AddVal(s, sep, s2);
     
     CString s3 = cvtTA3(iMode & (TA_NOUPDATECP | TA_UPDATECP));
     AddVal(s, sep, s3);

     CString s4 = cvtTA4(iMode & TA_RTLREADING);
     AddVal(s, sep, s4);

     return s;
    } // TreeDecode::cvtTA

/****************************************************************************
*                             TreeDecode::cvtTA1
* Inputs:
*       DWORD iMode: the vertical TA mode bits, all others masked out
* Result: CString
*       
****************************************************************************/

CString TreeDecode::cvtTA1(DWORD iMode)
    {
     switch(iMode)
        { /* iMode */
         STRINGCASE(TA_BASELINE);
         STRINGCASE(TA_BOTTOM);
         STRINGCASE(TA_TOP);
         //STRINGCASE(VTA_BASELINE);
         STRINGCASE(VTA_CENTER);
         STRINGDEFAULT(_T("%d"), iMode);
        } /* iMode */
    } // TreeDecode::cvtTA1

/****************************************************************************
*                             TreeDecode::cvtTA2
* Inputs:
*       DWORD iMode: The horizontal TA modes, all other bits stripped out
* Result: CString
*       
****************************************************************************/

CString TreeDecode::cvtTA2(DWORD iMode)
    {
     switch(iMode)
        { /* iMode */
         STRINGCASE(TA_CENTER);
         STRINGCASE(TA_LEFT);
         STRINGCASE(TA_RIGHT);
         STRINGDEFAULT(_T("%d"), iMode);
        } /* iMode */
    } // TreeDecode::cvtTA2

/****************************************************************************
*                             TreeDecode::cvtTA3
* Inputs:
*       DWORD iMode: the style mode with all but TA_UPDATECP/TA_NOUPDATECP stripped
* Result: CString
*       
****************************************************************************/

CString TreeDecode::cvtTA3(DWORD iMode)
    {
     switch(iMode)
        { /* iMode */
         STRINGCASE(TA_NOUPDATECP);
         STRINGCASE(TA_UPDATECP);
         STRINGDEFAULT(_T("%d"), iMode);
        } /* iMode */
    } // TreeDecode::cvtTA3

/****************************************************************************
*                             TreeDecode::cvtTA4
* Inputs:
*       DWORD iMode: The flags with all up TA_RTLREADING stripped
* Result: CString
*       TA_RTLREADING or empty
****************************************************************************/

CString TreeDecode::cvtTA4(DWORD iMode)
    {
     switch(iMode)
        { /* iMode */
         STRINGCASE(TA_RTLREADING);
         STRINGDEFAULT(_T(""), 0);
        } /* iMode */
    } // TreeDecode::cvtTA4

/****************************************************************************
*                         TreeDecode::cvtTextOptions
* Inputs:
*       DWORD
* Result: CString
*       
****************************************************************************/

CString TreeDecode::cvtTextOptions(DWORD options)
    {
     FLAG0(options);

     CString s;
     CString sep;
     AddFlag(s, sep, options, ETO_CLIPPED);
     AddFlag(s, sep, options, ETO_OPAQUE);
     AddResiduum(s, sep, options);
     return s;
    } // TreeDecode::cvtTextOptions

/****************************************************************************
*                            TreeDecode::cvt(BYTE)
* Inputs:
*       BYTE v
* Result: CString
*       
****************************************************************************/

CString TreeDecode::cvt(BYTE v)
    {
     CString s;
     s.Format(_T("%u"), v);
     return s;
    } // TreeDecode::cvt(BYTE)

/****************************************************************************
*                            TreeDecode::cvt(DWORD)
* Inputs:
*       DWORD v
* Result: CString
*       
****************************************************************************/

CString TreeDecode::cvt(DWORD v)
    {
     CString s;
     s.Format(_T("%u"), v);
     return s;
    } // TreeDecode::cvt(DWORD)

/****************************************************************************
*                           TreeDecode::cvt(FLOAT)
* Inputs:
*       FLOAT f: Value to format
* Result: CString
*       
****************************************************************************/

CString TreeDecode::cvt(FLOAT f)
    {
     CString s;
     s.Format(_T("%g"), f);
     return s;
    } // TreeDecode::cvt(FLOAT)

/****************************************************************************
*                            TreeDecode::cvt(LONG)
* Inputs:
*       LONG v
* Result: CString
*       
****************************************************************************/

CString TreeDecode::cvt(LONG v)
    {
     CString s;
     s.Format(_T("%d"), v);
     return s;
    } // TreeDecode::cvt(LONG)

/****************************************************************************
*                            TreeDecode::cvt(POINT)
* Inputs:
*       POINT r: Point
* Result: CString
*       Decoded point for printing
* Effect: 
*       Decodes the point
****************************************************************************/

CString TreeDecode::cvt(POINT pt)
   {
    CString s;
    s.Format(_T("{%d, %d}"), pt.x, pt.y);
    return s;
   } // TreeDecode::cvt(POINT)

/****************************************************************************
*                            TreeDecode::cvt(POINTL)
* Inputs:
*       POINTL r: Point
* Result: CString
*       Decoded point for printing
* Effect: 
*       Decodes the point
****************************************************************************/

CString TreeDecode::cvt(POINTL pt)
   {
    CString s;
    s.Format(_T("{%d, %d}"), pt.x, pt.y);
    return s;
   } // TreeDecode::cvt(POINTL)

/****************************************************************************
*                            TreeDecode::cvt(RECTL)
* Inputs:
*       RECTL & r: Rectangle
* Result: CString
*       Decoded rectangle for printing
* Effect: 
*       Decodes the rectangle
****************************************************************************/

CString TreeDecode::cvt(RECTL r)
    {
     CString s;
     s.Format(_T("{%d, %d, %d, %d}"), r.left, r.top, r.right, r.bottom);
     return s;
    } // TreeDecode::cvt(RECTL)

/****************************************************************************
*                            TreeDecode::cvt(RECT)
* Inputs:
*       RECT & r: Rectangle
* Result: CString
*       Decoded rectangle for printing
* Effect: 
*       Decodes the rectangle
****************************************************************************/

CString TreeDecode::cvt(RECT r)
    {
     CString s;
     s.Format(_T("{%d, %d, %d, %d}"), r.left, r.top, r.right, r.bottom);
     return s;
    } // TreeDecode::cvt(RECT)

/****************************************************************************
*                           TreeDecode::cvt(SHORT)
* Inputs:
*       SHORT v:
* Result: CString
*       The value formatted as a decimal number
****************************************************************************/

CString TreeDecode::cvt(SHORT v)
    {
     CString s;
     s.Format(_T("%d"), v);
     return s;
    } // TreeDecode::cvt(SHORT)

/****************************************************************************
*                            TreeDecode::cvt(SIZEL)
* Inputs:
*       SIZEL r: size
* Result: CString
*       Decoded point for printing
* Effect: 
*       Decodes the point
****************************************************************************/

CString TreeDecode::cvt(SIZEL sz)
   {
    CString s;
    s.Format(_T("{%d, %d}"), sz.cx, sz.cy);
    return s;
   } // TreeDecode::cvt(SIZEL)

/****************************************************************************
*                            TreeDecode::cvt(UINT)
* Inputs:
*       LONG v
* Result: CString
*       
****************************************************************************/

CString TreeDecode::cvt(UINT v)
    {
     CString s;
     s.Format(_T("%u"), v);
     return s;
    } // TreeDecode::cvt(UINT)


/****************************************************************************
*                            TreeDecode::cvt(WORD)
* Inputs:
*       WORD v
* Result: CString
*       
****************************************************************************/

CString TreeDecode::cvt(WORD v)
    {
     CString s;
     s.Format(_T("%u"), v);
     return s;
    } // TreeDecode::cvt(WORD)

/****************************************************************************
*                            TreeDecode::cvt(XFORM)
* Inputs:
*       XFORM X:
* Result: CString
*       Decoded XFORM for printing
****************************************************************************/

CString TreeDecode::cvt(XFORM X)
   {
    CString s;
    s.Format(_T("{%g, %g, %g, %g, %g, %g}"), X.eM11, X.eM12, X.eM21, X.eM22, X.eDx, X.eDy);
    return s;
   } // TreeDecode::cvt(POINTL)

/****************************************************************************
*                        TreeDecode::FormatBITMAPINFO
* Inputs:
*       BITMAPINFO * bmi:
*       LPVOID p1:
*       LPVLID p2:
* Result: void
*       
* Effect: 
*       Formats a BITMAPINFO header that is in the stream
* Notes:
*    typedef struct {
*       BITMAPINFOHEADER bmiHeader;
*       RGBQUAD bmiColors[1];
*    } BITMAPINFO;
****************************************************************************/

void TreeDecode::FormatBITMAPINFO(const BITMAPINFO * bmi, LPVOID p1, LPVOID p2)
    {
     GET_CTL_ITEM();

     HTREEITEM hdr = ctl->InsertItem(_T("BITMAPINFOHEADER bmiHeader"), item);
     FormatBITMAPINFOHEADER(&bmi->bmiHeader, p1, (LPVOID)hdr);

     HTREEITEM rgbq = ctl->InsertItem(_T("RGBQUAD bmiColors[1]"), item);
    } // TreeDecode::FormatBITMAPINFO

/****************************************************************************
*                     TreeDecode::FormatBITMAPINFOHEADER
* Inputs:
*       BITMAPINFOHEADER * hdr"
*       LPVOID p1:
*       LPVOID p2:
* Result: void
*       
* Effect: 
*       Formats a BITMAPINFOHEADER
* Notes:
*    typedef struct {
*      DWORD  biSize; 
*      LONG   biWidth; 
*      LONG   biHeight; 
*      WORD   biPlanes; 
*      WORD   biBitCount; 
*      DWORD  biCompression; 
*      DWORD  biSizeImage; 
*      LONG   biXPelsPerMeter; 
*      LONG   biYPelsPerMeter; 
*      DWORD  biClrUsed; 
*      DWORD  biClrImportant; 
*    } BITMAPINFOHEADER
****************************************************************************/

void TreeDecode::FormatBITMAPINFOHEADER(const BITMAPINFOHEADER * hdr, LPVOID p1, LPVOID p2)
    {
     GET_CTL_ITEM();

     AddFormattedItem(ctl, item, BITMAPINFOHEADER, hdr, DWORD, biSize, cvt);
     AddFormattedItem(ctl, item, BITMAPINFOHEADER, hdr, LONG, biWidth, cvt);
     AddFormattedItem(ctl, item, BITMAPINFOHEADER, hdr, LONG, biHeight, cvt);
     AddFormattedItem(ctl, item, BITMAPINFOHEADER, hdr, WORD, biPlanes, cvt);
     AddFormattedItem(ctl, item, BITMAPINFOHEADER, hdr, WORD, biBitCount, cvt);
     AddFormattedItem(ctl, item, BITMAPINFOHEADER, hdr, DWORD, biCompression, cvtBIC);
     AddFormattedItem(ctl, item, BITMAPINFOHEADER, hdr, DWORD, biSizeImage, cvt);
     AddFormattedItem(ctl, item, BITMAPINFOHEADER, hdr, LONG, biXPelsPerMeter, cvt);
     AddFormattedItem(ctl, item, BITMAPINFOHEADER, hdr, LONG, biYPelsPerMeter, cvt);
     AddFormattedItem(ctl, item, BITMAPINFOHEADER, hdr, DWORD, biClrUsed, cvt);
     AddFormattedItem(ctl, item, BITMAPINFOHEADER, hdr, DWORD, biClrImportant, cvt);
    } // TreeDecode::FormatBITMAPINFOHEADER

/****************************************************************************
*                          TreeDecode::FormatCIEXYZ
* Inputs:
*       CIEXYZ * cie
* Result: void
*       
* Effect: 
*       Formats a CIEXYZ
* Notes:
*    typedef struct {
*      FXPT2DOT30 ciexyzX; 
*      FXPT2DOT30 ciexyzY; 
*      FXPT2DOT30 ciexyzZ; 
*    } CIEXYZ;
****************************************************************************/

void TreeDecode::FormatCIEXYZ(const CIEXYZ * cie, LPVOID p1, LPVOID p2)
    {
     GET_CTL_ITEM();

     AddFormattedItem(ctl, item, CIEXYZ, cie, FXPT2DOT30, ciexyzX, cvtFXPT230);
     AddFormattedItem(ctl, item, CIEXYZ, cie, FXPT2DOT30, ciexyzY, cvtFXPT230);
     AddFormattedItem(ctl, item, CIEXYZ, cie, FXPT2DOT30, ciexyzZ, cvtFXPT230);
    } // TreeDecode::FormatCIEXYZ

/****************************************************************************
*                       TreeDecode::FormatCIEXYZTRIPLE
* Inputs:
*       CIEXYZTRIPLE * triple:
*       LPVOID p1:
*       LPVOID p2:
* Result: void
*       
* Effect: 
*       Formats a CIEXYZTRIPE
* Notes:
*    typedef struct tagCIEXYZTRIPLE {
*      CIEXYZ  ciexyzRed; 
*      CIEXYZ  ciexyzGreen; 
*      CIEXYZ  ciexyzBlue; 
*    } CIEXYZTRIPLE;
****************************************************************************/

void TreeDecode::FormatCIEXYZTRIPLE(const CIEXYZTRIPLE * triple, LPVOID p1, LPVOID p2)
    {
     GET_CTL_ITEM();
     
     HTREEITEM tr;
     tr = ctl->InsertItem(_T("CIEXYZ ciexyzRed"));
     FormatCIEXYZ(&triple->ciexyzRed, p1, (LPVOID)tr);
     tr = ctl->InsertItem(_T("CIEXYZ ciexyzGreen"));
     FormatCIEXYZ(&triple->ciexyzGreen, p1, (LPVOID)tr);
     tr = ctl->InsertItem(_T("CIEXYZ ciexyzBlue"));
     FormatCIEXYZ(&triple->ciexyzBlue, p1, (LPVOID)tr);
    } // TreeDecode::FormatCIEXYZTRIPLE

/****************************************************************************
*                      TreeDecode::FormatCOLORADJUSTMENT
* Inputs:
*       const COLORADJUSTMENT * ca:
*       LPVOID p1:
*       LPVOID p2:
* Result: void
*       
* Effect: 
*       FOrmats a COLORADJUSTMENT
* Notes:
*    typedef struct {
*       WORD  caSize;
*       WORD  caFlags;
*       WORD  caIlluminantIndex;
*       WORD  caRedGamma;
*       WORD  caGreenGamma;
*       WORD  caBlueGamma;
*       WORD  caReferenceBlack;
*       WORD  caReferenceWhite;
*       SHORT caContrast;
*       SHORT caBrightness;
*       SHORT caColorfulness;
*       SHORT caRedGreenTint;
*    } COLORADJUSTMENT;
*****************************************************************************/

void TreeDecode::FormatCOLORADJUSTMENT(const COLORADJUSTMENT * ca, LPVOID p1, LPVOID p2)
    {
     GET_CTL_ITEM();

     AddFormattedItem(ctl, item, COLORADJUSTMENT, ca, WORD, caSize, cvt);
     AddFormattedItem(ctl, item, COLORADJUSTMENT, ca, WORD, caFlags, cvtCAF);
     AddFormattedItem(ctl, item, COLORADJUSTMENT, ca, WORD, caIlluminantIndex, cvtII);
     AddFormattedItem(ctl, item, COLORADJUSTMENT, ca, WORD, caRedGamma, cvt);
     AddFormattedItem(ctl, item, COLORADJUSTMENT, ca, WORD, caGreenGamma, cvt);
     AddFormattedItem(ctl, item, COLORADJUSTMENT, ca, WORD, caBlueGamma, cvt);
     AddFormattedItem(ctl, item, COLORADJUSTMENT, ca, WORD, caReferenceBlack, cvt);
     AddFormattedItem(ctl, item, COLORADJUSTMENT, ca, WORD, caReferenceWhite, cvt);
     AddFormattedItem(ctl, item, COLORADJUSTMENT, ca, SHORT, caContrast, cvt);
     AddFormattedItem(ctl, item, COLORADJUSTMENT, ca, SHORT, caBrightness, cvt);
     AddFormattedItem(ctl, item, COLORADJUSTMENT, ca, SHORT, caColorfulness, cvt);
     AddFormattedItem(ctl, item, COLORADJUSTMENT, ca, SHORT, caRedGreenTint, cvt);

    } // TreeDecode::FormatCOLORADJUSTMENT

/****************************************************************************
*                         TreeDecode::FormatDrawTypes
* Inputs:
*       LPBYTE data: Data array
*       LPVOID p1
*       LPVOID p2:
*       DWORD count:
* Result: void
*       
* Effect: 
*       Formats the types array of EMRPOLYDRAW/EMRPOLYDRAW16
****************************************************************************/

void TreeDecode::FormatDrawTypes(LPBYTE data, LPVOID p1, LPVOID p2, DWORD count)
    {
     GET_CTL_ITEM();

     for(DWORD i = 0; i < count; i++)
        { /* format each */
         CString cf;
         if(data[i] & PT_CLOSEFIGURE)
            cf = _T(" | PT_CLOSEFIGURE");
         CString op = cvtDT(data[i] & ~PT_CLOSEFIGURE);
         CString s;
         s.Format(_T("[%d] %s%s"), i, op, cf);
         ctl->InsertItem(s, item);
        } /* format each */
     
    } // TreeDecode::FormatDrawTypes

/****************************************************************************
*                          TreeDecode::FormatDWORDs
* Inputs:
*       const LPDWORD data
*       LPVOID p1:
*       LPVOID p2:
*       DWORD count:
* Result: void
*       
* Effect: 
*       Formats an array of DWORDs
****************************************************************************/

void TreeDecode::FormatDWORDs(const DWORD * data, LPVOID p1, LPVOID p2, DWORD count)
    {
     GET_CTL_ITEM();

     for(DWORD i = 0; i < count; i++)
        { /* loop */
         CString s;
         s.Format(_T("[%d] %d (0x%x)"), i, data[i], data[i]);
         ctl->InsertItem(s, item);
        } /* loop */
    } // TreeDecode::FormatDWORDs

/****************************************************************************
*                         TreeDecode::FormatEMRText1
* Inputs:
*       const EMRTEXT * text:
*       LPVOID p1:
*       LPVOID P2:
* Result: void
*       
* Effect: 
*       Formats the first part (string-format-independent part) of EMRTEXT
****************************************************************************/

void TreeDecode::FormatEMRText1(const EMRTEXT * text, LPVOID p1, LPVOID p2)
    {
     GET_CTL_ITEM();

     AddFormattedItem(ctl, item, EMRTEXT, text, POINTL, ptlReference, cvt);
     AddFormattedItem(ctl, item, EMRTEXT, text, DWORD, nChars, cvt);
     AddFormattedItem(ctl, item, EMRTEXT, text, DWORD, offString, cvt);
     AddFormattedItem(ctl, item, EMRTEXT, text, DWORD, fOptions, cvtTextOptions);
     AddFormattedItem(ctl, item, EMRTEXT, text, RECTL, rcl, cvt);
     AddFormattedItem(ctl, item, EMRTEXT, text, DWORD, offDx, cvt);
     
    } // TreeDecode::FormatEMRText1

/****************************************************************************
*                         TreeDecode::FormatEMRText2
* Inputs:
*       const DWORD * sp: Array of DWORDs
*       DWORD offset of array (for documentation)
*       DWORD len: Number of elements in array
*       LPVOID p1:
*       LPVOID p2:
* Result: void
*       
* Effect: 
*       Formats the intercharacter spacing array
****************************************************************************/

void TreeDecode::FormatEMRText2(const DWORD * sp, DWORD offset, DWORD len, LPVOID p1, LPVOID p2)
    {
     GET_CTL_ITEM();
     
     CString s;
     s.Format(_T("[%d] {"), offset);
     CString sep;

     for(DWORD i = 0; i < len; i++)
        { /* add spacing */
         CString t;
         t.Format(_T("%d"), sp[i]);
         s += sep + t;
         sep = _T(", ");
        } /* add spacing */
     s += _T("}");
     ctl->InsertItem(s, item);
    } // TreeDecode::FormatEMRText2

/****************************************************************************
*                          TreeDecode::FormatEMRTextA
* Inputs:
*       const EMRTEXT * text:
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEM) item to add to
*       LPCVOID p3: (LPVOID)(ENHMETARECORD *) base of offset
* Result: void
*       
* Effect: 
*       Formats the EMRTEXT for EMREXTTEXTOUTW
****************************************************************************/

void TreeDecode::FormatEMRTextA(const EMRTEXT * text, LPVOID p1, LPVOID p2, LPCVOID p3)
    {
     GET_CTL_ITEM()

     const LPBYTE base = (BYTE *)p3;
     FormatEMRText1(text, p1, p2);
     
     CStringW str = CStringW((LPCSTR)(base + text->offString), text->nChars);
     str = cvtString(str);
     CStringW s;
     s.Format(_T("[%d] \"%s\""), text->offString, str);
     
     ctl->InsertItem(s, item);

     if(text->offDx != 0)
        { /* handle spacing array */
         DWORD * sp = (DWORD *)(base + text->offDx);
         FormatEMRText2(sp, text->offDx, text->nChars - 1, p1, p2);
        } /* handle spacing array */
    } // TreeDecode::FormatEMRTextA

/****************************************************************************
*                         TreeDecode::FormatEMRTextAa
* Inputs:
*       const EMRTEXT * text: Pointer to an ARRAY of
*       LPVOID p1:
*       LPVOID p2:
*       LPCVOID p3: base address value
*       LONG count: number of elements in the array
* Result: void
*       
* Effect: 
*       Formats an array of EMRTEXT elements
****************************************************************************/

void TreeDecode::FormatEMRTextAa(const EMRTEXT * text, LPVOID p1, LPVOID p2, LPCVOID p3, LONG count)
    {
     GET_CTL_ITEM();

     for(LONG i = 0; i < count; i++)
        { /* each */
         CString s;
         s.Format(_T("[%d] EMRTEXT"), i);
         HTREEITEM n = ctl->InsertItem(s, item);
         FormatEMRTextA(&text[i], p1, (LPVOID)n, p3);
        } /* each */
    } // TreeDecode::FormatEMRTextAa

/****************************************************************************
*                          TreeDecode::FormatEMRTextW
* Inputs:
*       const EMRTEXT * text:
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEM) item to add to
*       LPCVOID p3: (LPVOID)(ENHMETARECORD *) base of offset
* Result: void
*       
* Effect: 
*       Formats the EMRTEXT for EMREXTTEXTOUTW
****************************************************************************/

void TreeDecode::FormatEMRTextW(const EMRTEXT * text, LPVOID p1, LPVOID p2, LPCVOID p3)
    {
     GET_CTL_ITEM()
     const LPBYTE base = (BYTE *)p3;
     
     FormatEMRText1(text, p1, p2);
     
     CStringW str = CStringW((LPWSTR)(base + text->offString), text->nChars);
     CStringW s;
     s.Format(L"[%d] L\"%ls\"", text->offString, str);
     ctl->InsertItem(s, item);

     DWORD * sp = (DWORD *)(base + text->offDx);
     FormatEMRText2(sp, text->offDx, text->nChars - 1, p1, p2);
    } // TreeDecode::FormatEMRTextW

/****************************************************************************
*                         TreeDecode::FormatEMRTextWa
* Inputs:
*       const EMRTEXT * text: Pointer to an ARRAY of
*       LPVOID p1:
*       LPVOID p2:
*       LPCVOID p3: base address value
*       LONG count: number of elements in the array
* Result: void
*       
* Effect: 
*       Formats an array of EMRTEXT elements
****************************************************************************/

void TreeDecode::FormatEMRTextWa(const EMRTEXT * text, LPVOID p1, LPVOID p2, LPCVOID p3, LONG count)
    {
     GET_CTL_ITEM();

     for(LONG i = 0; i < count; i++)
        { /* each */
         CString s;
         s.Format(_T("[%d] EMRTEXT"), i);
         HTREEITEM n = ctl->InsertItem(s, item);
         FormatEMRTextW(&text[i], p1, (LPVOID)n, p3);
        } /* each */
    } // TreeDecode::FormatEMRTextWa

/****************************************************************************
*                        TreeDecode::FormatEXTLOGFONTW
* Inputs:
*       const EXTLOGFONTW * lf
*       LPVOID p1:
*       LPVOID p2:
* Result: void
*       
* Effect: 
*       Formats an EXTLOGFONTW
****************************************************************************/

void TreeDecode::FormatEXTLOGFONTW(const EXTLOGFONTW * lf, LPVOID p1, LPVOID p2)
    {
     GET_CTL_ITEM();

     HTREEITEM lfitem = ctl->InsertItem(_T("LOGFONT elfLogFont"), item);
     FormatLOGFONTW(&lf->elfLogFont, p1, (LPVOID)lfitem);
     //             ctl, item, emrtype,    str, type,  field,       size
     AddStringItemW(ctl, item, EXTLOGFONTW, lf, TCHAR, elfFullName, LF_FULLFACESIZE);
     AddStringItemW(ctl, item, EXTLOGFONTW, lf, TCHAR, elfStyle, LF_FACESIZE);
     AddFormattedItem(ctl, item, EXTLOGFONTW, lf, TCHAR, elfVersion, cvt);          
     AddFormattedItem(ctl, item, EXTLOGFONTW, lf, TCHAR, elfStyleSize, cvt);          
     AddFormattedItem(ctl, item, EXTLOGFONTW, lf, TCHAR, elfMatch, cvt);          
     AddFormattedItem(ctl, item, EXTLOGFONTW, lf, TCHAR, elfReserved, cvt);          
     AddItem(ctl, item, EXTLOGFONTW, lf, TCHAR, elfVendorId, _T("0x%08x"));
     AddFormattedItem(ctl, item, EXTLOGFONTW, lf, TCHAR, elfCulture, cvt);
     HTREEITEM panose = ctl->InsertItem(_T("PANOSE elfPanose"), item);
     FormatPANOSE(&lf->elfPanose, p1, (LPVOID)panose);
    } // TreeDecode::FormatEXTLOGFONTW

/****************************************************************************
*                          TreeDecode::FormatEXTLOGPEN
* Inputs:
*       EXTLOGPEN * lpen:
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEM) item to add to
* Result: void
*       
* Effect: 
*       Decodes a LOGPEN
****************************************************************************/

void TreeDecode::FormatEXTLOGPEN(const EXTLOGPEN * lp, LPVOID p1, LPVOID p2)
    {
     CTreeCtrl * ctl = (CTreeCtrl *)p1;
     HTREEITEM item = (HTREEITEM)p2;

     AddFormattedItem(ctl, item, EXTLOGPEN, lp, UINT, elpPenStyle, cvtEPS);
     AddFormattedItem(ctl, item, EXTLOGPEN, lp, DWORD, elpWidth, cvt);
     AddFormattedItem(ctl, item, EXTLOGPEN, lp, UINT, elpBrushStyle, cvtBS);
     AddFormattedItem(ctl, item, EXTLOGPEN, lp, COLORREF, elpColor, cvtCR);
     AddItem(ctl, item, EXTLOGPEN, lp, ULONG_PTR, elpHatch, _T("%i"));
     AddFormattedItem(ctl, item, EXTLOGPEN, lp, DWORD, elpNumEntries, cvt);
     HTREEITEM styles = ctl->InsertItem(_T("DWORD elpStyleEntry[1]"), item);
     FormatDWORDs(&lp->elpStyleEntry[0], p1, p2, lp->elpNumEntries);
    } // TreeDecode::FormatLOGPEN

/****************************************************************************
*                      TreeDecode::FormatGdiCommentData
* Inputs:
*       LPBYTE data:
*       LPVOID p1:
*       LPVOID p2:
*       DWORD len:
* Result: void
*       
* Effect: 
*       Handles EMRGDICOMMENT data
****************************************************************************/

void TreeDecode::FormatGdiCommentData(const LPBYTE data, LPVOID p1, LPVOID p2, DWORD len)
    {
     GET_CTL_ITEM();
     
     CString raw;
     CString ASCII;

     CString rawsep;
     CString asep;
     
     static const DWORD LIMIT = 80; // This limit is based on the restriction that a CTreeView will not show more than 260 characters
                                    // for a GDI comment, the length can be vastly longer (one case has 15776 bytes)
     CString ellipsis;
     if(len > LIMIT)
        ellipsis = _T("...");
     
     for(DWORD i = 0; i < min(len, LIMIT); i++)
        { /* format */
         CString t;
         t.Format(_T("%02x"), data[i]);
         raw += rawsep + t;
         rawsep = _T(" ");
         if((i + 1) % 4 == 0)
            rawsep += _T(" ");
         
         t = isprint(data[i]) ? CString((TCHAR)data[i]) : _T(".");
         ASCII += asep + t;

         if((i + 1) % 4 == 0)
            asep = _T(" ");
         else
            asep = _T("");
        } /* format */
     
     raw += ellipsis;
     
     ctl->InsertItem(raw, item);
     ctl->InsertItem(ASCII, item);
    } // TreeDecode::FormatGdiCommentData

/****************************************************************************
*                       TreeDecode::FormatGRADIENT_RECT
* Inputs:
*       const GRADIENT_RECT * gr: A gradient rect
*       LPVOID p1:
*       LPVOID p2:
* Result: void
*       
* Effect: 
*       Formats a single GRADIENT_RECT
****************************************************************************/

void TreeDecode::FormatGRADIENT_RECT(const GRADIENT_RECT * gr, LPVOID p1, LPVOID p2)
    {
     GET_CTL_ITEM();
     AddFormattedItem(ctl, item, GRADIENT_RECT, gr, ULONG, UpperLeft, cvt);
     AddFormattedItem(ctl, item, GRADIENT_RECT, gr, ULONG, LowerRight, cvt);
    } // TreeDecode::FormatGRADIENT_RECT

/****************************************************************************
*                       TreeDecode::FormatGRADIENT_RECTs
* Inputs:
*       const GRADIENT_RECT * gr:
*       LPVOID p1:
*       LPVOID p2:
*       DWORD count: Number of items
* Result: void
*       
* Effect: 
*       Formats the GRADIENT_RECT array
****************************************************************************/

void TreeDecode::FormatGRADIENT_RECTs(const GRADIENT_RECT * gr, LPVOID p1, LPVOID p2, DWORD count)
    {
     GET_CTL_ITEM();

     for(DWORD i = 0; i < count; i++)
        { /* scan rects */
         CString s;
         s.Format(_T("[%d] GRADIENT_RECT"), i);
         HTREEITEM gri = ctl->InsertItem(s, item);
         FormatGRADIENT_RECT(&gr[i], p1, (LPVOID)gri);
        } /* scan rects */
    } // TreeDecode::FormatGRADIENT_RECTs

/****************************************************************************
*                       TreeDecode::FormatGRADIENT_TRIANGLE
* Inputs:
*       const GRADIENT_TRIANGLE * gr: A gradient rect
*       LPVOID p1:
*       LPVOID p2:
* Result: void
*       
* Effect: 
*       Formats a single GRADIENT_TRIANGLE
****************************************************************************/

void TreeDecode::FormatGRADIENT_TRIANGLE(const GRADIENT_TRIANGLE * gr, LPVOID p1, LPVOID p2)
    {
     GET_CTL_ITEM();
     AddFormattedItem(ctl, item, GRADIENT_TRIANGLE, gr, ULONG, Vertex1, cvt);
     AddFormattedItem(ctl, item, GRADIENT_TRIANGLE, gr, ULONG, Vertex2, cvt);
     AddFormattedItem(ctl, item, GRADIENT_TRIANGLE, gr, ULONG, Vertex3, cvt);
    } // TreeDecode::FormatGRADIENT_TRIANGLE

/****************************************************************************
*                       TreeDecode::FormatGRADIENT_TRIANGLEs
* Inputs:
*       const GRADIENT_TRIANGLE * gr:
*       LPVOID p1:
*       LPVOID p2:
*       DWORD count: Number of items
* Result: void
*       
* Effect: 
*       Formats the GRADIENT_TRIANGLE array
****************************************************************************/

void TreeDecode::FormatGRADIENT_TRIANGLEs(const GRADIENT_TRIANGLE * gr, LPVOID p1, LPVOID p2, DWORD count)
    {
     GET_CTL_ITEM();

     for(DWORD i = 0; i < count; i++)
        { /* scan rects */
         CString s;
         s.Format(_T("[%d] GRADIENT_TRIANGLE"), i);
         HTREEITEM gri = ctl->InsertItem(s, item);
         FormatGRADIENT_TRIANGLE(&gr[i], p1, (LPVOID)gri);
        } /* scan rects */
    } // TreeDecode::FormatGRADIENT_TRIANGLEs


/****************************************************************************
*                       TreeDecode::FormatLOGCOLORSPACE
* Inputs:
*       const LOGCOLORSPACEA * lcs
*       LPVOID p1:
*       LPVOID p2:
* Result: void
*       
* Effect: 
*       Formats a LOGCOLORSPACEA object
****************************************************************************/

void TreeDecode::FormatLOGCOLORSPACE(const LOGCOLORSPACEA * lcs, LPVOID p1, LPVOID p2)
    {
     GET_CTL_ITEM();

     if(lcs->lcsSignature == LCS_SIGNATURE)
        ctl->InsertItem(_T("DWORD lcsSignature: LCS_SIGNATURE"), item);
     else
        AddItem(ctl, item, LOGCOLORSPACE, lcs, DWORD, lcsSignature, _T("0x%08x"));

     AddItem(ctl, item, LOGCOLORSPACE, lcs, DWORD, lcsVersion, _T("0x%x"));
     AddFormattedItem(ctl, item, LOGCOLORSPACE, lcs, DWORD, lcsSize, cvt);
     AddFormattedItem(ctl, item, LOGCOLORSPACE, lcs, LCSCSTYPE, lcsCSType, cvtCST);
     AddFormattedItem(ctl, item, LOGCOLORSPACE, lcs, LCSGAMUTMATCH, lcsIntent, cvtIN);
     HTREEITEM tr = ctl->InsertItem(_T("CIEXYZTRIPLE lcsEndpoints"), item);
     FormatCIEXYZTRIPLE(&lcs->lcsEndpoints, p1, (LPVOID)tr);
     AddFormattedItem(ctl, item, LOGCOLORSPACE, lcs, DWORD, lcsGammaRed, cvt);
     AddFormattedItem(ctl, item, LOGCOLORSPACE, lcs, DWORD, lcsGammaGreen, cvt);
     AddFormattedItem(ctl, item, LOGCOLORSPACE, lcs, DWORD, lcsGammaBlue, cvt);
     
    } // TreeDecode::FormatLOGCOLORSPACE

/****************************************************************************
*                       TreeDecode::FormatLOGCOLORSPACEA
* Inputs:
*       LOGCOLORSPACEA * lcs:
*       LPVOID p1:
*       LPVOID p2:
* Result: void
*       
* Effect: 
*       Adds the LOGCOLORSPACE decode
****************************************************************************/

void TreeDecode::FormatLOGCOLORSPACEA(const LOGCOLORSPACEA * lcs, LPVOID p1, LPVOID p2)
    {
     GET_CTL_ITEM();

     FormatLOGCOLORSPACE((LOGCOLORSPACEA*)lcs, p1, p2);
     HTREEITEM fn = ctl->InsertItem(_T("CHAR lcsFilename[MAX_PATH]"), item);
     AddFormattedItem(ctl, fn, LOGCOLORSPACE, lcs, CHAR, lcsFilename, cvtString);
    } // TreeDecode::FormatLOGCOLORSPACEA

/****************************************************************************
*                       TreeDecode::FormatLOGCOLORSPACEW
* Inputs:
*       LOGCOLORSPACEW * lcs:
*       LPVOID p1:
*       LPVOID p2:
* Result: void
*       
* Effect: 
*       Adds the LOGCOLORSPACE decode
****************************************************************************/

void TreeDecode::FormatLOGCOLORSPACEW(const LOGCOLORSPACEW * lcs, LPVOID p1, LPVOID p2)
    {
     GET_CTL_ITEM();

     FormatLOGCOLORSPACE((LOGCOLORSPACEA*)lcs, p1, p2); // interpret as LOGCOLORSPACEA
     HTREEITEM fn = ctl->InsertItem(_T("WCHAR lcsFilename[MAX_PATH]"), item);
     AddFormattedItem(ctl, fn, LOGCOLORSPACE, lcs, WCHAR, lcsFilename, cvtString);
    } // TreeDecode::FormatLOGCOLORSPACEW

/****************************************************************************
*                         TreeDecode::FormatLOGFONTW
* Inputs:
*       const LOGFONTW * lf: LOGFONTW
*       LPVOID p1:
*       LPVOID p2:
* Result: void
*       
* Effect: 
*       Formats a LOGFONTW
****************************************************************************/

void TreeDecode::FormatLOGFONTW(const LOGFONTW * lf, LPVOID p1, LPVOID p2)
    {
     GET_CTL_ITEM();

     AddFormattedItem(ctl, item, LOGFONTW, lf, LONG, lfHeight, cvt);
     AddFormattedItem(ctl, item, LOGFONTW, lf, LONG, lfWidth, cvt);
     AddFormattedItem(ctl, item, LOGFONTW, lf, LONG, lfEscapement, cvt);
     AddFormattedItem(ctl, item, LOGFONTW, lf, LONG, lfOrientation, cvt);
     AddFormattedItem(ctl, item, LOGFONTW, lf, LONG, lfWeight, cvtFW);
     AddFormattedItem(ctl, item, LOGFONTW, lf, BYTE, lfItalic, cvt);
     AddFormattedItem(ctl, item, LOGFONTW, lf, BYTE, lfUnderline, cvt);
     AddFormattedItem(ctl, item, LOGFONTW, lf, BYTE, lfStrikeOut, cvt);
     AddFormattedItem(ctl, item, LOGFONTW, lf, BYTE, lfOutPrecision, cvt);
     AddFormattedItem(ctl, item, LOGFONTW, lf, BYTE, lfClipPrecision, cvt);
     AddFormattedItem(ctl, item, LOGFONTW, lf, BYTE, lfQuality, cvt);
     AddFormattedItem(ctl, item, LOGFONTW, lf, BYTE, lfPitchAndFamily, cvt);

     AddStringItemW(ctl, item, LOGFONTW, lf, TCHAR, lfFaceName, LF_FACESIZE);

    } // TreeDecode::FormatLOGFONTW

/****************************************************************************
*                        TreeDecode::FormatLOGBRUSH32
* Inputs:
*       const LOGBRUSH32 * lb:
*       LPVOID p1:
*       LPVOID p2:
* Result: void
*       
* Effect: 
*       Formats a LOGBRUSH32
****************************************************************************/

void TreeDecode::FormatLOGBRUSH32(const LOGBRUSH32 * lb, LPVOID p1, LPVOID p2)
    {
     GET_CTL_ITEM();
     AddFormattedItem(ctl, item, LOGBRUSH32, lb, UINT, lbStyle, cvtLBS);
     switch(lb->lbStyle)
        { /* decode */
         case BS_HOLLOW:
            AddFormattedItem(ctl, item, LOGBRUSH32, lb, COLORREF, lbColor, cvtCR);
            ctl->InsertItem(_T("ULONG lbHatch: ignored"), item);
            break;
         case BS_PATTERN:
            AddFormattedItem(ctl, item, LOGBRUSH32, lb, COLORREF, lbColor, cvt);
            AddFormattedItem(ctl, item, LOGBRUSH32, lb, ULONG, lbHatch, cvt);
            break;
         case BS_DIBPATTERN:
         case BS_DIBPATTERNPT:
            AddFormattedItem(ctl, item, LOGBRUSH32, lb, COLORREF, lbColor, cvtDBC);
            AddFormattedItem(ctl, item, LOGBRUSH32, lb, ULONG, lbHatch, cvt);
            break;
         case BS_HATCHED:
            AddFormattedItem(ctl, item, LOGBRUSH32, lb, COLORREF, lbColor, cvtCR);
            AddFormattedItem(ctl, item, LOGBRUSH32, lb, ULONG, lbHatch, cvtHT);
         case BS_SOLID:
            AddFormattedItem(ctl, item, LOGBRUSH32, lb, COLORREF, lbColor, cvtCR);
            ctl->InsertItem(_T("ULONG lbHatch: ignored"), item);
            break;
        } /* decode */
    } // TreeDecode::FormatLOGBRUSH32

/****************************************************************************
*                        TreeDecode::FormatLOGPALETTE
* Inputs:
*       LOGPALETTE * pal:
*       LPVOID p1:
*       LPVOID p2:
* Result: void
*       
* Effect: 
*       Formats a LOGPALETTE structure
****************************************************************************/

void TreeDecode::FormatLOGPALETTE(const LOGPALETTE * pal, LPVOID p1, LPVOID p2)
    {
     GET_CTL_ITEM();

     AddFormattedItem(ctl, item, LOGPALETTE, pal, WORD, palVersion, cvt);
     AddFormattedItem(ctl, item, LOGPALETTE, pal, WORD, palNumEntries, cvt);
     HTREEITEM pe = ctl->InsertItem(_T("PALETTEENTRY palPalEntry[1]"), item);
     FormatPALETTEENTRIES(&pal->palPalEntry[0], pal->palNumEntries, p1, (LPVOID)pe);
    } // TreeDecode::FormatLOGPALETTE

/****************************************************************************
*                          TreeDecode::FormatLOGPEN
* Inputs:
*       LOGPEN * lpen:
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEM) item to add to
* Result: void
*       
* Effect: 
*       Decodes a LOGPEN
****************************************************************************/

void TreeDecode::FormatLOGPEN(const LOGPEN * lp, LPVOID p1, LPVOID p2)
    {
     CTreeCtrl * ctl = (CTreeCtrl *)p1;
     HTREEITEM item = (HTREEITEM)p2;

     AddFormattedItem(ctl, item, LOGPEN, lp, UINT, lopnStyle, cvtPS);
     AddFormattedItem(ctl, item, LOGPEN, lp, POINT, lopnWidth, cvt);
     AddFormattedItem(ctl, item, LOGPEN, lp, COLORREF, lopnColor, cvtCR);
    } // TreeDecode::FormatLOGPEN

/****************************************************************************
*                       TreeDecode::FormatPALETTEENTRIES
* Inputs:
*       const PALETTEENTRY * pal:
*       DWORD count: Number of palette entries
*       LPVOID p1:
*       LPVOID p2:
* Result: void
*       
* Effect: 
*       Formats the array of palette entries
****************************************************************************/

void TreeDecode::FormatPALETTEENTRIES(const PALETTEENTRY * pal, DWORD count, LPVOID p1, LPVOID p2)
    {
     GET_CTL_ITEM();

     for(DWORD i = 0; i < count; i++)
        { /* format each */
#if 0
         CString s;
         s.Format(_T("[%d] { %d, %d, %d, %s}"),
                  i,
                  pal[i].peRed,
                  pal[i].peGreen,
                  pal[i].peBlue,
                  cvtPEF(pal[i].peFlags));
#else
         CString s;
         s.Format(_T("[%d] PALETTEENTRY"), i);
         HTREEITEM pe = ctl->InsertItem(s, item);
         FormatPALETTEENTRY(&pal[i], p1, (LPVOID)pe);
#endif
         ctl->InsertItem(s, item);
        } /* format each */
    } // TreeDecode::FormatPALETTEENTRIES

/****************************************************************************
*                       TreeDecode::FormatPALETTEENTRY
* Inputs:
*       const PALETTEENTRY * pe:
*       LPVOID p1:
*       LPVOID p2:
* Result: void
*       
* Effect: 
*       Formats a PALETTEENTRY
****************************************************************************/

void TreeDecode::FormatPALETTEENTRY(const PALETTEENTRY * pe, LPVOID p1, LPVOID p2)
    {
     GET_CTL_ITEM();

     AddFormattedItem(ctl, item, PALETTEENTRY, pe, BYTE, peRed, cvt);
     AddFormattedItem(ctl, item, PALETTEENTRY, pe, BYTE, peGreen, cvt);
     AddFormattedItem(ctl, item, PALETTEENTRY, pe, BYTE, peBlue, cvt);
     AddFormattedItem(ctl, item, PALETTEENTRY, pe, BYTE, peFlags, cvtPEF);
    } // TreeDecode::FormatPALETTEENTRY

/****************************************************************************
*                          TreeDecode::FormatPANOSE
* Inputs:
*       const PANOSE * panose:
*       LPVOID p1:
*       LPVOID p2:
* Result: void
*       
* Effect: 
*       Formats a PANOSE as a subtree of (HTREEITEM)p2
****************************************************************************/

void TreeDecode::FormatPANOSE(const PANOSE * panose, LPVOID p1, LPVOID p2)
    {
     GET_CTL_ITEM();
     AddFormattedItem(ctl, item, PANOSE, panose, BYTE, bFamilyType, cvtPANFT);
     AddFormattedItem(ctl, item, PANOSE, panose, BYTE, bSerifStyle, cvtPANSS);
     AddFormattedItem(ctl, item, PANOSE, panose, BYTE, bWeight, cvtPANWT);
     AddFormattedItem(ctl, item, PANOSE, panose, BYTE, bProportion, cvtPANPR);
     AddFormattedItem(ctl, item, PANOSE, panose, BYTE, bContrast, cvtPANCO);
     AddFormattedItem(ctl, item, PANOSE, panose, BYTE, bStrokeVariation, cvtPANSV);
     AddFormattedItem(ctl, item, PANOSE, panose, BYTE, bArmStyle, cvtPANAS);
     AddFormattedItem(ctl, item, PANOSE, panose, BYTE, bLetterform, cvtPANLF);
     AddFormattedItem(ctl, item, PANOSE, panose, BYTE, bMidline, cvtPANML);
     AddFormattedItem(ctl, item, PANOSE, panose, BYTE, bXHeight, cvtPANXH);
    } // TreeDecode::FormatPANOSE

/****************************************************************************
*                   TreeDecode::FormatPIXELFORMATDESCRIPTOR
* Inputs:
*       PIXELFORMATDESCRIPTOR * pfd:
*       LPVOID p1:
*       LPVOID p2:
* Result: void
*       
* Effect: 
*       Formats the PIXELFORMATDESCRIPTOR
****************************************************************************/

void TreeDecode::FormatPIXELFORMATDESCRIPTOR(const PIXELFORMATDESCRIPTOR * pfd, LPVOID p1, LPVOID p2)
    {
     GET_CTL_ITEM();

     AddFormattedItem(ctl, item, PIXELFORMATDESCRIPTOR, pfd, WORD, nSize, cvt);
     AddFormattedItem(ctl, item, PIXELFORMATDESCRIPTOR, pfd, WORD, nVersion, cvt);
     AddFormattedItem(ctl, item, PIXELFORMATDESCRIPTOR, pfd, DWORD, dwFlags, cvtPFF);
     AddFormattedItem(ctl, item, PIXELFORMATDESCRIPTOR, pfd, BYTE, iPixelType, cvtPT);
     AddFormattedItem(ctl, item, PIXELFORMATDESCRIPTOR, pfd, BYTE, cColorBits, cvt);
     AddFormattedItem(ctl, item, PIXELFORMATDESCRIPTOR, pfd, BYTE, cRedBits, cvt);
     AddFormattedItem(ctl, item, PIXELFORMATDESCRIPTOR, pfd, BYTE, cRedShift, cvt);
     AddFormattedItem(ctl, item, PIXELFORMATDESCRIPTOR, pfd, BYTE, cGreenBits, cvt);
     AddFormattedItem(ctl, item, PIXELFORMATDESCRIPTOR, pfd, BYTE, cGreenShift, cvt);
     AddFormattedItem(ctl, item, PIXELFORMATDESCRIPTOR, pfd, BYTE, cBlueBits, cvt);
     AddFormattedItem(ctl, item, PIXELFORMATDESCRIPTOR, pfd, BYTE, cBlueShift, cvt);
     AddFormattedItem(ctl, item, PIXELFORMATDESCRIPTOR, pfd, BYTE, cAlphaBits, cvt);
     AddFormattedItem(ctl, item, PIXELFORMATDESCRIPTOR, pfd, BYTE, cAlphaShift, cvt);
     AddFormattedItem(ctl, item, PIXELFORMATDESCRIPTOR, pfd, BYTE, cRedBits, cvt);
     AddFormattedItem(ctl, item, PIXELFORMATDESCRIPTOR, pfd, BYTE, cAccumBits, cvt);
     AddFormattedItem(ctl, item, PIXELFORMATDESCRIPTOR, pfd, BYTE, cAccumRedBits, cvt);
     AddFormattedItem(ctl, item, PIXELFORMATDESCRIPTOR, pfd, BYTE, cAccumGreenBits, cvt);
     AddFormattedItem(ctl, item, PIXELFORMATDESCRIPTOR, pfd, BYTE, cAccumBlueBits, cvt);
     AddFormattedItem(ctl, item, PIXELFORMATDESCRIPTOR, pfd, BYTE, cAccumAlphaBits, cvt);
     AddFormattedItem(ctl, item, PIXELFORMATDESCRIPTOR, pfd, BYTE, cDepthBits, cvt);
     AddFormattedItem(ctl, item, PIXELFORMATDESCRIPTOR, pfd, BYTE, cStencilBits, cvt);
     AddFormattedItem(ctl, item, PIXELFORMATDESCRIPTOR, pfd, BYTE, cAuxBuffers, cvt);
     AddFormattedItem(ctl, item, PIXELFORMATDESCRIPTOR, pfd, BYTE, iLayerType, cvt);
     AddFormattedItem(ctl, item, PIXELFORMATDESCRIPTOR, pfd, BYTE, bReserved, cvt);
     AddFormattedItem(ctl, item, PIXELFORMATDESCRIPTOR, pfd, DWORD, dwLayerMask, cvt);
     AddFormattedItem(ctl, item, PIXELFORMATDESCRIPTOR, pfd, DWORD, dwVisibleMask, cvt);
     AddFormattedItem(ctl, item, PIXELFORMATDESCRIPTOR, pfd, DWORD, dwDamageMask, cvt);
    } // TreeDecode::FormatPIXELFORMATDESCRIPTOR

/****************************************************************************
*                          TreeDecode::FormatPOINTL
* Inputs:
*       const POINTL * ptl: Pointer to array of points
*       LPVOID p1:
*       LPVOID p2:
*       DWORD count: # of elements in ptl
* Result: void
*       
* Effect: 
*       Decodes the POINTL array
****************************************************************************/

void TreeDecode::FormatPOINTL(const POINTL * ptl, LPVOID p1, LPVOID p2, DWORD count)
    {
     GET_CTL_ITEM();

     for(DWORD i = 0; i < count; i++)
        { /* add item */
         CString s;
         s.Format(_T("[%d] {%d, %d}"), i, ptl[i].x, ptl[i].y);
         ctl->InsertItem(s, item);
        } /* add item */
    } // TreeDecode::FormatPOINTL

/****************************************************************************
*                          TreeDecode::FormatPOINTS
* Inputs:
*       const POINTS * pts:
*       LPVOID p1:
*       LPVOID p2:
*       DWORD count:
* Result: void
*       
* Effect: 
*       Decodes the POINTS array
****************************************************************************/

void TreeDecode::FormatPOINTS(const POINTS * pts, LPVOID p1, LPVOID p2, DWORD count)
    {
     GET_CTL_ITEM();

     for(DWORD i = 0; i < count; i++)
        { /* add item */
         CString s;
         s.Format(_T("[%d] {%d, %d}"), i, pts[i].x, pts[i].y);
         ctl->InsertItem(s, item);
        } /* add item */
    } // TreeDecode::FormatPOINTS

/****************************************************************************
*                         TreeDecode::FormatRECTArray
* Inputs:
*       const LPRECT array:
*       LPVOID p1:
*       LPVOID p2:
* Result: void
*       
* Effect: 
*       Formats an array of RECTs
****************************************************************************/

void TreeDecode::FormatRECTArray(const LPRECT array, LPVOID p1, LPVOID p2, DWORD count)
    {
     GET_CTL_ITEM();
     
     for(DWORD i = 0; i < count; i++)
        { /* arrays */
         CString s;
         s.Format(_T("[%d] %s"), i, cvt(array[i]));
         ctl->InsertItem(s, item);
        } /* arrays */
    } // TreeDecode::FormatRECTArray

/****************************************************************************
*                          TreeDecode::FormatRGNDATA
* Inputs:
*       const RGNDATA * rgn:
*       LPVOID p1:
*       LPVOID p2:
* Result: void
*       
* Effect: 
*       Formats RGNDATA
* Notes:
*    typedef struct { 
*       RGNDATAHEADER rdh;
*       char Buffer[1];
*    } RGNDATA;
****************************************************************************/

void TreeDecode::FormatRGNDATA(const RGNDATA * rgn, LPVOID p1, LPVOID p2)
    {
     GET_CTL_ITEM();

     HTREEITEM hdr = ctl->InsertItem(_T("RGNDATAHEADER rdh"), item);
     FormatRGNDATAHEADER(&rgn->rdh, p1, (LPVOID)hdr);
     HTREEITEM buffer = ctl->InsertItem(_T("char Buffer[1]"), item);
     // TODO: Figure out why nCount is so large (larger than the available data!)
     //FormatRECTArray((LPRECT)(&rgn->Buffer[0]), p1, (LPVOID)buffer, rgn->rdh.nCount);
    } // TreeDecode::FormatRGNDATA

/****************************************************************************
*                       TreeDecode::FormatRGNDATAHEADER
* Inputs:
*       const RGNDATAHEADER * hdr:
*       LPVOID p1:
*       LPVOID p2:
* Result: void
*       
* Effect: 
*       Formats a RGNDATAHEADER
* Notes:
*    typedef struct { 
*        DWORD dwSize; 
*        DWORD iType; 
*        DWORD nCount; 
*        DWORD nRgnSize; 
*        RECT  rcBound; 
*    } RGNDATAHEADER;
****************************************************************************/

void TreeDecode::FormatRGNDATAHEADER(const RGNDATAHEADER * hdr, LPVOID p1, LPVOID p2)
    {
     GET_CTL_ITEM();

     AddFormattedItem(ctl, item, RGNDATAHEADER, hdr, DWORD, dwSize, cvt);
     AddFormattedItem(ctl, item, RGNDATAHEADER, hdr, DWORD, iType, cvtRDHT);
     AddFormattedItem(ctl, item, RGNDATAHEADER, hdr, DWORD, nCount, cvt);
     AddFormattedItem(ctl, item, RGNDATAHEADER, hdr, DWORD, nRgnSize, cvt);
     AddFormattedItem(ctl, item, RGNDATAHEADER, hdr, RECT, rcBound, cvt);
    } // TreeDecode::FormatRGNDATAHEADER

/****************************************************************************
*                         TreeDecode::FormatTRIVERTEX
* Inputs:
*       const TRIVERTEX * tv:
*       LPVOID p1:
*       LPVOID p2:
* Result: void
*       
* Effect: 
*       Formats a TRIVERTEX
****************************************************************************/

void TreeDecode::FormatTRIVERTEX(const TRIVERTEX * tv, LPVOID p1, LPVOID p2)
    {
     GET_CTL_ITEM();

     AddFormattedItem(ctl, item, TRIVERTEX, tv, LONG, x, cvt);
     AddFormattedItem(ctl, item, TRIVERTEX, tv, LONG, y, cvt);
     AddFormattedItem(ctl, item, TRIVERTEX, tv, COLOR16, Red, cvtC16);
     AddFormattedItem(ctl, item, TRIVERTEX, tv, COLOR16, Green, cvtC16);
     AddFormattedItem(ctl, item, TRIVERTEX, tv, COLOR16, Blue, cvtC16);
     AddFormattedItem(ctl, item, TRIVERTEX, tv, COLOR16, Alpha, cvtC16);
    } // TreeDecode::FormatTRIVERTEX

/****************************************************************************
*                        TreeDecode::FormatTRIVERTEXes
* Inputs:
*       const TRIVERTEX * tv: TRIVERTEXT array
*       LPVOID p1:
*       LPVOID p2:
*       DWORD count: Count of vertices in array
* Result: void
*       
* Effect: 
*       Formats a TRIVERTEX array
****************************************************************************/

void TreeDecode::FormatTRIVERTEXes(const TRIVERTEX * tv, LPVOID p1, LPVOID p2, DWORD count)
    {
     GET_CTL_ITEM();

     for(DWORD i = 0; i < count; i++)
        { /* each */
         CString s;
         s.Format(_T("[%d] TRIVERTEX"), i);
         HTREEITEM v = ctl->InsertItem(s, item);
         FormatTRIVERTEX(&tv[i], p1, (LPVOID)v);
        } /* each */
    } // TreeDecode::FormatTRIVERTEXes

/****************************************************************************
*                           TreeDecode::DecodeAbortPath
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(EMRABORTPATH*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEweM) item to add to
* Result: void
*       
* Effect: 
*       Handles EMRABORTPATH
* Notes:
*    typedef struct { 
*      EMR emr; 
*    } EMRABORTPATH;
****************************************************************************/

void TreeDecode::DecodeAbortPath(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();
     
     SetState(ctl, item, StateIcon::State);
    } // TreeDecode::DecodeAbortPath

/****************************************************************************
*                      TreeDecode::DecodeAlphaBlend
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(EMRALPHABLEND*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEM) item to add to
* Result: void
*       
* Effect: 
*       Handles EMRALPHABLEND
* Notes:
*    typedef struct { 
*      EMR     emr;       
*      RECTL   rclBounds; 
*      LONG    xDest;     
*      LONG    yDest;     
*      LONG    cxDest;    
*      LONG    cyDest;    
*      DWORD   dwRop;     
*      LONG    xSrc;      
*      LONG    ySrc;      
*      XFORM   xformSrc;  
*      COLORREF crBkColorSrc;
*      DWORD   iUsageSrc; 
*      DWORD   offBmiSrc; 
*      DWORD   cbBmiSrc;  
*      DWORD   offBitsSrc;
*      DWORD   cbBitsSrc; 
*      LONG    cxSrc; 
*      LONG    cySrc; 
*    } EMRALPHABLEND
****************************************************************************/

void TreeDecode::DecodeAlphaBlend(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();

     SetState(ctl, item, StateIcon::Drawing);
     
     AddFormattedItem(ctl, item, EMRALPHABLEND, rec, RECTL, rclBounds, cvt);
     AddFormattedItem(ctl, item, EMRALPHABLEND, rec, LONG, xDest, cvt);
     AddFormattedItem(ctl, item, EMRALPHABLEND, rec, LONG, yDest, cvt);
     AddFormattedItem(ctl, item, EMRALPHABLEND, rec, LONG, cxDest, cvt);
     AddFormattedItem(ctl, item, EMRALPHABLEND, rec, LONG, cyDest, cvt);
     AddFormattedItem(ctl, item, EMRALPHABLEND, rec, DWORD, dwRop, cvtROP);
     AddFormattedItem(ctl, item, EMRALPHABLEND, rec, LONG, xSrc, cvt);
     AddFormattedItem(ctl, item, EMRALPHABLEND, rec, LONG, ySrc, cvt);
     AddFormattedItem(ctl, item, EMRALPHABLEND, rec, XFORM, xformSrc, cvt);
     AddFormattedItem(ctl, item, EMRALPHABLEND, rec, COLORREF, crBkColorSrc, cvtCR);
     AddFormattedItem(ctl, item, EMRALPHABLEND, rec, DWORD, iUsageSrc, cvtDBC);
     HTREEITEM bmi;
     AddFormattedItemEx(ctl, item, bmi, EMRALPHABLEND, rec, DWORD, offBmiSrc, cvt);
     if(((EMRALPHABLEND *)rec)->offBmiSrc != 0)
        FormatBITMAPINFO((LPBITMAPINFO)((LPBYTE)rec + ((EMRALPHABLEND *)rec)->offBmiSrc), p1, (LPVOID)bmi);
     AddFormattedItem(ctl, item, EMRALPHABLEND, rec, DWORD, cbBmiSrc, cvt);
     AddFormattedItem(ctl, item, EMRALPHABLEND, rec, DWORD, offBitsSrc, cvt);
     AddFormattedItem(ctl, item, EMRALPHABLEND, rec, DWORD, cbBitsSrc, cvt);
     AddFormattedItem(ctl, item, EMRALPHABLEND, rec, LONG, cxSrc, cvt);
     AddFormattedItem(ctl, item, EMRALPHABLEND, rec, LONG, cySrc, cvt);
    } // TreeDecode::DecodeAlphaBlend

/****************************************************************************
*                         TreeDecode::DecodeAngleArc
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(EMRANGLEARC*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEM) item to add to
* Result: void
*       
* Effect: 
*       Decodes EMRANGLEARC
* Notes:
*    typedef struct {
*      EMR     emr;
*      POINTL  ptlCenter; 
*      DWORD   nRadius;
*      FLOAT   eStartAngle;
*      FLOAT   eSweepAngle;
*    } EMRANGLEARC;
****************************************************************************/

void TreeDecode::DecodeAngleArc(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();

     SetState(ctl, item, StateIcon::Drawing);
     AddFormattedItem(ctl, item, EMRANGLEARC, rec, POINTL, ptlCenter, cvt);
     AddFormattedItem(ctl, item, EMRANGLEARC, rec, DWORD, nRadius, cvt);
     AddFormattedItem(ctl, item, EMRANGLEARC, rec, FLOAT, eStartAngle, cvt);
     AddFormattedItem(ctl, item, EMRANGLEARC, rec, FLOAT, eSweepAngle, cvt);
     
    } // TreeDecode::DecodeAngleArc

/****************************************************************************
*                            TreeDecode::DecodeArc
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(EMRARC*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEM) item to add to
* Result: void
*       
* Effect: 
*       Handles EMRARC
* Notes:
*    typedef struct {
*      EMR    emr; 
*      RECTL  rclBox;
*      POINTL ptlStart;
*      POINTL ptlEnd;
*    } EMRARC;
****************************************************************************/

void TreeDecode::DecodeArc(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();

     SetState(ctl, item, StateIcon::Drawing);
     AddFormattedItem(ctl, item, EMRARC, rec, RECTL, rclBox, cvt);
     AddFormattedItem(ctl, item, EMRARC, rec, POINTL, ptlStart, cvt);
     AddFormattedItem(ctl, item, EMRARC, rec, POINTL, ptlEnd, cvt);
    } // TreeDecode::DecodeArc

/****************************************************************************
*                            TreeDecode::DecodeArcTo
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(EMRARCTO*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEM) item to add to
* Result: void
*       
* Effect: 
*       Handles EMRARCTO
* Notes:
*    typedef struct tagEMRARC {
*      EMR    emr; 
*      RECTL  rclBox;
*      POINTL ptlStart;
*      POINTL ptlEnd;
*    } EMRARCTO;
****************************************************************************/

void TreeDecode::DecodeArcTo(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();

     SetState(ctl, item, StateIcon::Drawing);
     AddFormattedItem(ctl, item, EMRARCTO, rec, RECTL, rclBox, cvt);
     AddFormattedItem(ctl, item, EMRARCTO, rec, POINTL, ptlStart, cvt);
     AddFormattedItem(ctl, item, EMRARCTO, rec, POINTL, ptlEnd, cvt);
    } // TreeDecode::DecodeArcTo

/****************************************************************************
*                           TreeDecode::DecodeBeginPath
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(EMRBEGINPATH*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEM) item to add to
* Result: void
*       
* Effect: 
*       Marks element as state
* Notes:
*    typedef struct { 
*      EMR emr; 
*    } EMRBEGINPATH;
****************************************************************************/

void TreeDecode::DecodeBeginPath(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();
     
     SetState(ctl, item, StateIcon::State);
    } // TreeDecode::DecodeBeginPath

/****************************************************************************
*                          TreeDecode::DecodeBitBlt
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(EMRBITBLT*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEM) item to add to
* Result: void
*       
* Effect: 
*       Handles EMRBITBLT
* Notes:
*    typedef struct {
*      EMR      emr; 
*      RECTL    rclBounds; 
*      LONG     xDest; 
*      LONG     yDest; 
*      LONG     cxDest; 
*      LONG     cyDest; 
*      DWORD    dwRop; 
*      LONG     xSrc; 
*      LONG     ySrc; 
*      XFORM    xformSrc; 
*      COLORREF crBkColorSrc; 
*      DWORD    iUsageSrc; 
*      DWORD    offBmiSrc; 
*      DWORD    cbBmiSrc; 
*      DWORD    offBitsSrc; 
*      DWORD    cbBitsSrc; 
*    } EMRBITBLT;
****************************************************************************/

void TreeDecode::DecodeBitBlt(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();

     SetState(ctl, item, StateIcon::Drawing);
     AddFormattedItem(ctl, item, EMRBITBLT, rec, RECTL, rclBounds, cvt);
     AddFormattedItem(ctl, item, EMRBITBLT, rec, LONG, xDest, cvt);
     AddFormattedItem(ctl, item, EMRBITBLT, rec, LONG, yDest, cvt);
     AddFormattedItem(ctl, item, EMRBITBLT, rec, LONG, cxDest, cvt);
     AddFormattedItem(ctl, item, EMRBITBLT, rec, LONG, cyDest, cvt);
     AddFormattedItem(ctl, item, EMRBITBLT, rec, DWORD, dwRop, cvtROP);
     AddFormattedItem(ctl, item, EMRBITBLT, rec, LONG, xSrc, cvt);
     AddFormattedItem(ctl, item, EMRBITBLT, rec, LONG, ySrc, cvt);
     AddFormattedItem(ctl, item, EMRBITBLT, rec, XFORM, xformSrc, cvt);
     AddFormattedItem(ctl, item, EMRBITBLT, rec, COLORREF, crBkColorSrc, cvtCR);
     AddFormattedItem(ctl, item, EMRBITBLT, rec, DWORD, iUsageSrc, cvtDBC);
     HTREEITEM bmi;
     AddItemEx(ctl, item, bmi, EMRBITBLT, rec, DWORD, offBmiSrc, _T("%d"));
     if(((EMRBITBLT *)rec)->offBmiSrc != 0)
        FormatBITMAPINFO((LPBITMAPINFO)((LPBYTE)rec + ((EMRBITBLT *)rec)->offBmiSrc), p1, (LPVOID)bmi);
     AddFormattedItem(ctl, item, EMRBITBLT, rec, DWORD, cbBmiSrc, cvt);
     AddFormattedItem(ctl, item, EMRBITBLT, rec, DWORD, offBitsSrc, cvt);
     AddFormattedItem(ctl, item, EMRBITBLT, rec, DWORD, cbBitsSrc, cvt);
    } // TreeDecode::DecodeBitBlt

/****************************************************************************
*                            TreeDecode::DecodeChord
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(EMRCHORD*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEM) item to add to
* Result: void
*       
* Effect: 
*       Handles EMRCHORD
* Notes:
*    typedef struct {
*      EMR    emr; 
*      RECTL  rclBox;
*      POINTL ptlStart;
*      POINTL ptlEnd;
*    } EMRCHORD;
****************************************************************************/

void TreeDecode::DecodeChord(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();

     SetState(ctl, item, StateIcon::Drawing);
     AddFormattedItem(ctl, item, EMRCHORD, rec, RECTL, rclBox, cvt);
     AddFormattedItem(ctl, item, EMRCHORD, rec, POINTL, ptlStart, cvt);
     AddFormattedItem(ctl, item, EMRCHORD, rec, POINTL, ptlEnd, cvt);
    } // TreeDecode::DecodeChord

/****************************************************************************
*                           TreeDecode::DecodeCloseFigure
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(EMRCLOSEFIGURE*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEM) item to add to
* Result: void
*       
* Effect: 
*       Marks element as state
* Notes:
*    typedef struct { 
*      EMR emr; 
*    } EMRCLOSEFIGURE;
****************************************************************************/

void TreeDecode::DecodeCloseFigure(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();
     
     SetState(ctl, item, StateIcon::State);
    } // TreeDecode::DecodeCloseFigure

/****************************************************************************
*                    TreeDecode::DecodeColorCorrectPalette
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(EMRCOLORCORRECTPALETTE*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEweM) item to add to
* Result: void
*       
* Effect: 
*       Handles EMRCOLORCORRECTPALETTE
* Notes:
*    typedef struct {
*      EMR      emr; 
*      DWORD    ihPalette;
*      DWORD    nFirstEntry;
*      DWORD    nPalEntries;
*      DWORD    nReserved;
*    } EMRCOLORCORRECTPALETTE;
****************************************************************************/

void TreeDecode::DecodeColorCorrectPalette(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();

     SetState(ctl, item, StateIcon::State);
     AddFormattedItem(ctl, item, EMRCOLORCORRECTPALETTE, rec, DWORD, ihPalette, cvt);
     AddFormattedItem(ctl, item, EMRCOLORCORRECTPALETTE, rec, DWORD, nFirstEntry, cvt);
     AddFormattedItem(ctl, item, EMRCOLORCORRECTPALETTE, rec, DWORD, nPalEntries, cvt);
     AddFormattedItem(ctl, item, EMRCOLORCORRECTPALETTE, rec, DWORD, nReserved, cvt);
    } // TreeDecode::DecodeColorCorrectPalette

/****************************************************************************
*                    TreeDecode::DecodeColorMatchToTargetW
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(EMRCOLORMATCHTOTARGET*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEM) item to add to
* Result: void
*       
* Effect: 
*       Handles EMRCOLORMATCHTOTARGET
* Notes:
*    typedef struct tagCOLORMATCHTOTARGET {
*      EMR   emr;
*      DWORD dwAction;
*      DWORD dwFlags;
*      DWORD cbName;
*      DWORD cbData;
*      BYTE  Data[1];
*    } EMRCOLORMATCHTOTARGET;
****************************************************************************/

void TreeDecode::DecodeColorMatchToTargetW(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();

     SetState(ctl, item, StateIcon::State);
     AddFormattedItem(ctl, item, EMRCOLORMATCHTOTARGET, rec, DWORD, dwAction, cvtAC);
     AddFormattedItem(ctl, item, EMRCOLORMATCHTOTARGET, rec, DWORD, dwFlags, cvtCMTT);
     AddFormattedItem(ctl, item, EMRCOLORMATCHTOTARGET, rec, DWORD, cbName, cvt);
     AddFormattedItem(ctl, item, EMRCOLORMATCHTOTARGET, rec, DWORD, cbData, cvt);
     HTREEITEM data = ctl->InsertItem(_T("BYTE Data[1]"));
     // TODO: decode data
    } // TreeDecode::DecodeColorMatchToTargetW

/****************************************************************************
*                    TreeDecode::DecodeCreateBrushIndirect
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(EMRCREATEBRUSHINDIRECT*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEM) item to add to
* Result: void
*       
* Effect: 
*       Handles EMRCREATEBRUSHINDIRECT
* Notes:
*    typedef struct {
*      EMR      emr; 
*      DWORD    ihBrush; 
*      LOGBRUSH32 lb; 
*    } EMRCREATEBRUSHINDIRECT;
****************************************************************************/

void TreeDecode::DecodeCreateBrushIndirect(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();

     SetState(ctl, item, StateIcon::State);
     AddFormattedItem(ctl, item, EMRCREATEBRUSHINDIRECT, rec, DWORD, ihBrush, cvt);
     HTREEITEM lb = ctl->InsertItem(_T("LOGBRUSH32 lb"), item);
     FormatLOGBRUSH32(&((EMRCREATEBRUSHINDIRECT *)rec)->lb, p1, (LPVOID)lb);
    } // TreeDecode::DecodeCreateBrushIndirect

/****************************************************************************
*                     TreeDecode::DecodeCreateColorSpace
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(EMRCREATECOLORSPACE*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEM) item to add to
* Result: void
*       
* Effect: 
*       Handles EMRCREATECOLORSPACE
* Notes:
*    typedef struct {
*      EMR           emr; 
*      DWORD         ihCS; 
*      LOGCOLORSPACE lcs; 
*    } EMRCREATECOLORSPACE;
****************************************************************************/

void TreeDecode::DecodeCreateColorSpace(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();

     SetState(ctl, item, StateIcon::State);
     AddFormattedItem(ctl, item, EMRCREATECOLORSPACE, rec, DWORD, ihCS, cvt);
     HTREEITEM cs = ctl->InsertItem(_T("LOGCOLORSPACE lcs"), item);
     FormatLOGCOLORSPACE(&((EMRCREATECOLORSPACE *)rec)->lcs, p1, (LPVOID)cs);
    } // TreeDecode::DecodeCreateColorSpace

/****************************************************************************
*                     TreeDecode::DecodeCreateColorSpaceW
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(EMRCREATECOLORSPACEW*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEM) item to add to
* Result: void
*       
* Effect: 
*       Handles EMRCREATECOLORSPACEW
* Notes:
*    typedef struct {
*      EMR           emr; 
*      DWORD         ihCS; 
*      LOGCOLORSPACEW lcs;
*      DWORD          dwFlags;
*      DWORD          cbData;
*      BYTE           Data[1]; 
*    } EMRCREATECOLORSPACEW;
****************************************************************************/

void TreeDecode::DecodeCreateColorSpaceW(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();

     SetState(ctl, item, StateIcon::State);
     AddFormattedItem(ctl, item, EMRCREATECOLORSPACEW, rec, DWORD, ihCS, cvt);
     HTREEITEM cs = ctl->InsertItem(_T("LOGCOLORSPACEW lcs"), item);
     FormatLOGCOLORSPACEW(&((EMRCREATECOLORSPACEW *)rec)->lcs, p1, (LPVOID)cs);
     AddFormattedItem(ctl, item, EMRCREATECOLORSPACEW, rec, DWORD, dwFlags, cvtCCSF);
     AddFormattedItem(ctl, item, EMRCREATECOLORSPACEW, rec, DWORD, cbData, cvt);     
     HTREEITEM data = ctl->InsertItem(_T("BYTE Data[1]"), item);
    } // TreeDecode::DecodeCreateColorSpaceW

/****************************************************************************
*                  TreeDecode::DecodeCreateDIBPatternBrushPt
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(EMR_CREATEDIBPATTERNBRUSHPT*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEM) item to add to
* Result: void
*       
* Effect: 
*       Handles EMR_CREATEDIBPATTERNBRUSHPT
* Notes:
*    typedef struct {
*      EMR   emr; 
*      DWORD ihBrush; 
*      DWORD iUsage; 
*      DWORD offBmi; 
*      DWORD cbBmi; 
*      DWORD offBits; 
*      DWORD cbBits; 
*    } EMRCREATEDIBPATTERNBRUSHPT;
****************************************************************************/

void TreeDecode::DecodeCreateDIBPatternBrushPt(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();

     SetState(ctl, item, StateIcon::State);
     
     AddFormattedItem(ctl, item, EMRCREATEDIBPATTERNBRUSHPT, rec, DWORD, ihBrush, cvtOB);
     AddFormattedItem(ctl, item, EMRCREATEDIBPATTERNBRUSHPT, rec, DWORD, iUsage, cvtDBC);
     HTREEITEM bmi;
     AddFormattedItemEx(ctl, item, bmi, EMRCREATEDIBPATTERNBRUSHPT, rec, DWORD, offBmi, cvt);
     if(((EMRCREATEDIBPATTERNBRUSHPT *)rec)->offBmi != 0)
        FormatBITMAPINFO((LPBITMAPINFO)((LPBYTE)rec + ((EMRCREATEDIBPATTERNBRUSHPT *)rec)->offBmi), p1, (LPVOID)bmi);
     AddFormattedItem(ctl, item, EMRCREATEDIBPATTERNBRUSHPT, rec, DWORD, cbBmi, cvt);
     AddFormattedItem(ctl, item, EMRCREATEDIBPATTERNBRUSHPT, rec, DWORD, offBits, cvt);
     AddFormattedItem(ctl, item, EMRCREATEDIBPATTERNBRUSHPT, rec, DWORD, cbBits, cvt);
     // TODO: Decode bitmap
    } // TreeDecode::DecodeCreateDIBPatternBrushPt

/****************************************************************************
*                      TreeDecode::DecodeCreateMonoBrush
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(EMRCREATEMONOBRUSH*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEM) item to add to
* Result: void
*       
* Effect: 
*       Handles EMRCREATEMONOBRUSH
* Notes:
*    typedef struct {
*      EMR   emr; 
*      DWORD ihBrush; 
*      DWORD iUsage; 
*      DWORD offBmi; 
*      DWORD cbBmi; 
*      DWORD offBits; 
*      DWORD cbBits; 
*    } EMRCREATEMONOBRUSH;
****************************************************************************/

void TreeDecode::DecodeCreateMonoBrush(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();

     SetState(ctl, item, StateIcon::State);

     AddFormattedItem(ctl, item, EMRCREATEMONOBRUSH, rec, DWORD, ihBrush, cvtOB);
     AddFormattedItem(ctl, item, EMRCREATEMONOBRUSH, rec, DWORD, iUsage, cvtDBC);
     HTREEITEM bmi;
     AddFormattedItemEx(ctl, item, bmi, EMRCREATEMONOBRUSH, rec, DWORD, offBmi, cvt);
     if(((EMRCREATEMONOBRUSH *)rec)->offBmi != 0)
        FormatBITMAPINFO((LPBITMAPINFO)((LPBYTE)rec + ((EMRCREATEMONOBRUSH *)rec)->offBmi), p1, (LPVOID)bmi);
     AddFormattedItem(ctl, item, EMRCREATEMONOBRUSH, rec, DWORD, cbBmi, cvt);
     AddFormattedItem(ctl, item, EMRCREATEMONOBRUSH, rec, DWORD, offBits, cvt);
     AddFormattedItem(ctl, item, EMRCREATEMONOBRUSH, rec, DWORD, cbBits, cvt);
     // TODO: Decode bitmap
    } // TreeDecode::DecodeCreateMonoBrush

/****************************************************************************
*                       TreeDecode::DecodeCreatePalette
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(EMRCREATEPALLETE*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEweM) item to add to
* Result: void
*       
* Effect: 
*       Handles EMRCREATEPALLETE
* Notes:
*    typedef struct {
*      EMR        emr; 
*      DWORD      ihPal; 
*      LOGPALETTE lgpl; 
*    } EMRCREATEPALETTE
****************************************************************************/

void TreeDecode::DecodeCreatePalette(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();

     SetState(ctl, item, StateIcon::State);
     AddFormattedItem(ctl, item, EMRCREATEPALETTE, rec, DWORD, ihPal, cvt);     
     HTREEITEM pal = ctl->InsertItem(_T("LOGPALETTE lgpl"), item);
     FormatLOGPALETTE(&(((EMRCREATEPALETTE*)rec)->lgpl), p1, (LPVOID)pal);
    } // TreeDecode::DecodeCreatePalette

/****************************************************************************
*                         TreeDecode::DecodeCreatePen
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(EMRCREATEPEN*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEM) item to add to
* Result: void
*       
* Effect: 
*       Decodes an EMRCREATEPEN
* Notes:
*    typedef struct {
*      EMR    emr; 
*      DWORD  ihPen; 
*      LOGPEN lopn; 
*    } EMRCREATEPEN;
****************************************************************************/

void TreeDecode::DecodeCreatePen(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();
     
     SetState(ctl, item, StateIcon::State);
     AddFormattedItem(ctl, item, EMRCREATEPEN, rec, DWORD, ihPen, cvt);
     HTREEITEM lpen = ctl->InsertItem(_T("LOGPEN lopn"), item);
     FormatLOGPEN(&((EMRCREATEPEN*)rec)->lopn, p1, (LPVOID)lpen);
    } // TreeDecode::DecodeCreatePen

/****************************************************************************
*                     TreeDecode::DecodeDeleteColorSpace
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(EMRDELETECOLORSPACE*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEM) item to add to
* Result: void
*       
* Effect: 
*       Handles EMRDELETECOLORSPACE
* Notes:
*    typedef struct { 
*      EMR     emr; 
*      DWORD   ihCS; 
*    } EMRDELETECOLORSPACE;
****************************************************************************/

void TreeDecode::DecodeDeleteColorSpace(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();

     SetState(ctl, item, StateIcon::State);
     AddFormattedItem(ctl, item, EMRDELETECOLORSPACE, rec, DWORD, ihCS, cvt);
    } // TreeDecode::DecodeDeleteColorSpace

/****************************************************************************
*                          TreeDecode::DecodeDeleteObject
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(EMRDELETEOBJECT*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEM) item to add to
* Result: void
*       
* Effect: 
*       Decodes EMRDELETEOBJECT
* Notes:
*    typedef struct {
*      EMR   emr; 
*      DWORD ihObject;
*    } EMRDELETEOBJECT;
****************************************************************************/

void TreeDecode::DecodeDeleteObject(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();
     
     SetState(ctl, item, StateIcon::State);
     AddFormattedItem(ctl, item, EMRDELETEOBJECT, rec, DWORD, ihObject, cvt);
    } // TreeDecode::DecodeDeleteObject

/****************************************************************************
*                           TreeDecode::DecodeEllipse
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(EMRELLIPSE*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEM) item to add to
* Result: void
*       
* Effect: 
*       Decodes EMRELLIPSE
* Notes:
*    typedef struct {
*      EMR   emr; 
*      RECTL rclBox; 
*    } EMRELLIPSE;
****************************************************************************/

void TreeDecode::DecodeEllipse(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();

     SetState(ctl, item, StateIcon::Drawing);
     AddFormattedItem(ctl, item, EMRELLIPSE, rec, RECTL, rclBox, cvt);
    } // TreeDecode::DecodeEllipse

/****************************************************************************
*                           TreeDecode::DecodeEndPath
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(EMRENDPATH*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEM) item to add to
* Result: void
*       
* Effect: 
*       Marks element as state
* Notes:
*    typedef struct { 
*      EMR emr; 
*    } EMRENDPATH;
****************************************************************************/

void TreeDecode::DecodeEndPath(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();
     
     SetState(ctl, item, StateIcon::State);
    } // TreeDecode::DecodeEndPath

/****************************************************************************
*                            TreeDecode::DecodeEOF
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(EMREOF*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEM) item to add to
* Result: void
*       
* Effect: 
*       Decodes an EMREOF
* Notes:
*    typedef struct {
*      EMR     emr;
*      DWORD   nPalEntries; 
*      DWORD   offPalEntries; 
*      DWORD   nSizeLast;  // See below
*    } EMREOF;
* Notes:
*       +----------------+  - - - - - - - - - - - - - - - - - - - - - - - -
*       | EMR_EOF        |          ^                         ^
*       +----------------+          |                         |
*       | .nSize         |          |                         |
*       +----------------+          | offPalEntries           |
*       | .nPalEntries   |          |                         | offPaleEntries +
*       +----------------+          |                         |   nPalEntries * sizeof(PALETTEENTRY)
*       | .offPalEntries |          V                         |
*       +----------------+ - - - - - - - -                    |
*       |                | \                                  |
*       :                :  > .nPalEntries * sizeof(RGBQUAD)  |
*       |                | /                                  V
*       +----------------+  - - - - - - - - - - - - - - - - - - - - - - - -
*       | .nSizeLast     |
*       +----------------+
****************************************************************************/

void TreeDecode::DecodeEOF(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();

     SetState(ctl, item, StateIcon::State);
     AddFormattedItem(ctl, item, EMREOF, rec, DWORD, nPalEntries, cvt);
     AddFormattedItem(ctl, item, EMREOF, rec, DWORD, offPalEntries, cvt);

     DWORD nPalEntries = ((EMREOF*)rec)->nPalEntries;
     DWORD offPalEntries = ((EMREOF*)rec)->offPalEntries;

     if(nPalEntries > 0)
        { /* has palette entries */
         HTREEITEM pal = ctl->InsertItem(_T("PALETTEENTRIEs"), item);
         FormatPALETTEENTRIES((PALETTEENTRY*)(((LPBYTE)rec) + offPalEntries), nPalEntries, p1, (LPVOID)pal);
        } /* has palette entries */

     DWORD nSizeLast = *((DWORD*) ( ((LPBYTE)rec) +  offPalEntries + nPalEntries * sizeof(PALETTEENTRY)));
     CString s;
     s.Format(_T("DWORD nSizeLast: %u"), nSizeLast);
     ctl->InsertItem(s, item);
    } // TreeDecode::DecodeEOF

/****************************************************************************
*                         TreeDecode::DecodeExcludeClipRect
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(EMREXCLUDECLIPRECT*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEM) item to add to
* Result: void
*       
* Effect: 
*       Handles EMRINTERSECTCLIPRECT
* Notes:
*    typedef struct {
*      EMR   emr; 
*      RECTL rclClip; 
*    } EMREXCLUDECLIPRECT;
****************************************************************************/

void TreeDecode::DecodeExcludeClipRect(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();

     SetState(ctl, item, StateIcon::State);
     AddFormattedItem(ctl, item, EMREXCLUDECLIPRECT, rec, RECTL, rclClip, cvt);
    } // TreeDecode::DecodeClipRect

/****************************************************************************
*                  TreeDecode::DecodeExtCreateFontIndirectW
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(EMREXTCREATEFONTINDIRECTW*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEM) item to add to
* Result: void
*       
* Effect: 
*       Decodes EMREXTREATEFONTINDIRECTW
* Notes:
*    typedef struct {
*      EMR         emr; 
*      DWORD       ihFont; 
*      EXTLOGFONTW elfw; 
*    } EMREXTCREATEFONTINDIRECTW
****************************************************************************/

void TreeDecode::DecodeExtCreateFontIndirectW(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();

     SetState(ctl, item, StateIcon::State);
     AddFormattedItem(ctl, item, EMREXTCREATEFONTINDIRECTW, rec, DWORD, ihFont, cvt);
     HTREEITEM font = ctl->InsertItem(_T("EXTLOGFONTW elfw"), item);
     FormatEXTLOGFONTW(&((EMREXTCREATEFONTINDIRECTW*)rec)->elfw, p1, p2);
    } // TreeDecode::DecodeExtCreateFontIndirectW

/****************************************************************************
*                         TreeDecode::DecodeExtCreatePen
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(EMREXTCREATEPEN*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEM) item to add to
* Result: void
*       
* Effect: 
*       Decodes an EMREXTCREATEPEN
* Notes:
*    typedef struct {
*      EMR       emr; 
*      DWORD     ihPen; 
*      DWORD     offBmi; 
*      DWORD     cbBmi; 
*      DWORD     offBits; 
*      DWORD     cbBits; 
*      EXTLOGPEN elp; 
*    } EMREXTCREATEPEN;
****************************************************************************/

void TreeDecode::DecodeExtCreatePen(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();
     
     SetState(ctl, item, StateIcon::State);
     AddFormattedItem(ctl, item, EMREXTCREATEPEN, rec, DWORD, ihPen, cvt);
     AddFormattedItem(ctl, item, EMREXTCREATEPEN, rec, DWORD, offBmi, cvt);
     HTREEITEM bmi;
     AddFormattedItemEx(ctl, item, bmi, EMREXTCREATEPEN, rec, DWORD, offBmi, cvt);
     if(((EMREXTCREATEPEN *)rec)->offBmi != 0)
        FormatBITMAPINFO((LPBITMAPINFO)((LPBYTE)rec + ((EMREXTCREATEPEN *)rec)->offBmi), p1, (LPVOID)bmi);
     AddFormattedItem(ctl, item, EMREXTCREATEPEN, rec, DWORD, cbBmi, cvt);
     AddFormattedItem(ctl, item, EMREXTCREATEPEN, rec, DWORD, offBits, cvt);
     AddFormattedItem(ctl, item, EMREXTCREATEPEN, rec, DWORD, cbBits, cvt);
     // TODO: Decode bitmap
     HTREEITEM lpen = ctl->InsertItem(_T("EXTLOGPEN elp"), item);
     FormatEXTLOGPEN(&((EMREXTCREATEPEN*)rec)->elp, p1, (LPVOID)lpen);
    } // TreeDecode::DecodeExtCreatePen

/****************************************************************************
*                       TreeDecode::DecodeExtFloodFill
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(EMREXTFLOODFILL*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEM) item to add to
* Result: void
*       
* Effect: 
*       Handles EMREXTFLOODFILL
* Notes:
*    typedef struct {
*      EMR     emr; 
*      POINTL  ptlStart; 
*      COLORREF crColor; 
*      DWORD   iMode; 
*    } EMREXTFLOODFILL;
****************************************************************************/

void TreeDecode::DecodeExtFloodFill(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();

     SetState(ctl, item, StateIcon::Drawing);
     AddFormattedItem(ctl, item, EMREXTFLOODFILL, rec, POINTL, ptlStart, cvt);
     AddFormattedItem(ctl, item, EMREXTFLOODFILL, rec, COLORREF, crColor, cvtCR);
     AddFormattedItem(ctl, item, EMREXTFLOODFILL, rec, DWORD, iMode, cvtFF);
    } // TreeDecode::DecodeExtFloodFill

/****************************************************************************
*                     TreeDecode::DecodeExtSelectClipRgn
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(EMRSELECTCLIPRGN*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEM) item to add to
* Result: void
*       
* Effect: 
*       Decodes EMREXTSELECTCLIPRGN
* Notes:
*    typedef struct {
*      EMR   emr; 
*      DWORD cbRgnData; 
*      DWORD iMode; 
*      BYTE  RgnData[1]; 
*    } EMREXTSELECTCLIPRGN
****************************************************************************/

void TreeDecode::DecodeExtSelectClipRgn(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();

     SetState(ctl, item, StateIcon::State);
     AddFormattedItem(ctl, item, EMREXTSELECTCLIPRGN, rec, DWORD, cbRgnData, cvt);
     AddFormattedItem(ctl, item, EMREXTSELECTCLIPRGN, rec, DWORD, iMode, cvtRGN);
     HTREEITEM rgn = ctl->InsertItem(_T("BYTE RgnData[1]"), item);
     FormatRGNDATA((const RGNDATA *)&((EMREXTSELECTCLIPRGN*)rec)->RgnData[0], p1, (LPVOID)rgn);
    } // TreeDecode::DecodeExtSelectClipRgn

/****************************************************************************
*                        TreeDecode::DecodeExtTextOutA
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(EMREXTTEXTOUTA*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEM) item to add to
* Result: void
*       
* Effect: 
*       Decodes an EMREXTEXTOUTA
* Notes:
*    typedef struct {
*      EMR     emr; 
*      RECTL   rclBounds; 
*      DWORD   iGraphicsMode; 
*      FLOAT   exScale; 
*      FLOAT   eyScale;
*      EMRTEXT emrtext; 
*    } EMREXTTEXTOUTA;
****************************************************************************/

void TreeDecode::DecodeExtTextOutA(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();

     SetState(ctl, item, StateIcon::Drawing);
     AddFormattedItem(ctl, item, EMREXTTEXTOUTA, rec, RECTL, rclBounds, cvt);
     AddFormattedItem(ctl, item, EMREXTTEXTOUTA, rec, DWORD, iGraphicsMode, cvtGM);
     AddFormattedItem(ctl, item, EMREXTTEXTOUTA, rec, FLOAT, exScale, cvt);
     AddFormattedItem(ctl, item, EMREXTTEXTOUTA, rec, FLOAT, eyScale, cvt);
     HTREEITEM text = ctl->InsertItem(_T("EMRTEXT emrtext"), item);
     FormatEMRTextA(&((EMREXTTEXTOUTA*)rec)->emrtext, p1, (LPVOID)text, (LPCVOID)rec);
    } // TreeDecode::DecodeExtTextOutA

/****************************************************************************
*                        TreeDecode::DecodeExtTextOutW
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(EMREXTTEXTOUTW*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEM) item to add to
* Result: void
*       
* Effect: 
*       Decodes an EMREXTEXTOUTW
* Notes:
*    typedef struct {
*      EMR     emr; 
*      RECTL   rclBounds; 
*      DWORD   iGraphicsMode; 
*      FLOAT   exScale; 
*      FLOAT   eyScale;
*      EMRTEXT emrtext; 
*    } EMREXTTEXTOUTW;
****************************************************************************/

void TreeDecode::DecodeExtTextOutW(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();

     SetState(ctl, item, StateIcon::Drawing);
     AddFormattedItem(ctl, item, EMREXTTEXTOUTW, rec, RECTL, rclBounds, cvt);
     AddFormattedItem(ctl, item, EMREXTTEXTOUTW, rec, DWORD, iGraphicsMode, cvtGM);
     AddFormattedItem(ctl, item, EMREXTTEXTOUTW, rec, FLOAT, exScale, cvt);
     AddFormattedItem(ctl, item, EMREXTTEXTOUTW, rec, FLOAT, eyScale, cvt);
     HTREEITEM text = ctl->InsertItem(_T("EMRTEXT emrtext"), item);
     FormatEMRTextW(&((EMREXTTEXTOUTW*)rec)->emrtext, p1, (LPVOID)text, (LPCVOID)rec);
    } // TreeDecode::DecodeExtTextOutW

/****************************************************************************
*                         TreeDecode::DecodeFillPath
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(EMRFILLPATH*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEM) item to add to
* Result: void
*       
* Effect: 
*       Handles EMRFILLPATH
* Notes:
*    typedef struct {
*      EMR   emr; 
*      RECTL rclBounds; 
*    } EMRFILLPATH;
****************************************************************************/

void TreeDecode::DecodeFillPath(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();

     SetState(ctl, item, StateIcon::Drawing);
     AddFormattedItem(ctl, item, EMRFILLPATH, rec, RECTL, rclBounds, cvt);
    } // TreeDecode::DecodeFillPath

/****************************************************************************
*                          TreeDecode::DecodeFillRgn
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(EMRFILLRGN*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEM) item to add to
* Result: void
*       
* Effect: 
*       Handles EMRFILLRGN
* Notes:
*    typedef struct {
*      EMR   emr; 
*      RECTL rclBounds; 
*      DWORD cbRgnData; 
*      DWORD ihBrush; 
*      BYTE  RgnData[1]; 
*    } EMRFILLRGN
****************************************************************************/

void TreeDecode::DecodeFillRgn(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();

     SetState(ctl, item, StateIcon::Drawing);
     AddFormattedItem(ctl, item, EMRFILLRGN, rec, RECTL, rclBounds, cvt);
     AddFormattedItem(ctl, item, EMRFILLRGN, rec, DWORD, cbRgnData, cvt);
     AddFormattedItem(ctl, item, EMRFILLRGN, rec, DWORD, ihBrush,   cvt);
     HTREEITEM rgn = ctl->InsertItem(_T("BYTE RgnData[1]"), item);
     FormatRGNDATA((const RGNDATA *)&((EMRFILLRGN*)rec)->RgnData[0], p1, (LPVOID)rgn);
    } // TreeDecode::DecodeFillRgn

/****************************************************************************
*                           TreeDecode::DecodeFlattenPath
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(EMRFLATTENPATH*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEM) item to add to
* Result: void
*       
* Effect: 
*       Marks element as state
* Notes:
*    typedef struct { 
*      EMR emr; 
*    } EMRFLATTENPATH;
****************************************************************************/

void TreeDecode::DecodeFlattenPath(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();

     SetState(ctl, item, StateIcon::State);
    } // TreeDecode::DecodeFlattenPath

/****************************************************************************
*                          TreeDecode::DecodeFrameRgn
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(EMRFRAMERGN*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEM) item to add to
*       
* Result: void
*       
* Effect: 
*       Handles EMRFRAMERGN
* Notes:
*    typedef struct {
*      EMR   emr; 
*      RECTL rclBounds; 
*      DWORD cbRgnData; 
*      DWORD ihBrush; 
*      SIZEL szlStroke; 
*      BYTE  RgnData[1]; 
*    } EMRFRAMERGN
****************************************************************************/

void TreeDecode::DecodeFrameRgn(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();

     SetState(ctl, item, StateIcon::Drawing);
     AddFormattedItem(ctl, item, EMRFRAMERGN, rec, RECTL, rclBounds, cvt);
     AddFormattedItem(ctl, item, EMRFRAMERGN, rec, DWORD, cbRgnData, cvt);
     AddFormattedItem(ctl, item, EMRFRAMERGN, rec, DWORD, ihBrush,   cvt);
     AddFormattedItem(ctl, item, EMRFRAMERGN, rec, SIZEL, szlStroke, cvt);
     HTREEITEM rgn = ctl->InsertItem(_T("BYTE RgnData[1]"), item);
     FormatRGNDATA((const RGNDATA *)&((EMRFRAMERGN*)rec)->RgnData[0], p1, (LPVOID)rgn);
    } // TreeDecode::DecodeFrameRgn

/****************************************************************************
*                        TreeDecode::DecodeGdiComment
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(EMRGDICOMMENT*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEM) item to add to
* Result: void
*       
* Effect: 
*       Handles EMRGDICOMMENT
* Notes:
*    typedef struct {
*      EMR   emr; 
*      DWORD cbData; 
*      BYTE  Data[1]; 
*    } EMRGDICOMMENT
****************************************************************************/

void TreeDecode::DecodeGdiComment(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();

     SetState(ctl, item, StateIcon::State);
     
     AddFormattedItem(ctl, item, EMRGDICOMMENT, rec, DWORD, cbData, cvt);
     HTREEITEM data = ctl->InsertItem(_T("BYTE Data[1]"), item);
     FormatGdiCommentData(&((EMRGDICOMMENT*)rec)->Data[0], p1, (LPVOID)data, ((EMRGDICOMMENT*)rec)->cbData);
    } // TreeDecode::DecodeGdiComment

/****************************************************************************
*                         TreeDecode::DecodeGLSBoundedRecord
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(ENHGLSBOUNDEDRECORD*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEM) item to add to
* Result: void
*       
* Effect: 
*       Handles EMRGLSBOUNDEDRECORD
* Notes:
*    typedef struct {
*      EMR    emr;         // base structure
*      RECTL  rclBounds;   // bounds in recording coordinates
*      DWORD  cbData;      // size of Data[], in bytes
*      BYTE   Data[1]; 
*    } EMRGLSBOUNDEDRECORD
****************************************************************************/

void TreeDecode::DecodeGLSBoundedRecord(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();

     SetState(ctl, item, StateIcon::State);
     AddFormattedItem(ctl, item, EMRGLSBOUNDEDRECORD, rec, RECTL, rclBounds, cvt);
     AddFormattedItem(ctl, item, EMRGLSBOUNDEDRECORD, rec, DWORD, cbData, cvt);
     HTREEITEM data = ctl->InsertItem(_T("BYTE Data[1]"), item);
    } // TreeDecode::DecodeGLSRecord

/****************************************************************************
*                         TreeDecode::DecodeGLSRecord
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(ENHGLSRECORD*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEM) item to add to
* Result: void
*       
* Effect: 
*       Handles EMRGLSRECORD
* Notes:
*    typedef struct {
*      EMR    emr;         // base structure
*      DWORD  cbData;      // size of Data[], in bytes
*      BYTE   Data[1];
*    } EMRGLSRECORD
****************************************************************************/

void TreeDecode::DecodeGLSRecord(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();

     SetState(ctl, item, StateIcon::State);
     AddFormattedItem(ctl, item, EMRGLSRECORD, rec, DWORD, cbData, cvt);
     HTREEITEM data = ctl->InsertItem(_T("BYTE Data[1]"), item);
    } // TreeDecode::DecodeGLSRecord

/****************************************************************************
*                       TreeDecode::DecodeGradientFill
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(EMRGRADIENTFILL*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEM) item to add to
* Result: void
*       
* Effect: 
*       Handles EMRGRADIENTFILL
* Notes:
*    typedef struct { 
*      EMR       emr; 
*      RECTL     rclBounds;
*      DWORD     nVer; 
*      DWORD     nTri; 
*      ULONG     ulMode; 
*      TRIVERTEX Ver[1]; 
*    } EMRGRADIENTFILL
****************************************************************************/

void TreeDecode::DecodeGradientFill(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();

     SetState(ctl, item, StateIcon::Drawing);
     AddFormattedItem(ctl, item, EMRGRADIENTFILL, rec, RECTL, rclBounds, cvt);
     AddFormattedItem(ctl, item, EMRGRADIENTFILL, rec, DWORD, nVer, cvt);
     AddFormattedItem(ctl, item, EMRGRADIENTFILL, rec, DWORD, nTri, cvt);
     AddFormattedItem(ctl, item, EMRGRADIENTFILL, rec, ULONG, ulMode, cvtGF);
     HTREEITEM tv = ctl->InsertItem(_T("TRIVERTEX Ver[1]"), item);
     FormatTRIVERTEXes(&(((EMRGRADIENTFILL*)rec)->Ver[0]), p1, (LPVOID)tv, ((EMRGRADIENTFILL*)rec)->nVer);
     /*****************************************************************************
       +--------------+
       |    EMR emr   |
       :              :
       |              |
       +--------------+
       |  DWORD nVer  |
       +--------------+
       |  DWORD nTri  |
       +--------------+
   +---+ ULONG ulMode +---------------------------------------------------+
   |   +--------------+    +--------------+   - - - - - - - - - - - - - - | - - - 
   |   | TRIVERTEX    |    | TRIVERTEX[0] | \                             |   ^
   |   |     Ver[1]   |    |              |  |                            |   |
   |   +--------------+    +--------------+  |                            |   | &Ver + 
   |                       | TRIVERTEX[1] |  |                            |   | ((nVer - 1) * sizeof(TRIVERTEX))
   |                       |              |  |                            |   |
   |                       +--------------+  |                            |   |
   |                       |              |   > nVer                      |   |
   |                       :              :  |                            |   |
   |                       :              :  |                            |   |
   +->GRADIENT_FILL_RECT_H |              |  |                            |   |
      GRADIENT_FILL_RECT_V +--------------+  | GRADIENT_FILL_TRIANGLE <---+   |
               |           | TRIVERTEX    |  |        |                       |
               v           |     [nVer-1] | /         V                       V
       +--------------+    +--------------+    +--------------+ - - - - - - - - - - - base
       | GRADIENT_RECT|                        | GRADIENT_TRI-|
       |   [0]        |                        | ANGLE[0]     |
       +--------------+                        +--------------+
       |              |                        |              |
       :              :                        :              :
       |              |                        |              |
       +--------------+                        +--------------+
       | GRADIENT_RECT|                        | GRADIENT_TRI-|
       |  [nTri-1]    |                        | ANGLE[nTri-1]|
       +--------------+                        +--------------+

     *****************************************************************************/
     LPVOID base = (LPBYTE)(&((EMRGRADIENTFILL*)rec)->Ver[0]) + ((((EMRGRADIENTFILL*)rec)->nVer)) * sizeof(TRIVERTEX);
     HTREEITEM things;

     switch(((EMRGRADIENTFILL*)rec)->ulMode)
        { /* mode */
         case GRADIENT_FILL_RECT_H:
         case GRADIENT_FILL_RECT_V:
            things = ctl->InsertItem(_T("GRADIENT_RECTs"), tv);
            FormatGRADIENT_RECTs((LPGRADIENT_RECT)base, p1, (LPVOID)things, ((EMRGRADIENTFILL*)rec)->nTri);
            break;
         case GRADIENT_FILL_TRIANGLE:
            things = ctl->InsertItem(_T("GRADIENT_TRIANGLEs"), tv);
            FormatGRADIENT_TRIANGLEs((LPGRADIENT_TRIANGLE)base, p1, (LPVOID)things, ((EMRGRADIENTFILL*)rec)->nTri);
            break;
        } /* mode */
    } // TreeDecode::DecodeGradientFill

/****************************************************************************
*                          TreeDecode::DecodeHeader
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(ENHMETAHEADER*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEM) item to add to
* Result: void
*       
* Effect: 
*       Handles ENHMETAHEADER
* Notes:
*    typedef struct tagENHMETAHEADER { 
*      DWORD iType; 
*      DWORD nSize; 
*      RECTL rclBounds; 
*      RECTL rclFrame; 
*      DWORD dSignature; 
*      DWORD nVersion; 
*      DWORD nBytes; 
*      DWORD nRecords; 
*      WORD  nHandles; 
*      WORD  sReserved; 
*      DWORD nDescription; 
*      DWORD offDescription; 
*      DWORD nPalEntries; 
*      SIZEL szlDevice; 
*      SIZEL szlMillimeters;
*    #if (WINVER >= 0x0400)
*      DWORD cbPixelFormat;
*      DWORD offPixelFormat;
*      DWORD bOpenGL;
*    #endif // WINVER >= 0x0400
*    #if (WINVER >= 0x0500)
*      SIZEL szlMicrometers;
*    #endif // WINVER >= 0x0500
*    } ENHMETAHEADER
****************************************************************************/

void TreeDecode::DecodeHeader(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();
     DWORD ver = GetWinVer();

     SetState(ctl, item, StateIcon::State);
     AddFormattedItem(ctl, item, ENHMETAHEADER, rec, DWORD, iType, cvt);
     AddFormattedItem(ctl, item, ENHMETAHEADER, rec, DWORD, nSize, cvt);
     AddFormattedItem(ctl, item, ENHMETAHEADER, rec, RECTL, rclBounds, cvt);
     AddFormattedItem(ctl, item, ENHMETAHEADER, rec, RECTL, rclFrame, cvtRCmm);
     AddFormattedItem(ctl, item, ENHMETAHEADER, rec, DWORD, dSignature, cvtSig);
     AddItem(ctl, item, ENHMETAHEADER, rec, DWORD, nVersion, _T("0x%x"));
     AddFormattedItem(ctl, item, ENHMETAHEADER, rec, DWORD, nBytes, cvt);
     AddFormattedItem(ctl, item, ENHMETAHEADER, rec, DWORD, nRecords, cvt);
     AddFormattedItem(ctl, item, ENHMETAHEADER, rec, WORD, nHandles, cvt);
     AddFormattedItem(ctl, item, ENHMETAHEADER, rec, WORD, sReserved, cvt);
     AddFormattedItem(ctl, item, ENHMETAHEADER, rec, DWORD, nDescription, cvt);
     HTREEITEM last;
     AddItemEx(ctl, item, last, ENHMETAHEADER, rec, DWORD, offDescription, _T("%u"));
     int off = 0;
     if( ((ENHMETAHEADER*)rec)->offDescription != 0)
        { /* show description */
         off = ((ENHMETAHEADER*)rec)->offDescription;
         CStringW str = CStringW((LPWSTR)((LPBYTE)rec + off));
         CString s;
         s.Format(L"[%d] L\"%ls\"", off, str);
         ctl->InsertItem(s, last);
        } /* show description */
     AddFormattedItem(ctl, item, ENHMETAHEADER, rec, DWORD, nPalEntries, cvt);
     AddFormattedItem(ctl, item, ENHMETAHEADER, rec, SIZEL, szlDevice, cvt);
     AddFormattedItem(ctl, item, ENHMETAHEADER, rec, SIZEL, szlMillimeters, cvt);
     if(ver >= 0x0400 && offsetof(ENHMETAHEADER, cbPixelFormat) < ((ENHMETAHEADER*)rec)->offDescription)
        { /* 400 */
         AddFormattedItem(ctl, item, ENHMETAHEADER, rec, DWORD, cbPixelFormat, cvt);
         AddFormattedItem(ctl, item, ENHMETAHEADER, rec, DWORD, offPixelFormat, cvt);
         AddFormattedItem(ctl, item, ENHMETAHEADER, rec, DWORD, bOpenGL, cvt);
         if(ver >= 0x0500 && offsetof(ENHMETAHEADER, cbPixelFormat) < ((ENHMETAHEADER*)rec)->offDescription)
           { /* 500 */
            AddFormattedItem(ctl, item, ENHMETAHEADER, rec, SIZEL, szlMicrometers, cvt);
           } /* 500 */
        } /* 400 */
    } // TreeDecode::DecodeHeader

/****************************************************************************
*                         TreeDecode::DecodeIntersectClipRect
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(EMRINTERSECTCLIPRECT*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEM) item to add to
* Result: void
*       
* Effect: 
*       Handles EMRINTERSECTCLIPRECT
* Notes:
*    typedef struct tagEMREXCLUDECLIPRECT {
*      EMR   emr; 
*      RECTL rclClip; 
*    } EMRINTERSECTCLIPRECT;
****************************************************************************/

void TreeDecode::DecodeIntersectClipRect(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();

     SetState(ctl, item, StateIcon::State);
     AddFormattedItem(ctl, item, EMREXCLUDECLIPRECT, rec, RECTL, rclClip, cvt);
    } // TreeDecode::DecodeIntersectClipRect

/****************************************************************************
*                          TreeDecode::DecodeInvertRgn
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(EMRINVERTRGN*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEM) item to add to
* Result: void
*       
* Effect: 
*       Handles EMRFILLRGN
* Notes:
*    typedef struct {
*      EMR   emr; 
*      RECTL rclBounds; 
*      DWORD cbRgnData; 
*      BYTE  RgnData[1]; 
*    } EMRINVERTRGN;
****************************************************************************/

void TreeDecode::DecodeInvertRgn(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();

     SetState(ctl, item, StateIcon::Drawing);
     AddFormattedItem(ctl, item, EMRINVERTRGN, rec, RECTL, rclBounds, cvt);
     AddFormattedItem(ctl, item, EMRINVERTRGN, rec, DWORD, cbRgnData, cvt);
     HTREEITEM rgn = ctl->InsertItem(_T("BYTE RgnData[1]"), item);
     FormatRGNDATA((const RGNDATA *)&((EMRINVERTRGN*)rec)->RgnData[0], p1, (LPVOID)rgn);
    } // TreeDecode::DecodeInvertRgn

/****************************************************************************
*                          TreeDecode::DecodeLineTo
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(EMRLINETO*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEM) item to add to
* Result: void
*       
* Effect: 
*       Decodes an EMRLINETO
* Notes:
*    typedef struct {
*      EMR    emr; 
*      POINTL ptl; 
*    } EMRLINETO;
****************************************************************************/

void TreeDecode::DecodeLineTo(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();

     SetState(ctl, item, StateIcon::Drawing);
     AddFormattedItem(ctl, item, EMRLINETO, rec, POINTL, ptl, cvt);
    } // TreeDecode::DecodeLineTo

/****************************************************************************
*                      TreeDecode::DecodeMaskBlt
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(EMRMASKBLT*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEM) item to add to
* Result: void
*       
* Effect: 
*       Handles EMRMASKBLT
* Notes:
*    typedef struct {
*      EMR     emr; 
*      RECTL   rclBounds; 
*      LONG    xDest; 
*      LONG    yDest; 
*      LONG    cxDest; 
*      LONG    cyDest; 
*      DWORD   dwRop; 
*      LONG    xSrc; 
*      LONG    ySrc; 
*      XFORM   xformSrc; 
*      COLORREF crBkColorSrc; 
*      DWORD   iUsageSrc; 
*      DWORD   offBmiSrc; 
*      DWORD   cbBmiSrc; 
*      DWORD   offBitsSrc; 
*      DWORD   cbBitsSrc; 
*      LONG    xMask; 
*      LONG    yMask; 
*      DWORD   iUsageMask; 
*      DWORD   offBmiMask; 
*      DWORD   cbBmiMask; 
*      DWORD   offBitsMask; 
*      DWORD   cbBitsMask; 
*    } EMRMASKBLT
****************************************************************************/

void TreeDecode::DecodeMaskBlt(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();

     SetState(ctl, item, StateIcon::Drawing);
     
     AddFormattedItem(ctl, item, EMRMASKBLT, rec, RECTL, rclBounds, cvt);
     AddFormattedItem(ctl, item, EMRMASKBLT, rec, LONG, xDest, cvt);
     AddFormattedItem(ctl, item, EMRMASKBLT, rec, LONG, yDest, cvt);
     AddFormattedItem(ctl, item, EMRMASKBLT, rec, LONG, cxDest, cvt);
     AddFormattedItem(ctl, item, EMRMASKBLT, rec, LONG, cyDest, cvt);
     AddFormattedItem(ctl, item, EMRMASKBLT, rec, DWORD, dwRop, cvtROP);
     AddFormattedItem(ctl, item, EMRMASKBLT, rec, LONG, xSrc, cvt);
     AddFormattedItem(ctl, item, EMRMASKBLT, rec, LONG, ySrc, cvt);
     AddFormattedItem(ctl, item, EMRMASKBLT, rec, XFORM, xformSrc, cvt);
     AddFormattedItem(ctl, item, EMRMASKBLT, rec, COLORREF, crBkColorSrc, cvtCR);
     AddFormattedItem(ctl, item, EMRMASKBLT, rec, DWORD, iUsageSrc, cvtDBC);
     HTREEITEM bmi;
     AddFormattedItemEx(ctl, item, bmi, EMRMASKBLT, rec, DWORD, offBmiSrc, cvt);
     if(((EMRMASKBLT *)rec)->offBmiSrc != 0)
        FormatBITMAPINFO((LPBITMAPINFO)((LPBYTE)rec + ((EMRMASKBLT *)rec)->offBmiSrc), p1, (LPVOID)bmi);
     AddFormattedItem(ctl, item, EMRMASKBLT, rec, DWORD, cbBmiSrc, cvt);
     AddFormattedItem(ctl, item, EMRMASKBLT, rec, DWORD, offBitsSrc, cvt);
     AddFormattedItem(ctl, item, EMRMASKBLT, rec, DWORD, cbBitsSrc, cvt);
     // TODO: decode bitmap
     AddFormattedItem(ctl, item, EMRMASKBLT, rec, LONG, xMask, cvt);
     AddFormattedItem(ctl, item, EMRMASKBLT, rec, LONG, yMask, cvt);
     AddFormattedItem(ctl, item, EMRMASKBLT, rec, DWORD, iUsageMask, cvt);
     AddFormattedItemEx(ctl, item, bmi, EMRMASKBLT, rec, DWORD, offBmiMask, cvt);
     if(((EMRMASKBLT *)rec)->offBmiMask != 0)
        FormatBITMAPINFO((LPBITMAPINFO)((LPBYTE)rec + ((EMRMASKBLT *)rec)->offBmiMask), p1, (LPVOID)bmi);
     AddFormattedItem(ctl, item, EMRMASKBLT, rec, DWORD, offBitsMask, cvt);
     AddFormattedItem(ctl, item, EMRMASKBLT, rec, DWORD, cbBitsMask, cvt);
     // TODO: Decode bitmap
    } // TreeDecode::DecodeMaskBlt

/****************************************************************************
*                   TreeDecode::DecodeModifyWorldTransform
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(EMRMODIFYWORLDTRANSFORM*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEM) item to add to
* Result: void
*       
* Effect: 
*       Decodes EMRMODIFYWORLDTRANSFORM
* Notes:
*    typedef struct {
*      EMR   emr; 
*      XFORM xform; 
*      DWORD iMode; 
*       transformation
*    } EMRMODIFYWORLDTRANSFORM
****************************************************************************/

void TreeDecode::DecodeModifyWorldTransform(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();

     SetState(ctl, item, StateIcon::State);
     AddFormattedItem(ctl, item, EMRMODIFYWORLDTRANSFORM, rec, XFORM, xform, cvt);
     AddFormattedItem(ctl, item, EMRMODIFYWORLDTRANSFORM, rec, DWORD, iMode, cvtMWT);
    } // TreeDecode::DecodeModifyWorldTransform

/****************************************************************************
*                          TreeDecode::DecodeMoveToEx
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(EMRMOVETOEX*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEM) item to add to
* Result: void
*       
* Effect: 
*       Decodes an EMRMOVETOEX
* Notes:
*    typedef struct {
*      EMR    emr; 
*      POINTL ptl; 
*    } EMRMOVETOEX
****************************************************************************/

void TreeDecode::DecodeMoveToEx(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();

     SetState(ctl, item, StateIcon::State);
     AddFormattedItem(ctl, item, EMRMOVETOEX, rec, POINTL, ptl, cvt);
    } // TreeDecode::DecodeMoveToEx

/****************************************************************************
*                       TreeDecode::DecodeOffsetClipRgn
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(EMROFFSETCLIPRGN*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEM) item to add to
* Result: void
*
* Effect:
*       Handles EMROFFSETCLIPRGN
* Notes:
*    typedef struct {
*      EMR    emr; 
*      POINTL ptlOffset; 
*    } EMROFFSETCLIPRGN;
****************************************************************************/

void TreeDecode::DecodeOffsetClipRgn(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();

     SetState(ctl, item, StateIcon::State);
     AddFormattedItem(ctl, item, EMROFFSETCLIPRGN, rec, POINTL, ptlOffset, cvt);
    } // TreeDecode::DecodeOffsetClipRgn

/****************************************************************************
*                          TreeDecode::DecodePaintRgn
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(EMRFILLRGN*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEM) item to add to
* Result: void
*       
* Effect: 
*       Handles EMRPAINTRGN
* Notes:
*    typedef struct tagEMRINVERTRGN {
*      EMR   emr; 
*      RECTL rclBounds; 
*      DWORD cbRgnData; 
*      BYTE  RgnData[1]; 
*    } EMRPAINTRGN;
****************************************************************************/

void TreeDecode::DecodePaintRgn(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();

     SetState(ctl, item, StateIcon::Drawing);
     AddFormattedItem(ctl, item, EMRPAINTRGN, rec, RECTL, rclBounds, cvt);
     AddFormattedItem(ctl, item, EMRPAINTRGN, rec, DWORD, cbRgnData, cvt);
     HTREEITEM rgn = ctl->InsertItem(_T("BYTE RgnData[1]"), item);
     FormatRGNDATA((const RGNDATA *)&((EMRPAINTRGN*)rec)->RgnData[0], p1, (LPVOID)rgn);
    } // TreeDecode::DecodePaintRgn

/****************************************************************************
*                            TreeDecode::DecodePie
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(EMRPIE*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEM) item to add to
* Result: void
*       
* Effect: 
*       Handles EMRPIE
* Notes:
*    typedef struct {
*      EMR    emr; 
*      RECTL  rclBox;
*      POINTL ptlStart;
*      POINTL ptlEnd;
*    } EMRPIE;
****************************************************************************/

void TreeDecode::DecodePie(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();

     SetState(ctl, item, StateIcon::Drawing);
     AddFormattedItem(ctl, item, EMRPIE, rec, RECTL, rclBox, cvt);
     AddFormattedItem(ctl, item, EMRPIE, rec, POINTL, ptlStart, cvt);
     AddFormattedItem(ctl, item, EMRPIE, rec, POINTL, ptlEnd, cvt);
    } // TreeDecode::DecodePie

/****************************************************************************
*                        TreeDecode::DecodePixelFormat
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(EMRPIXELFORMAT*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEweM) item to add to
* Result: void
*       
* Effect: 
*       Handles EMRPIXELFORMAT
* Notes:
*    typedef struct {
*      EMR         emr; 
*      PIXELFORMATDESCRIPTOR  pfd; 
*    } EMRPIXELFORMAT;
****************************************************************************/

void TreeDecode::DecodePixelFormat(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();

     SetState(ctl, item, StateIcon::State);
     HTREEITEM pf = ctl->InsertItem(_T("PIXELFORMATDESCRIPTOR pfd"), item);
     FormatPIXELFORMATDESCRIPTOR(&(((EMRPIXELFORMAT*)rec)->pfd), p1, (LPVOID)pf);
    } // TreeDecode::DecodePixelFormat

/****************************************************************************
*                      TreeDecode::DecodePlgBlt
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(EMRPLGBLT*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEM) item to add to
* Result: void
*       
* Effect: 
*       Handles EMRPLGBLT
* Notes:
*    typedef struct {
*      EMR      emr; 
*      RECTL    rclBounds; 
*      POINTL   aptlDest[3]; 
*      LONG    xSrc; 
*      LONG    ySrc; 
*      LONG     cxSrc; 
*      LONG     cySrc; 
*      XFORM   xformSrc; 
*      COLORREF crBkColorSrc; 
*      DWORD    iUsageSrc; 
*      DWORD    offBmiSrc; 
*      DWORD   cbBmiSrc; 
*      DWORD   offBitsSrc; 
*      DWORD   cbBitsSrc; 
*      LONG    xMask;
*      LONG    yMask; 
*      DWORD   iUsageMask; 
*      DWORD   offBmiMask; 
*      DWORD   cbBmiMask;
*      DWORD   offBitsMask;
*      DWORD   cbBitsMask;
*    } EMRPLGBLT;
*****************************************************************************/

void TreeDecode::DecodePlgBlt(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();

     SetState(ctl, item, StateIcon::Drawing);
     
     AddFormattedItem(ctl, item, EMRPLGBLT, rec, RECTL, rclBounds, cvt);
     AddFormattedItem(ctl, item, EMRPLGBLT, rec, POINTL, aptlDest[0], cvt);
     AddFormattedItem(ctl, item, EMRPLGBLT, rec, POINTL, aptlDest[1], cvt);
     AddFormattedItem(ctl, item, EMRPLGBLT, rec, POINTL, aptlDest[2], cvt);
     AddFormattedItem(ctl, item, EMRPLGBLT, rec, LONG, xSrc, cvt);
     AddFormattedItem(ctl, item, EMRPLGBLT, rec, LONG, ySrc, cvt);
     AddFormattedItem(ctl, item, EMRPLGBLT, rec, LONG, cxSrc, cvt);
     AddFormattedItem(ctl, item, EMRPLGBLT, rec, LONG, cySrc, cvt);
     AddFormattedItem(ctl, item, EMRPLGBLT, rec, XFORM, xformSrc, cvt);
     AddFormattedItem(ctl, item, EMRPLGBLT, rec, COLORREF, crBkColorSrc, cvtCR);
     AddFormattedItem(ctl, item, EMRPLGBLT, rec, DWORD, iUsageSrc, cvtDBC);
     HTREEITEM bmi;
     AddFormattedItemEx(ctl, item, bmi, EMRPLGBLT, rec, DWORD, offBmiSrc, cvt);
     if(((EMRPLGBLT *)rec)->offBmiSrc != 0)
        FormatBITMAPINFO((LPBITMAPINFO)((LPBYTE)rec + ((EMRPLGBLT *)rec)->offBmiSrc), p1, (LPVOID)bmi);
     AddFormattedItem(ctl, item, EMRPLGBLT, rec, DWORD, cbBmiSrc, cvt);
     AddFormattedItem(ctl, item, EMRPLGBLT, rec, DWORD, offBitsSrc, cvt);
     AddFormattedItem(ctl, item, EMRPLGBLT, rec, DWORD, cbBitsSrc, cvt);
     // TODO: decode bitmap
     AddFormattedItem(ctl, item, EMRPLGBLT, rec, LONG,  xMask, cvt);
     AddFormattedItem(ctl, item, EMRPLGBLT, rec, LONG, yMask, cvt);
     AddFormattedItem(ctl, item, EMRPLGBLT, rec, DWORD, iUsageMask, cvt);
     AddFormattedItemEx(ctl, item, bmi, EMRPLGBLT, rec, DWORD, offBmiMask, cvt);
     if(((EMRPLGBLT *)rec)->offBmiMask != 0)
        FormatBITMAPINFO((LPBITMAPINFO)((LPBYTE)rec + ((EMRPLGBLT *)rec)->offBmiMask), p1, (LPVOID)bmi);
     AddFormattedItem(ctl, item, EMRPLGBLT, rec, DWORD, cbBitsMask, cvt);
    } // TreeDecode::DecodePlgBlt

/****************************************************************************
*                          TreeDecode::DecodePolyBezier
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(EMRPOLYBEZIER*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEM) item to add to
* Result: void
*       
* Effect: 
*       Decodes EMRPOLYBEZIER
* Notes:
*    typedef struct tagEMRPOLYLINE {
*      EMR    emr;
*      RECTL  rclBounds; 
*      DWORD  cptl; 
*      POINTL aptl[1]; 
*    } EMRPOLYBEZIER;
*****************************************************************************/

void TreeDecode::DecodePolyBezier(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();

     SetState(ctl, item, StateIcon::Drawing);
     AddFormattedItem(ctl, item, EMRPOLYBEZIER, rec, RECTL, rclBounds, cvt);
     AddFormattedItem(ctl, item, EMRPOLYBEZIER, rec, DWORD, cptl, cvt);
     HTREEITEM points = ctl->InsertItem(_T("POINTL sptl[1]"), item);
     FormatPOINTL(&(((EMRPOLYBEZIER*)rec)->aptl[0]), p1, (LPVOID)points, ((EMRPOLYBEZIER*)rec)->cptl);
    } // TreeDecode::DecodePolyBezier

/****************************************************************************
*                          TreeDecode::DecodePolyBezier16
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(EMRPOLYBEZIER16*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEM) item to add to
* Result: void
*       
* Effect: 
*       Decodes EMRPOLYBEZIER16
* Notes:
*    typedef struct {
*      EMR    emr; 
*      RECTL  rclBounds; 
*      DWORD  cpts; 
*      POINTS apts[1]; 
*    } EMRPOLYBEZIER16;
****************************************************************************/

void TreeDecode::DecodePolyBezier16(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();

     SetState(ctl, item, StateIcon::Drawing);
     AddFormattedItem(ctl, item, EMRPOLYBEZIER16, rec, RECTL, rclBounds, cvt);
     AddFormattedItem(ctl, item, EMRPOLYBEZIER16, rec, DWORD, cpts, cvt);
     HTREEITEM points = ctl->InsertItem(_T("POINTS apts[1]"), item);
     FormatPOINTS(&(((EMRPOLYBEZIER16*)rec)->apts[0]), p1, (LPVOID)points, ((EMRPOLYBEZIER16*)rec)->cpts);
    } // TreeDecode::DecodePolyBezier16

/****************************************************************************
*                          TreeDecode::DecodePolyBezierTo
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(EMRPOLYBEZIERTO*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEM) item to add to
* Result: void
*       
* Effect: 
*       Decodes EMRPOLYBEZIERTO
* Notes:
*    typedef struct {
*      EMR    emr;
*      RECTL  rclBounds; 
*      DWORD  cptl; 
*      POINTL aptl[1]; 
*    } EMRPOLYBEZIERTO;
****************************************************************************/

void TreeDecode::DecodePolyBezierTo(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();

     SetState(ctl, item, StateIcon::Drawing);
     AddFormattedItem(ctl, item, EMRPOLYBEZIERTO, rec, RECTL, rclBounds, cvt);
     AddFormattedItem(ctl, item, EMRPOLYBEZIERTO, rec, DWORD, cptl, cvt);
     HTREEITEM points = ctl->InsertItem(_T("POINTL sptl[1]"), item);
     FormatPOINTL(&(((EMRPOLYBEZIERTO*)rec)->aptl[0]), p1, (LPVOID)points, ((EMRPOLYBEZIERTO*)rec)->cptl );
    } // TreeDecode::DecodePolyBezierTo

/****************************************************************************
*                          TreeDecode::DecodePolyBezierTo16
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(EMRPOLYBEZIERTO16*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEM) item to add to
* Result: void
*       
* Effect: 
*       Decodes EMRPOLYBEZIERTO16
* Notes:
*    typedef struct {
*      EMR    emr; 
*      RECTL  rclBounds; 
*      DWORD  cpts; 
*      POINTS apts[1]; 
*    } EMRPOLYBEZIERTO16;
****************************************************************************/

void TreeDecode::DecodePolyBezierTo16(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();

     SetState(ctl, item, StateIcon::Drawing);
     AddFormattedItem(ctl, item, EMRPOLYBEZIERTO16, rec, RECTL, rclBounds, cvt);
     AddFormattedItem(ctl, item, EMRPOLYBEZIERTO16, rec, DWORD, cpts, cvt);
     HTREEITEM points = ctl->InsertItem(_T("POINTS apts[1]"), item);
     FormatPOINTS(&(((EMRPOLYBEZIERTO16*)rec)->apts[0]), p1, (LPVOID)points, ((EMRPOLYBEZIERTO16*)rec)->cpts);
    } // TreeDecode::DecodePolyBezierTo16

/****************************************************************************
*                         TreeDecode::DecodePolyDraw
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(EMRPOLYDRAW*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEM) item to add to
* Result: void
*       
* Effect: 
*       Handles EMRPOLYDRAW
* Notes:
*    typedef struct {
*      EMR    emr; 
*      RECTL  rclBounds; 
*      DWORD  cptl; 
*      POINTL aptl[1]; 
*      BYTE   abTypes[1]; 
*    } EMRPOLYDRAW;
* Notes:
*      +-------------+
*      |     emr     |
*      |             |
*      +-------------+-----------+
*      |  rclBounds  | .left     |
*      |             | .top      |
*      |             | .right    |
*      |             | .bottom   |
*      +-------------+-----------+
*      |    cptl     |
*      +-------------+-----------------+  - - - - - - - - - - 
*      |   aptl      | aptl[0]         |                  ^
*      +-------------+-----------------+                  |
*                    | aplt[1]         |                  |
*                    +-----------------+                  |
*                    |                 |                  | cptl * sizeof(POINTL)
*                    :                 :                  |
*                    |                 |                  |
*                    +-----------------+                  |
*                    | aplt[cptl-1]    |                  v
*                    +-----------+-----+  - - - - - - - - - - &aptl[0] + cptl * sizeof(POINTL)
*                    | ty[0]     |                        ^
*                    +-----------+                        |
*                    |           |                        |
*                    :           :                        |   cptl * sizeof(BYTE)
*                    |           |                        |
*                    +-----------+                        |
*                    | ty[cptl-1]|                        v
*                    +-----------+  - - - - - - - - - - - - -
****************************************************************************/

void TreeDecode::DecodePolyDraw(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();

     SetState(ctl, item, StateIcon::Drawing);
     AddFormattedItem(ctl, item, EMRPOLYDRAW, rec, RECTL, rclBounds, cvt);
     AddFormattedItem(ctl, item, EMRPOLYDRAW, rec, DWORD, cptl, cvt);
     HTREEITEM points = ctl->InsertItem(_T("POINTL aptl[1]"), item);
     FormatPOINTL(&(((EMRPOLYDRAW*)rec)->aptl[0]), p1, (LPVOID)points, ((EMRPOLYDRAW*)rec)->cptl );
     LPBYTE data = ((LPBYTE)&(((EMRPOLYDRAW*)rec)->aptl[0])) + (((EMRPOLYDRAW*)rec)->cptl) * sizeof(POINTL);
     HTREEITEM types = ctl->InsertItem(_T("BYTE abTypes[1]"));
     FormatDrawTypes(data, p1, (LPVOID)types, ((EMRPOLYDRAW*)rec)->cptl);
    } // TreeDecode::DecodePolyDraw

/****************************************************************************
*                         TreeDecode::DecodePolyDraw16
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(EMRPOLYDRAW16*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEM) item to add to
* Result: void
*       
* Effect: 
*       Handles EMRPOLYDRAW16
* Notes:
*    typedef struct {
*      EMR    emr; 
*      RECTL  rclBounds; 
*      DWORD  cptl; 
*      POINTS apts[1]; 
*      BYTE   abTypes[1]; 
*    } EMRPOLYDRAW16;
* Notes:
*      +-------------+
*      |     emr     |
*      |             |
*      +-------------+-----------+
*      |  rclBounds  | .left     |
*      |             | .top      |
*      |             | .right    |
*      |             | .bottom   |
*      +-------------+-----------+
*      |    cpts     |
*      +-------------+-----------------+  - - - - - - - - - - 
*      |    apts     | apts[0]         |                  ^
*      +-------------+-----------------+                  |
*                    | apts[1]         |                  |
*                    +-----------------+                  |
*                    |                 |                  | cpts * sizeof(POINTS)
*                    :                 :                  |
*                    |                 |                  |
*                    +-----------------+                  |
*                    | apts[cpts-1]    |                  v
*                    +-----------+-----+  - - - - - - - - - - &apts[0] + cpts * sizeof(POINTS)
*                    | ty[0]     |                        ^
*                    +-----------+                        |
*                    |           |                        |
*                    :           :                        |   cpts * sizeof(BYTE)
*                    |           |                        |
*                    +-----------+                        |
*                    | ty[cpts-1]|                        v
*                    +-----------+  - - - - - - - - - - - - -
****************************************************************************/

void TreeDecode::DecodePolyDraw16(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();

     SetState(ctl, item, StateIcon::Drawing);
     AddFormattedItem(ctl, item, EMRPOLYDRAW16, rec, RECTL, rclBounds, cvt);
     AddFormattedItem(ctl, item, EMRPOLYDRAW16, rec, DWORD, cpts, cvt);
     HTREEITEM points = ctl->InsertItem(_T("POINTS apts[1]"), item);
     FormatPOINTS(&(((EMRPOLYDRAW16*)rec)->apts[0]), p1, (LPVOID)points, ((EMRPOLYDRAW16*)rec)->cpts );
     LPBYTE data = ((LPBYTE)&(((EMRPOLYDRAW16*)rec)->apts[0])) + (((EMRPOLYDRAW16*)rec)->cpts) * sizeof(POINTS);
     HTREEITEM types = ctl->InsertItem(_T("BYTE abTypes[1]"), points);
     FormatDrawTypes(data, p1, (LPVOID)types, ((EMRPOLYDRAW16*)rec)->cpts);
    } // TreeDecode::DecodePolyDraw

/****************************************************************************
*                          TreeDecode::DecodePolygon
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(EMRPOLYLINE*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEM) item to add to
* Result: void
*       
* Effect: 
*       Decodes EMRPOLYGON
* Notes:
*    typedef struct {
*      EMR    emr;
*      RECTL  rclBounds; 
*      DWORD  cptl; 
*      POINTL aptl[1]; 
*    } EMRPOLYGON;
****************************************************************************/

void TreeDecode::DecodePolygon(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();

     SetState(ctl, item, StateIcon::Drawing);
     AddFormattedItem(ctl, item, EMRPOLYGON, rec, RECTL, rclBounds, cvt);
     AddFormattedItem(ctl, item, EMRPOLYGON, rec, DWORD, cptl, cvt);
     HTREEITEM points = ctl->InsertItem(_T("POINTL aptl[1]"), item);
     FormatPOINTL(&(((EMRPOLYGON*)rec)->aptl[0]), p1, (LPVOID)points, ((EMRPOLYGON*)rec)->cptl);
    } // TreeDecode::DecodePolygon

/****************************************************************************
*                          TreeDecode::DecodePolygon16
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(EMRPOLYGON16*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEM) item to add to
* Result: void
*       
* Effect: 
*       Decodes EMRPOLYGON
* Notes:
*    typedef struct {
*      EMR    emr; 
*      RECTL  rclBounds; 
*      DWORD  cpts; 
*      POINTS apts[1]; 
*    } EMRPOLYGON16;
****************************************************************************/

void TreeDecode::DecodePolygon16(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();

     SetState(ctl, item, StateIcon::Drawing);
     AddFormattedItem(ctl, item, EMRPOLYGON16, rec, RECTL, rclBounds, cvt);
     AddFormattedItem(ctl, item, EMRPOLYGON16, rec, DWORD, cpts, cvt);
     HTREEITEM points = ctl->InsertItem(_T("POINTL apts[1]"), item);
     FormatPOINTS(&(((EMRPOLYGON16*)rec)->apts[0]), p1, (LPVOID)points, ((EMRPOLYGON16*)rec)->cpts);
    } // TreeDecode::DecodePolygon16

/****************************************************************************
*                          TreeDecode::DecodePolyLine
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(EMRPOLYLINE*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEM) item to add to
* Result: void
*       
* Effect: 
*       Decodes EMRPOLYLINE
* Notes:
*    typedef struct {
*      EMR    emr;
*      RECTL  rclBounds; 
*      DWORD  cptl; 
*      POINTL aptl[1]; 
*    } EMRPOLYLINE;
****************************************************************************/

void TreeDecode::DecodePolyLine(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();

     SetState(ctl, item, StateIcon::Drawing);
     AddFormattedItem(ctl, item, EMRPOLYLINE, rec, RECTL, rclBounds, cvt);
     AddFormattedItem(ctl, item, EMRPOLYLINE, rec, DWORD, cptl, cvt);
     HTREEITEM points = ctl->InsertItem(_T("POINTL sptl[1]"), item);
     FormatPOINTL(&(((EMRPOLYLINE*)rec)->aptl[0]), p1, (LPVOID)points, ((EMRPOLYLINE*)rec)->cptl);
    } // TreeDecode::DecodePolyLine

/****************************************************************************
*                          TreeDecode::DecodePolyLine16
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(EMRPOLYLINE16*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEM) item to add to
* Result: void
*       
* Effect: 
*       Decodes EMRPOLYLINE16
* Notes:
*    typedef struct {
*      EMR    emr; 
*      RECTL  rclBounds; 
*      DWORD  cpts; 
*      POINTS apts[1]; 
*    } EMRPOLYLINETO16;
****************************************************************************/

void TreeDecode::DecodePolyLine16(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();

     SetState(ctl, item, StateIcon::Drawing);
     AddFormattedItem(ctl, item, EMRPOLYLINE16, rec, RECTL, rclBounds, cvt);
     AddFormattedItem(ctl, item, EMRPOLYLINE16, rec, DWORD, cpts, cvt);
     HTREEITEM points = ctl->InsertItem(_T("POINTS apts[1]"), item);
     FormatPOINTS(&(((EMRPOLYLINE16*)rec)->apts[0]), p1, (LPVOID)points, ((EMRPOLYLINE16*)rec)->cpts);
    } // TreeDecode::DecodePolyLine16

/****************************************************************************
*                       TreeDecode::DecodePolyLineTo
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(EMRPOLYLINETO*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEM) item to add to
* Result: void
*       
* Effect: 
*       Decodes EMRPOLYLINETO
* Notes:
*    typedef struct {
*      EMR    emr;
*      RECTL  rclBounds; 
*      DWORD  cptl; 
*      POINTL aptl[1]; 
*    } EMRPOLYLINETO;
****************************************************************************/

void TreeDecode::DecodePolyLineTo(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();

     SetState(ctl, item, StateIcon::Drawing);
     AddFormattedItem(ctl, item, EMRPOLYLINETO, rec, RECTL, rclBounds, cvt);
     AddFormattedItem(ctl, item, EMRPOLYLINETO, rec, DWORD, cptl, cvt);
     HTREEITEM points = ctl->InsertItem(_T("POINTL sptl[1]"), item);
     FormatPOINTL(&(((EMRPOLYLINETO*)rec)->aptl[0]), p1, (LPVOID)points, ((EMRPOLYLINETO*)rec)->cptl);
    } // TreeDecode::DecodePolyLineTo

/****************************************************************************
*                          TreeDecode::DecodePolyLineTo16
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(EMRPOLYLINETO16*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEM) item to add to
* Result: void
*       
* Effect: 
*       Decodes EMRPOLYLINETO16
* Notes:
*    typedef struct {
*      EMR    emr; 
*      RECTL  rclBounds; 
*      DWORD  cpts; 
*      POINTS apts[1]; 
*    } EMRPOLYLINETO16;
****************************************************************************/

void TreeDecode::DecodePolyLineTo16(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();

     SetState(ctl, item, StateIcon::Drawing);
     AddFormattedItem(ctl, item, EMRPOLYLINETO16, rec, RECTL, rclBounds, cvt);
     AddFormattedItem(ctl, item, EMRPOLYLINETO16, rec, DWORD, cpts, cvt);
     HTREEITEM points = ctl->InsertItem(_T("POINTS apts[1]"), item);
     FormatPOINTS(&(((EMRPOLYLINETO16*)rec)->apts[0]), p1, (LPVOID)points, ((EMRPOLYLINETO16*)rec)->cpts);
    } // TreeDecode::DecodePolyLineTo16

/****************************************************************************
*                         TreeDecode::DecodePolyPolygon
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(EMRPOLYPOLYGON*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEM) item to add to
* Result: void
*       
* Effect: 
*       Handles EMRPOLYPOLYGON
* Notes:
*    typedef struct tagEMRPOLYPOLYLINE {
*      EMR     emr; 
*      RECTL   rclBounds; 
*      DWORD   nPolys; 
*      DWORD   cptl; 
*      DWORD   aPolyCounts[1]; 
*      POINTL  aptl[1]; 
*    } EMRPOLYPOLYGON;
*
*    +----------------------+ - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
*    | EMR emr              |                                                                       ^
*    :                      :                                                                       |
*    |                      |                                                                       |
*    +----------------------+-------------+                                                         |
*    | RECTL                | LONG left   |                                                         |
*    |  rclBounds           | LONG top    |                                                         |
*    |                      | LONG right  |                                                         |
*    |                      | LONG bottom |                                                         |
*    +----------------------+-------------+                                                         |
*    | DWORD nPolys         |                                                                       | delta
*    +----------------------+                                                                       |
*    | DWORD cptl           |                                                                       |
*    +----------------------+----------------------+                                                |
*    | DWORD aPolyCounts[1] | aPolyCounts[0]       | \                                              |
*    +----------------------+----------------------+  |                                             |
*    | DWORD aplt[1]        | aPolyCounts[1]       |  |                                             |
*    +----------------------+----------------------+  |                                             |
*                           | aPolyCounts[2]       |  |                                             |
*                           +----------------------+   > nPolys                                     |
*                           |                      |  |                                             |
*                           :                      :  |                                             |
*                           |                      |  |                                             |
*                           +----------------------+  |                                             |
*                           | aPolyCounts[nPolys-1]| /                                              V
*                           +----------------------+------------------+ - - - - - - - - - - - - - - - - -
*                                                  | POINTL [0]       | \
*                                                  +------------------+  |
*                                                  | POINTL [1]       |  |
*                                                  +------------------+  |
*                                                  |                  |   > cptl
*                                                  :                  :  |
*                                                  |                  |  |
*                                                  +------------------+  |
*                                                  | POINTL [cptl-1]  | /   
*                                                  +------------------+
*
*  POINTL * points = base + offsetof(aPolyCounts) + (nPolys - 1) * sizeof(DWORD)
****************************************************************************/

void TreeDecode::DecodePolyPolygon(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();
     
     SetState(ctl, item, StateIcon::Drawing);
     AddFormattedItem(ctl, item, EMRPOLYPOLYGON, rec, RECTL, rclBounds, cvt);
     AddFormattedItem(ctl, item, EMRPOLYPOLYGON, rec, DWORD, nPolys, cvt);
     AddFormattedItem(ctl, item, EMRPOLYPOLYGON, rec, DWORD, cptl, cvt);
     HTREEITEM count = ctl->InsertItem(_T("DWORD aPolyCounts[1]"), item);
     FormatDWORDs(&(((EMRPOLYPOLYGON*)rec)->aPolyCounts[0]), p1, (LPVOID)count, ((EMRPOLYPOLYGON*)rec)->nPolys);
     HTREEITEM pointl = ctl->InsertItem(_T("POINTL aptl[1]"), item);
     POINTL * pts = (POINTL*)(
                              ((LPBYTE)rec) +
                              offsetof(EMRPOLYPOLYGON, aPolyCounts) +
                              (((EMRPOLYPOLYGON*)rec)->nPolys) * sizeof(DWORD)
                             );
     FormatPOINTL(pts, p1, (LPVOID)pointl, ((EMRPOLYPOLYGON*)rec)->cptl);
    } // TreeDecode::DecodePolyPolygon

/****************************************************************************
*                         TreeDecode::DecodePolyPolygon16
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(EMRPOLYPOLYGON16*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEM) item to add to
* Result: void
*       
* Effect: 
*       Handles EMRPOLPOLYGON16
* Notes:
*    typedef struct {
*      EMR     emr; 
*      RECTL   rclBounds;
*      DWORD   nPolys; 
*      DWORD   cpts; 
*      DWORD   aPolyCounts[1]; 
*      POINTS  apts[1]; 
*    } EMRPOLYPOLYGON16;
****************************************************************************/

void TreeDecode::DecodePolyPolygon16(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();
     
     SetState(ctl, item, StateIcon::Drawing);
     AddFormattedItem(ctl, item, EMRPOLYPOLYGON16, rec, RECTL, rclBounds, cvt);
     AddFormattedItem(ctl, item, EMRPOLYPOLYGON16, rec, DWORD, nPolys, cvt);
     AddFormattedItem(ctl, item, EMRPOLYPOLYGON16, rec, DWORD, cpts, cvt);
     HTREEITEM count = ctl->InsertItem(_T("DWORD aPolyCounts[1]"), item);
     FormatDWORDs(&(((EMRPOLYPOLYGON16*)rec)->aPolyCounts[0]), p1, (LPVOID)count, ((EMRPOLYPOLYGON16*)rec)->nPolys);
     HTREEITEM points = ctl->InsertItem(_T("POINTS apts[1]"), item);
     POINTS * pts = (POINTS*)(
                              ((LPBYTE)rec) +
                              offsetof(EMRPOLYPOLYGON, aPolyCounts) +
                              (((EMRPOLYPOLYGON*)rec)->nPolys) * sizeof(DWORD)
                             );
     FormatPOINTS(pts, p1, (LPVOID)points, ((EMRPOLYPOLYGON16*)rec)->cpts);
    } // TreeDecode::DecodePolyPolygon16

/****************************************************************************
*                         TreeDecode::DecodePolyPolyLine
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(EMRPOLYPOLYLINE*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEM) item to add to
* Result: void
*       
* Effect: 
*       Handles EMRPOLYPOLYLINE
* Notes:
*    typedef struct {
*      EMR     emr; 
*      RECTL   rclBounds; 
*      DWORD   nPolys; 
*      DWORD   cptl; 
*      DWORD   aPolyCounts[1]; 
*      POINTL  aptl[1]; 
*    } EMRPOLYPOLYLINE;
****************************************************************************/

void TreeDecode::DecodePolyPolyLine(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();
     
     SetState(ctl, item, StateIcon::Drawing);
     AddFormattedItem(ctl, item, EMRPOLYPOLYLINE, rec, RECTL, rclBounds, cvt);
     AddFormattedItem(ctl, item, EMRPOLYPOLYLINE, rec, DWORD, nPolys, cvt);
     AddFormattedItem(ctl, item, EMRPOLYPOLYLINE, rec, DWORD, cptl, cvt);
     HTREEITEM count = ctl->InsertItem(_T("DWORD aPolyCounts[1]"), item);
     FormatDWORDs(&(((EMRPOLYPOLYLINE*)rec)->aPolyCounts[0]), p1, (LPVOID)count, ((EMRPOLYPOLYLINE*)rec)->nPolys);
     HTREEITEM pointl = ctl->InsertItem(_T("POINTL aptl[1]"), item);
     FormatPOINTL(&(((EMRPOLYPOLYLINE*)rec)->aptl[0]), p1, (LPVOID)pointl, ((EMRPOLYPOLYLINE*)rec)->cptl);
    } // TreeDecode::DecodePolyPolyLine

/****************************************************************************
*                         TreeDecode::DecodePolyPolyLine16
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(EMRPOLYPOLYLINE16*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEM) item to add to
* Result: void
*       
* Effect: 
*       Handles EMRPOLYPOLYLINE16
* Notes:
*    typedef struct {
*      EMR     emr; 
*      RECTL   rclBounds;
*      DWORD   nPolys; 
*      DWORD   cpts; 
*      DWORD   aPolyCounts[1]; 
*      POINTS  apts[1]; 
*    } EMRPOLYPOLYLINE16;
****************************************************************************/

void TreeDecode::DecodePolyPolyLine16(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();
     
     SetState(ctl, item, StateIcon::Drawing);
     AddFormattedItem(ctl, item, EMRPOLYPOLYLINE16, rec, RECTL, rclBounds, cvt);
     AddFormattedItem(ctl, item, EMRPOLYPOLYLINE16, rec, DWORD, nPolys, cvt);
     AddFormattedItem(ctl, item, EMRPOLYPOLYLINE16, rec, DWORD, cpts, cvt);
     HTREEITEM count = ctl->InsertItem(_T("DWORD aPolyCounts[1]"), item);
     FormatDWORDs(&(((EMRPOLYPOLYLINE16*)rec)->aPolyCounts[0]), p1, (LPVOID)count, ((EMRPOLYPOLYLINE16*)rec)->nPolys);
     HTREEITEM points = ctl->InsertItem(_T("POINTS apts[1]"), item);
     FormatPOINTS(&(((EMRPOLYPOLYLINE16*)rec)->apts[0]), p1, (LPVOID)points, ((EMRPOLYPOLYLINE16*)rec)->cpts);
    } // TreeDecode::DecodePolyPolyLine16

/****************************************************************************
*                       TreeDecode::DecodePolyTextOutA
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(EMRPOLYTEXTOUTA*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEM) item to add to
* Result: void
*       
* Effect: 
*       Handles EMRPOLYTEXTOUTA
* Notes:
*    typedef struct {
*      EMR     emr; 
*      RECTL   rclBounds; 
*      DWORD   iGraphicsMode; 
*      FLOAT   exScale; 
*      FLOAT   eyScale; 
*      LONG    cStrings; 
*      EMRTEXT aemrtext[1]; 
*    } EMRPOLYTEXTOUTA
****************************************************************************/

void TreeDecode::DecodePolyTextOutA(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();

     SetState(ctl, item, StateIcon::Drawing);
     AddFormattedItem(ctl, item, EMRPOLYTEXTOUTA, rec, RECTL, rclBounds, cvt);
     AddFormattedItem(ctl, item, EMRPOLYTEXTOUTA, rec, DWORD, iGraphicsMode, cvtGM);
     AddFormattedItem(ctl, item, EMRPOLYTEXTOUTA, rec, FLOAT, exScale, cvt);
     AddFormattedItem(ctl, item, EMRPOLYTEXTOUTA, rec, FLOAT, eyScale, cvt);
     AddFormattedItem(ctl, item, EMRPOLYTEXTOUTA, rec, LONG,  cStrings, cvt);
     HTREEITEM t = ctl->InsertItem(_T("EMRTEXT aemrtext[1]"), item);
     FormatEMRTextAa(&(((EMRPOLYTEXTOUTA*)rec)->aemrtext[0]), p1, (LPVOID)t, (LPCVOID)rec,((EMRPOLYTEXTOUTA*)rec)->cStrings);
    } // TreeDecode::DecodePolyTextOutA

/****************************************************************************
*                       TreeDecode::DecodePolyTextOutW
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(EMRPOLYTEXTOUTW*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEM) item to add to
* Result: void
*       
* Effect: 
*       Handles EMRPOLYTEXTOUTW
* Notes:
*    typedef struct {
*      EMR     emr; 
*      RECTL   rclBounds; 
*      DWORD   iGraphicsMode; 
*      FLOAT   exScale; 
*      FLOAT   eyScale; 
*      LONG    cStrings; 
*      EMRTEXT aemrtext[1]; 
*    } EMRPOLYTEXTOUTW
****************************************************************************/

void TreeDecode::DecodePolyTextOutW(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();

     SetState(ctl, item, StateIcon::Drawing);
     AddFormattedItem(ctl, item, EMRPOLYTEXTOUTW, rec, RECTL, rclBounds, cvt);
     AddFormattedItem(ctl, item, EMRPOLYTEXTOUTW, rec, DWORD, iGraphicsMode, cvtGM);
     AddFormattedItem(ctl, item, EMRPOLYTEXTOUTW, rec, FLOAT, exScale, cvt);
     AddFormattedItem(ctl, item, EMRPOLYTEXTOUTW, rec, FLOAT, eyScale, cvt);
     AddFormattedItem(ctl, item, EMRPOLYTEXTOUTW, rec, LONG,  cStrings, cvt);
     HTREEITEM t = ctl->InsertItem(_T("EMRTEXT aemrtext[1]"), item);
     FormatEMRTextWa(&(((EMRPOLYTEXTOUTW*)rec)->aemrtext[0]), p1, (LPVOID)t, (LPCVOID)rec,((EMRPOLYTEXTOUTW*)rec)->cStrings);
    } // TreeDecode::DecodePolyTextOutW

/****************************************************************************
*                      TreeDecode::DecodeRealizePalette
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(EMRREALIZEPALETTE*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEM) item to add to
* Result: void
*       
* Effect: 
*       Handles EMRREALIZEPALETTE
* Notes:
*    typedef struct { 
*      EMR emr; 
*        } EMRREALIZEPALETTE;
****************************************************************************/

void TreeDecode::DecodeRealizePalette(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();
     
     SetState(ctl, item, StateIcon::State);
    } // TreeDecode::DecodeRealizePalette

/****************************************************************************
*                           TreeDecode::DecodeRectangle
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(EMRRECTANGLE*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEM) item to add to
* Result: void
*       
* Effect: 
*       Decodes EMRRECTANGLE
* Notes:
*    typedef struct {
*      EMR   emr; 
*      RECTL rclBox; 
*    } EMRRECTANGLE;
****************************************************************************/

void TreeDecode::DecodeRectangle(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();

     SetState(ctl, item, StateIcon::Drawing);
     AddFormattedItem(ctl, item, EMRRECTANGLE, rec, RECTL, rclBox, cvt);
    } // TreeDecode::DecodeRectangle

/****************************************************************************
*                       TreeDecode::DecodeResizePalette
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(EMRRESIZEPALETTE*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEweM) item to add to
* Result: void
*       
* Effect: 
*       Handles EMRRESIZEPALETTE
* Notes:
*    typedef struct { 
*      EMR emr; 
*    } EMRREALIZEPALETTE;
****************************************************************************/

void TreeDecode::DecodeResizePalette(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();

     SetState(ctl, item, StateIcon::State);
     AddFormattedItem(ctl, item, EMRRESIZEPALETTE, rec, DWORD, ihPal, cvt);
     AddFormattedItem(ctl, item, EMRRESIZEPALETTE, rec, DWORD, cEntries, cvt);
    } // TreeDecode::DecodeResizePalette

/****************************************************************************
*                         TreeDecode::DecodeRestoreDC
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(EMRRESTOREDC*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEM) item to add to
* Result: void
*       
* Effect: 
*       Handles EMRRESTOREDC
* Notes:
*    typedef struct {
*      EMR  emr; 
*      LONG iRelative; 
*    } EMRRESTOREDC;
****************************************************************************/

void TreeDecode::DecodeRestoreDC(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();

     SetState(ctl, item, StateIcon::State);
     AddFormattedItem(ctl, item, EMRRESTOREDC, rec, LONG, iRelative, cvt);
    } // TreeDecode::DecodeRestoreDC

/****************************************************************************
*                           TreeDecode::DecodeRoundRect
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(EMRROUNDRECT*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEM) item to add to
* Result: void
*       
* Effect: 
*       Decodes EMRROUNDRECT
* Notes:
*    typedef struct {
*      EMR   emr; 
*      RECTL rclBox; 
*      SIZEL szlCorner; 
*    } EMRROUNDRECT;
****************************************************************************/

void TreeDecode::DecodeRoundRect(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();

     SetState(ctl, item, StateIcon::Drawing);
     AddFormattedItem(ctl, item, EMRROUNDRECT, rec, RECTL, rclBox, cvt);
     AddFormattedItem(ctl, item, EMRROUNDRECT, rec, SIZEL, szlCorner, cvt);
    } // TreeDecode::DecodeRoundRect

/****************************************************************************
*                           TreeDecode::DecodeSaveDC
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(EMRSAVEDC*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEM) item to add to
* Result: void
*       
* Effect: 
*       Marks element as state
* Notes:
*    typedef struct { 
*      EMR emr; 
*    }  EMRSAVEDC;
****************************************************************************/

void TreeDecode::DecodeSaveDC(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();
     
     SetState(ctl, item, StateIcon::State);
    } // TreeDecode::DecodeSaveDC

/****************************************************************************
*                    TreeDecode::DecodeScaleViewportExtEx
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(EMRSCALEVIEWPORTEXTEX*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEM) item to add to
* Result: void
*       
* Effect: 
*       Handles EMRSCALEVIEWPORTEXTEX
* Notes:
*    typedef struct {
*      EMR  emr; 
*      LONG xNum; 
*      LONG xDenom; 
*      LONG yNum; 
*      LONG yDenom; 
*    } EMRSCALEVIEWPORTEXTEX;
****************************************************************************/

void TreeDecode::DecodeScaleViewportExtEx(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();
     
     SetState(ctl, item, StateIcon::Drawing);
     AddFormattedItem(ctl, item, EMRSCALEVIEWPORTEXTEX, rec, LONG, xNum, cvt);
     AddFormattedItem(ctl, item, EMRSCALEVIEWPORTEXTEX, rec, LONG, xDenom, cvt);
     AddFormattedItem(ctl, item, EMRSCALEVIEWPORTEXTEX, rec, LONG, yNum, cvt);
     AddFormattedItem(ctl, item, EMRSCALEVIEWPORTEXTEX, rec, LONG, yDenom, cvt);
    } // TreeDecode::DecodeScaleViewportExtEx

/****************************************************************************
*                    TreeDecode::DecodeScaleWindowExtEx
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(EMRSCALEWINDOWEXTEX*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEM) item to add to
* Result: void
*       
* Effect: 
*       Handles EMRSCALEWINDOWEXTEX
* Notes:
*    typedef struct {
*      EMR  emr; 
*      LONG xNum; 
*      LONG xDenom; 
*      LONG yNum; 
*      LONG yDenom; 
*    } EMRSCALEWINDOWEXTEX;
****************************************************************************/

void TreeDecode::DecodeScaleWindowExtEx(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();
     
     SetState(ctl, item, StateIcon::Drawing);
     AddFormattedItem(ctl, item, EMRSCALEWINDOWEXTEX, rec, LONG, xNum, cvt);
     AddFormattedItem(ctl, item, EMRSCALEWINDOWEXTEX, rec, LONG, xDenom, cvt);
     AddFormattedItem(ctl, item, EMRSCALEWINDOWEXTEX, rec, LONG, yNum, cvt);
     AddFormattedItem(ctl, item, EMRSCALEWINDOWEXTEX, rec, LONG, yDenom, cvt);
    } // TreeDecode::DecodeScaleWindowExtEx

/****************************************************************************
*                         TreeDecode::DecodeSelectClipPath
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(EMRSELECTCLIPPATH*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEM) item to add to
* Result: void
*       
* Effect: 
*       Handles EMRSELECTCLIPPATH
* Notes:
*    typedef struct {
*      EMR   emr; 
*      DWORD iMode; 
*    } EMRSELECTCLIPPATH;
****************************************************************************/

void TreeDecode::DecodeSelectClipPath(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();

     SetState(ctl, item, StateIcon::State);
     AddFormattedItem(ctl, item, EMRSELECTCLIPPATH, rec, DWORD, iMode, cvtRGN);
     
    } // TreeDecode::DecodeSelectClipPath

/****************************************************************************
*                     TreeDecode::DecodeSelectColorSpace
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(EMRSELECTCOLORSPACE*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEM) item to add to
* Result: void
*       
* Effect: 
*       Handles EMRSELECTCOLORSPACE
* Notes:
*    typedef struct { 
*      EMR     emr; 
*      DWORD   ihCS; 
*    } EMRSELECTCOLORSPACE;
****************************************************************************/

void TreeDecode::DecodeSelectColorSpace(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();

     SetState(ctl, item, StateIcon::State);
     AddFormattedItem(ctl, item, EMRSELECTCOLORSPACE, rec, DWORD, ihCS, cvt);
    } // TreeDecode::DecodeSelectColorSpace

/****************************************************************************
*                          TreeDecode::DecodeSelectObject
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(EMRSELECTOBJECT*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEM) item to add to
* Result: void
*       
* Effect: 
*       Decodes EMRSELECTOBJECT
* Notes:
*    typedef struct {
*      EMR   emr; 
*      DWORD ihObject;
*    } EMRSELECTOBJECT;
****************************************************************************/

void TreeDecode::DecodeSelectObject(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();
     
     SetState(ctl, item, StateIcon::State);
     AddFormattedItem(ctl, item, EMRSELECTOBJECT, rec, DWORD, ihObject, cvtOB);
    } // TreeDecode::DecodeSelectObject

/****************************************************************************
*                          TreeDecode::DecodeSelectPalette
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(EMRSELECTPALETTE*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEM) item to add to
* Result: void
*       
* Effect: 
*       Decodes EMRSELECTPALETTE
* Notes:
*    typedef struct {
*      EMR   emr; 
*      DWORD ihPal; 
*    } EMRSELECTPALETTE;
****************************************************************************/

void TreeDecode::DecodeSelectPalette(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();
     
     SetState(ctl, item, StateIcon::State);
     AddFormattedItem(ctl, item, EMRSELECTPALETTE, rec, DWORD, ihPal, cvtOB);
    } // TreeDecode::DecodeSelectPalette

/****************************************************************************
*                      TreeDecode::DecodeSetArcDirection
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(EMRSETARCDIRECTION*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEM) item to add to
* Result: void
*       
* Effect: 
*       Handles EMRSETARCDIRECTION
* Notes:
*    typedef struct {
*      EMR   emr; 
*      DWORD iArcDirection;
*    } EMRSETARCDIRECTION;
****************************************************************************/

void TreeDecode::DecodeSetArcDirection(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();

     SetState(ctl, item, StateIcon::State);
     AddFormattedItem(ctl, item, EMRSETARCDIRECTION, rec, DWORD, iArcDirection, cvtAD);
    } // TreeDecode::DecodeSetArcDirection

/****************************************************************************
*                         TreeDecode::DecodeSetBkColor
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(EMRSETBKCOLOR*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEM) item to add to
* Result: void
*       
* Effect: 
*       Decodes EMRSETBKCOLOR
* Notes:
*    typedef struct {
*      EMR      emr; 
*      COLORREF crColor; 
*    } EMRSETBKCOLOR
****************************************************************************/

void TreeDecode::DecodeSetBkColor(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();

     SetState(ctl, item, StateIcon::State);
     AddFormattedItem(ctl, item, EMRSETBKCOLOR, rec, COLORREF, crColor, cvtCR);
    } // TreeDecode::DecodeBkSetColor

/****************************************************************************
*                         TreeDecode::DecodeSetBkMode
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(EMRSETBKMODE*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEM) item to add to
* Result: void
*       
* Effect: 
*       Handles EMRSETBKMODE
* Notes:
*    typedef struct {
*      EMR   emr; 
*      DWORD iMode; 
*    } EMRSETBKMODE;
****************************************************************************/

void TreeDecode::DecodeSetBkMode(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();

     SetState(ctl, item, StateIcon::State);
     AddFormattedItem(ctl, item, EMRSETBKMODE, rec, DWORD, iMode, cvtBKM);
    } // TreeDecode::DecodeSetBkMode

/****************************************************************************
*                            TreeDecode::DecodeSetBrushOrgEx
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(EMRSETBRUSHORGEX*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEM) item to add to
* Result: void
*       
* Effect: 
*       Decodes EMRSETBRUSHORGEX
* Notes:
*    typedef struct {
*      EMR    emr; 
*      POINTL ptlOrigin; 
*    } EMRSETBRUSHORGEX;
****************************************************************************/

void TreeDecode::DecodeSetBrushOrgEx(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();

     SetState(ctl, item, StateIcon::State);
     AddFormattedItem(ctl, item, EMRSETBRUSHORGEX, rec, POINTL, ptlOrigin, cvt);
    } // TreeDecode::DecodeSetBrushOrgEx

/****************************************************************************
*                    TreeDecode::DecodeSetColorAdjustment
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(EMRSETCOLORADJUSTMENT*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEM) item to add to
* Result: void
*       
* Effect: 
*       Handles EMRSETCOLORADJUSTMENT
* Notes:
*    typedef struct {
*      EMR  emr; 
*      COLORADJUSTMENT ColorAdjustment; 
*    } EMRSETCOLORADJUSTMENT
****************************************************************************/

void TreeDecode::DecodeSetColorAdjustment(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();

     SetState(ctl, item, StateIcon::State);
     HTREEITEM ca = ctl->InsertItem(_T("COLORADJUSTMENT ColorAdjustment"), item);
     FormatCOLORADJUSTMENT(&(((EMRSETCOLORADJUSTMENT*)rec)->ColorAdjustment), p1, (LPVOID)ca);
    } // TreeDecode::DecodeSetColorAdjustment

/****************************************************************************
*                     TreeDecode::DecodeSetColorSpace
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(EMRSETCOLORSPACE*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEM) item to add to
* Result: void
*       
* Effect: 
*       Handles EMRSETCOLORSPACE
* Notes:
*    typedef struct { 
*      EMR     emr; 
*      DWORD   ihCS; 
*    } EMRSETCOLORSPACE;
****************************************************************************/

void TreeDecode::DecodeSetColorSpace(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();

     SetState(ctl, item, StateIcon::State);
     AddFormattedItem(ctl, item, EMRSETCOLORSPACE, rec, DWORD, ihCS, cvt);
    } // TreeDecode::DecodeSetColorSpace

/****************************************************************************
*                        TreeDecode::SetDIBitsToDevice
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(EMRSETDIBITSTODEVICE*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEM) item to add to
* Result: void
*       
* Effect: 
*       Handes EMRSETDIBITSTODEVICE
* Notes:
*    typedef struct {
*      EMR   emr; 
*      RECTL rclBounds; 
*      LONG  xDest; 
*      LONG  yDest; 
*      LONG  xSrc; 
*      LONG  ySrc; 
*      LONG  cxSrc; 
*      LONG  cySrc; 
*      DWORD offBmiSrc; 
*      DWORD cbBmiSrc; 
*      DWORD offBitsSrc; 
*      DWORD cbBitsSrc; 
*      DWORD iUsageSrc; 
*      DWORD iStartScan; 
*      DWORD cScans; 
*    } EMRSETDIBITSTODEVICE
****************************************************************************/

void TreeDecode::DecodeSetDIBitsToDevice(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();

     SetState(ctl, item, StateIcon::Drawing); // TODO: Is this State or Drawing?

     AddFormattedItem(ctl, item, EMRSETDIBITSTODEVICE, rec, RECTL, rclBounds, cvt);
     AddFormattedItem(ctl, item, EMRSETDIBITSTODEVICE, rec, LONG, xDest, cvt);
     AddFormattedItem(ctl, item, EMRSETDIBITSTODEVICE, rec, LONG, yDest, cvt);
     AddFormattedItem(ctl, item, EMRSETDIBITSTODEVICE, rec, LONG, xSrc, cvt);
     AddFormattedItem(ctl, item, EMRSETDIBITSTODEVICE, rec, LONG, ySrc, cvt);
     AddFormattedItem(ctl, item, EMRSETDIBITSTODEVICE, rec, LONG, cxSrc, cvt);
     AddFormattedItem(ctl, item, EMRSETDIBITSTODEVICE, rec, LONG, cySrc, cvt);
     HTREEITEM bmi;
     AddFormattedItemEx(ctl, item, bmi, EMRSETDIBITSTODEVICE, rec, DWORD, offBmiSrc, cvt);
     if(((EMRSETDIBITSTODEVICE *)rec)->offBmiSrc != 0)
        FormatBITMAPINFO((LPBITMAPINFO)((LPBYTE)rec + ((EMRSETDIBITSTODEVICE *)rec)->offBmiSrc), p1, (LPVOID)bmi);
     AddFormattedItem(ctl, item, EMRSETDIBITSTODEVICE, rec, DWORD, cbBmiSrc, cvt);

     AddFormattedItem(ctl, item, EMRSETDIBITSTODEVICE, rec, DWORD, offBitsSrc, cvt);
     AddFormattedItem(ctl, item, EMRSETDIBITSTODEVICE, rec, DWORD, cbBitsSrc, cvt);
     // TODO: decode bitmap
     AddFormattedItem(ctl, item, EMRSETDIBITSTODEVICE, rec, DWORD, iUsageSrc, cvtDBC);
     AddFormattedItem(ctl, item, EMRSETDIBITSTODEVICE, rec, DWORD, iStartScan, cvt);
     AddFormattedItem(ctl, item, EMRSETDIBITSTODEVICE, rec, DWORD, cScans, cvt);

    } // TreeDecode::DecodeSetDIBitsToDevice

/****************************************************************************
*                        TreeDecode::DecodeSetICMMode
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(EMRSETICMMODE*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEM) item to add to
* Result: void
*       
* Effect: 
*       Decodes EMRSETICMMODE
* Notes:
*    typedef struct {
*      EMR   emr; 
*      DWORD iMode; 
*    } EMRSETICMMODE;
****************************************************************************/

void TreeDecode::DecodeSetICMMode(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();
     
     SetState(ctl, item, StateIcon::State);
     AddFormattedItem(ctl, item, EMRSETICMMODE, rec, DWORD, iMode, cvtIC);
    } // TreeDecode::DecodeSetICMMode

/****************************************************************************
*                      TreeDecode::DecodeSetICMProfileA
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(EMRSETICMPROFILEA*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEM) item to add to
* Result: void
*       
* Effect: 
*       Handles EMRSETICMPROFILEA
* Notes:
*    typedef struct { 
*      EMR     emr;
*      DWORD   dwFlags;
*      DWORD   cbName; 
*      DWORD   cbData; 
*      BYTE    Data[1]; 
*    } EMRSETICMPROFILEA
*****************************************************************************/

void TreeDecode::DecodeSetICMProfileA(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();

     SetState(ctl, item, StateIcon::State);
     AddItem(ctl, item, EMRSETICMPROFILEA, rec, DWORD, dwFlags, _T("0x%08x"));// TODO: undocumented fields
     AddFormattedItem(ctl, item, EMRSETICMPROFILEA, rec, DWORD, cbName, cvt);
     AddFormattedItem(ctl, item, EMRSETICMPROFILEA, rec, DWORD, cbData, cvt);
     HTREEITEM nm = ctl->InsertItem(_T("BYTE Data[1]"), item);
     LPBYTE b = &(((EMRSETICMPROFILEA*)rec)->Data[0]);
     LPBYTE data = b;

     DWORD len = ((EMRSETICMPROFILEA*)rec)->cbName;
     if(len > 0)
        { /* has string */
         CStringA nameA = CStringA((LPCSTR)b, len);
         CStringW nameW(nameA);
         nameW = cvtString(nameW); // handle escapes
         CString s;
         s.Format(_T("[0] \"%ls\""), nameW);
         ctl->InsertItem(s, nm);
         data = b + len;
        } /* has string */
     // TODO: show data
    } // TreeDecode::DecodeSetICMProfileA

/****************************************************************************
*                      TreeDecode::DecodeSetICMProfileW
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(EMRSETICMPROFILEW*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEM) item to add to
* Result: void
*       
* Effect: 
*       Handles EMRSETICMPROFILEW
* Notes:
*    typedef struct tagEMRSETICMPROFILE { 
*      EMR     emr;
*      DWORD   dwFlags;
*      DWORD   cbName; 
*      DWORD   cbData; 
*      BYTE    Data[1]; 
*    } EMRSETICMPROFILEW;
*****************************************************************************/

void TreeDecode::DecodeSetICMProfileW(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();

     SetState(ctl, item, StateIcon::State);
     AddItem(ctl, item, EMRSETICMPROFILEW, rec, DWORD, dwFlags, _T("0x%08x"));// TODO: undocumented fields
     AddFormattedItem(ctl, item, EMRSETICMPROFILEW, rec, DWORD, cbName, cvt);
     AddFormattedItem(ctl, item, EMRSETICMPROFILEW, rec, DWORD, cbData, cvt);
     HTREEITEM nm = ctl->InsertItem(_T("BYTE Data[1]"), item);
     LPBYTE b = &(((EMRSETICMPROFILEW*)rec)->Data[0]);
     LPBYTE data = b;

     DWORD len = ((EMRSETICMPROFILEW*)rec)->cbName;
     if(len > 0)
        { /* has string */
         CStringW nameW = CStringW((LPCWSTR)b, len / 2);  // TODO: really /2?  Check this! /2 is just a guess!
         nameW = cvtString(nameW); // handle escapes
         CString s;
         s.Format(_T("[0] \"%ls\""), nameW);
         ctl->InsertItem(s, nm);
         data = b + len;
        } /* has string */
     // TODO: show data
    } // TreeDecode::DecodeSetICMProfileW

/****************************************************************************
*                         TreeDecode::DecodeSetLayout
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(EMRSETLAYOUT*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEweM) item to add to
* Result: void
*       
* Effect: 
*       Handles EMRSETLAYOUT
* Notes:
*    typedef struct {
*      EMR   emr; 
*      DWORD iMode; 
*    } EMRSETLAYOUT;
****************************************************************************/

void TreeDecode::DecodeSetLayout(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();

     SetState(ctl, item, StateIcon::State);
     AddFormattedItem(ctl, item, EMRSETLAYOUT, rec, DWORD, iMode, cvtLY);
    } // TreeDecode::DecodeSetLayout

/****************************************************************************
*                        TreeDecode::DecodeSetMapMode
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(EMRSETMAPMODE*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEM) item to add to
* Result: void
*       
* Effect: 
*       Decodes EMRSETMAPMODE
* Notes:
*    typedef struct {
*      EMR   emr; 
*      DWORD iMode; 
*    } EMRSETMAPMODE;
****************************************************************************/

void TreeDecode::DecodeSetMapMode(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();
     
     SetState(ctl, item, StateIcon::State);
     AddFormattedItem(ctl, item, EMRSETMAPMODE, rec, DWORD, iMode, cvtMM);
    } // TreeDecode::DecodeSetMapMode

/****************************************************************************
*                      TreeDecode::DecodeSetMapperFlags
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(EMRSETMAPPERFLAGS*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEM) item to add to
* Result: void
*       
* Effect: 
*       Handles EMRSETMAPPERFLAGS
* Notes:
*    typedef struct tagEMRSETMAPPERFLAGS {
*      EMR   emr; 
*      DWORD dwFlags; 
*    } EMRSETMAPPERFLAGS
****************************************************************************/

void TreeDecode::DecodeSetMapperFlags(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();

     SetState(ctl, item, StateIcon::State);
     AddFormattedItem(ctl, item, EMRSETMAPPERFLAGS, rec, DWORD, dwFlags, cvt);
     
    } // TreeDecode::DecodeSetMapperFlags

/****************************************************************************
*                           TreeDecode::DecodeSetMetaRegion
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(EMRSETMETARGN*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEM) item to add to
* Result: void
*       
* Effect: 
*       Marks element as state
* Notes:
*    typedef struct { 
*      EMR emr; 
*    } EMRSETMETARGN;
****************************************************************************/

void TreeDecode::DecodeSetMetaRegion(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();
     
     SetState(ctl, item, StateIcon::State);
    } // TreeDecode::DecodeSetMetaRegion

/****************************************************************************
*                          TreeDecode::DecodeSetMiterLimit
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(EMRSETMITERLIMIT*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEM) item to add to
* Result: void
*       
* Effect: 
*       Handles EMRSETMITERLIMIT
* Notes:
*    typedef struct {
*      EMR   emr;
*      FLOAT eMiterLimit; 
*    } EMRSETMITERLIMIT
****************************************************************************/

void TreeDecode::DecodeSetMiterLimit(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();

     SetState(ctl, item, StateIcon::State);
     
     AddFormattedItem(ctl, item, EMRSETMITERLIMIT, rec, FLOAT, eMiterLimit, cvt);
    } // TreeDecode::DecodeSetMiterLimit

/****************************************************************************
*                     TreeDecode::DecodeSetPaletteEntries
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(EMRSETPALETTEENTRIES*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEM) item to add to
* Result: void
*       
* Effect: 
*       Handles EMRSETPALETTEENTRIES
* Notes:
*    typedef struct {
*      EMR          emr; 
*      DWORD        ihPal; 
*      DWORD        iStart; 
*      DWORD        cEntries; 
*      PALETTEENTRY aPalEntries[1]; 
*    } EMRSETPALETTEENTRIES
*****************************************************************************/

void TreeDecode::DecodeSetPaletteEntries(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();

     SetState(ctl, item, StateIcon::State);
     AddFormattedItem(ctl, item, EMRSETPALETTEENTRIES, rec, DWORD, ihPal, cvt);
     AddFormattedItem(ctl, item, EMRSETPALETTEENTRIES, rec, DWORD, iStart, cvt);
     AddFormattedItem(ctl, item, EMRSETPALETTEENTRIES, rec, DWORD, cEntries, cvt);
     HTREEITEM pe = ctl->InsertItem(_T("PALETTEENTRY aPalEntries[1]"), item);
     FormatPALETTEENTRIES(&(((EMRSETPALETTEENTRIES*)rec)->aPalEntries[0]), ((EMRSETPALETTEENTRIES*)rec)->cEntries, p1, (LPVOID)pe);
    } // TreeDecode::DecodeSetPaletteEntries

/****************************************************************************
*                         TreeDecode::DecodeSetPixelV
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(EMRSETPIXELV*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEweM) item to add to
* Result: void
*       
* Effect: 
*       Handles EMRSETPIXELV
* Notes:
*    typedef struct {
*      EMR     emr; 
*      POINTL  ptlPixel; 
*      COLORREF crColor; 
*    } EMRSETPIXELV
****************************************************************************/

void TreeDecode::DecodeSetPixelV(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();

     SetState(ctl, item, StateIcon::Drawing);
     AddFormattedItem(ctl, item, EMRSETPIXELV, rec, POINTL, ptlPixel, cvt);
     AddFormattedItem(ctl, item, EMRSETPIXELV, rec, COLORREF, crColor, cvtCR);
    } // TreeDecode::DecodeSetPixelV

/****************************************************************************
*                      TreeDecode::DecodeSetPolyFillMode
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(EMRSETPOLYFILLMODE*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEM) item to add to
* Result: void
*       
* Effect: 
*       Handles EMRSETPOLYFILLMODE
* Notes:
*    typedef struct {
*      EMR   emr; 
*      DWORD iMode; 
*    } EMRSETPOLYFILLMODE;
****************************************************************************/

void TreeDecode::DecodeSetPolyFillMode(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();

     SetState(ctl, item, StateIcon::State);
     AddFormattedItem(ctl, item, EMRSETPOLYFILLMODE, rec, DWORD, iMode, cvtPFM);
    } // TreeDecode::DecodeSetPolyFillMode

/****************************************************************************
*                        TreeDecode::DecodeSetROP2
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(EMRSETROP2*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEM) item to add to
* Result: void
*       
* Effect: 
*       Decodes EMRSETROP2
* Notes:
*    typedef struct {
*      EMR   emr; 
*      DWORD iMode; 
*    } EMRSETROP2;
****************************************************************************/

void TreeDecode::DecodeSetROP2(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();

     SetState(ctl, item, StateIcon::State);
     AddFormattedItem(ctl, item, EMRSETROP2, rec, DWORD, iMode, cvtROP2);
    } // TreeDecode::DecodeSetROP2

/****************************************************************************
*                      TreeDecode::DecodeSetStretchBltMode
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(EMRSETSTRETCHBLTMODE*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEM) item to add to
* Result: void
*       
* Effect: 
*       Decodes EMRSETSTRETCHBLTMODE
* Notes:
*    typedef struct {
*      EMR   emr; 
*      DWORD iMode; 
*    } EMRSETSTRETCHBLTMODE;
****************************************************************************/

void TreeDecode::DecodeSetStretchBltMode(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();

     SetState(ctl, item, StateIcon::State);
     AddFormattedItem(ctl, item, EMRSETSTRETCHBLTMODE, rec, DWORD, iMode, cvtSBM);
    } // TreeDecode::DecodeSetStretchBltMode

/****************************************************************************
*                         TreeDecode::DecodeSetTextAlign
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(EMRSETTEXTALIGN*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEM) item to add to
* Result: void
*       
* Effect: 
*       Handles EMRSETTEXTALIGN
* Notes:
*    typedef struct {
*      EMR   emr; 
*      DWORD iMode; 
*    } EMRSETTEXTALIGN;
****************************************************************************/

void TreeDecode::DecodeSetTextAlign(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();

     SetState(ctl, item, StateIcon::State);
     AddFormattedItem(ctl, item, EMRSETTEXTALIGN, rec, DWORD, iMode, cvtTA);
    } // TreeDecode::DecodeSetTextAlign

/****************************************************************************
*                         TreeDecode::DecodeSetTextColor
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(EMRSETTEXTCOLOR*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEM) item to add to
* Result: void
*       
* Effect: 
*       Decodes EMRSETTEXTCOLOR
* Notes:
*    typedef struct {
*      EMR      emr; 
*      COLORREF crColor; 
*    } EMRSETTEXTCOLOR;
****************************************************************************/

void TreeDecode::DecodeSetTextColor(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();

     SetState(ctl, item, StateIcon::State);
     AddFormattedItem(ctl, item, EMRSETTEXTCOLOR, rec, COLORREF, crColor, cvtCR);
    } // TreeDecode::DecodeSetTextColor

/****************************************************************************
*                     TreeDecode::DecodeSetViewportExtEx
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(EMRSETVIEWPORTEXTEX*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEM) item to add to
* Result: void
*       
* Effect: 
*       Decodes EMRSETVIEWPORTEXTEX
* Notes:
*    typedef struct {
*      EMR   emr; 
*      SIZEL szlExtent; 
*    } EMRSETVIEWPORTEXTEX;
****************************************************************************/

void TreeDecode::DecodeSetViewportExtEx(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();

     SetState(ctl, item, StateIcon::State);
     AddFormattedItem(ctl, item, EMRSETVIEWPORTEXTEX, rec, SIZEL, szlExtent, cvt);
    } // TreeDecode::DecodeSetViewportExtEx

/****************************************************************************
*                       TreeDecode::DecodeSetViewportOrgEx
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(EMRSETVIEWPORTORGEX*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEM) item to add to
* Result: void
*       
* Effect: 
*       Decodes EMRSETVIEWPORTORGEX
* Notes:
*    typedef struct {
*      EMR    emr; 
*      POINTL ptlOrigin; 
*    } EMRSETVIEWPORTORGEX;
****************************************************************************/

void TreeDecode::DecodeSetViewportOrgEx(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();

     SetState(ctl, item, StateIcon::State);
     AddFormattedItem(ctl, item, EMRSETVIEWPORTORGEX, rec, POINTL, ptlOrigin, cvt);
    } // TreeDecode::DecodeSetViewportOrgEx

/****************************************************************************
*                        TreeDecode::DecodeSetWindowExtEx
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(EMRSETWINDOWEXTEX*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEM) item to add to
* Result: void
*       
* Effect: 
*       Decodes EMRSETWINDOWEXTEX
* Notes:
*    typedef struct {
*      EMR   emr; 
*      SIZEL szlExtent; 
*    } EMRSETWINDOWEXTEX;
****************************************************************************/

void TreeDecode::DecodeSetWindowExtEx(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();

     SetState(ctl, item, StateIcon::State);
     AddFormattedItem(ctl, item, EMRSETWINDOWEXTEX, rec, SIZEL, szlExtent, cvt);
    } // TreeDecode::DecodeSetWindowExtEx

/****************************************************************************
*                        TreeDecode::DecodeSetWindowOrgEx
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(EMRSETWINDOWORGEX*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEM) item to add to
* Result: void
*       
* Effect: 
*       Decodes EMRSETWINDOWORGEX
* Notes:
*    typedef struct tagEMRSETVIEWPORTORGEX {
*      EMR    emr; 
*      POINTL ptlOrigin; 
*    } EMRSETWINDOWORGEX
****************************************************************************/

void TreeDecode::DecodeSetWindowOrgEx(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();

     SetState(ctl, item, StateIcon::State);
     AddFormattedItem(ctl, item, EMRSETVIEWPORTORGEX, rec, POINTL, ptlOrigin, cvt);
    } // TreeDecode::DecodeSetWindowOrgEx

/****************************************************************************
*                   TreeDecode::DecodeSetWorldTransform
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(EMRSETWORLDTRANSFORM*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEM) item to add to
* Result: void
*       
* Effect: 
*       Decodes EMRSETWORLDTRANSFORM
* Notes:
*    typedef struct {
*      EMR   emr; 
*      XFORM xform; 
*    } EMRSETWORLDTRANSFORM;
****************************************************************************/

void TreeDecode::DecodeSetWorldTransform(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();

     SetState(ctl, item, StateIcon::State);
     AddFormattedItem(ctl, item, EMRSETWORLDTRANSFORM, rec, XFORM, xform, cvt);
    } // TreeDecode::DecodeSetWorldTransform

/****************************************************************************
*                        TreeDecode::DecodeStretchBlt
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(EMRSTRETCHBLT*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEM) item to add to
* Result: void
*       
* Effect: 
*       Handles EMRSTRETCHBLT
* Notes:
*    typedef struct {
*      EMR      emr; 
*      RECTL    rclBounds; 
*      LONG     xDest; 
*      LONG     yDest; 
*      LONG     cxDest; 
*      LONG     cyDest; 
*      DWORD    dwRop; 
*      LONG     xSrc; 
*      LONG     ySrc; 
*      XFORM    xformSrc; 
*      COLORREF crBkColorSrc; 
*      DWORD    iUsageSrc; 
*      DWORD    offBmiSrc; 
*      DWORD    cbBmiSrc; 
*      DWORD    offBitsSrc; 
*      DWORD    cbBitsSrc; 
*      LONG     cxSrc; 
*      LONG     cySrc; 
*    } EMRSTRETCHBLT;
****************************************************************************/

void TreeDecode::DecodeStretchBlt(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();

     SetState(ctl, item, StateIcon::Drawing);

     AddFormattedItem(ctl, item, EMRSTRETCHBLT, rec, RECTL, rclBounds, cvt);
     AddFormattedItem(ctl, item, EMRSTRETCHBLT, rec, LONG, xDest, cvt);
     AddFormattedItem(ctl, item, EMRSTRETCHBLT, rec, LONG, yDest, cvt);
     AddFormattedItem(ctl, item, EMRSTRETCHBLT, rec, LONG, cxDest, cvt);
     AddFormattedItem(ctl, item, EMRSTRETCHBLT, rec, LONG, cyDest, cvt);
     AddFormattedItem(ctl, item, EMRSTRETCHBLT, rec, DWORD, dwRop, cvtROP);
     AddFormattedItem(ctl, item, EMRSTRETCHBLT, rec, LONG, xSrc, cvt);
     AddFormattedItem(ctl, item, EMRSTRETCHBLT, rec, LONG, ySrc, cvt);
     AddFormattedItem(ctl, item, EMRSTRETCHBLT, rec, XFORM, xformSrc, cvt);
     AddFormattedItem(ctl, item, EMRSTRETCHBLT, rec, COLORREF, crBkColorSrc, cvtCR);
     AddFormattedItem(ctl, item, EMRSTRETCHBLT, rec, DWORD, iUsageSrc, cvtDBC);
     HTREEITEM bmi;
     AddFormattedItemEx(ctl, item, bmi, EMRSTRETCHBLT, rec, DWORD, offBmiSrc, cvt);
     if(((EMRSTRETCHBLT *)rec)->offBmiSrc != 0)
        FormatBITMAPINFO((LPBITMAPINFO)((LPBYTE)rec + ((EMRSTRETCHBLT *)rec)->offBmiSrc), p1, (LPVOID)bmi);
     AddFormattedItem(ctl, item, EMRSTRETCHBLT, rec, DWORD, cbBmiSrc, cvt);
     AddFormattedItem(ctl, item, EMRSTRETCHBLT, rec, DWORD, offBitsSrc, cvt);
     AddFormattedItem(ctl, item, EMRSTRETCHBLT, rec, DWORD, cbBitsSrc, cvt);
     // TODO: decode bitmap
     AddFormattedItem(ctl, item, EMRSTRETCHBLT, rec, LONG, cxSrc, cvt);
     AddFormattedItem(ctl, item, EMRSTRETCHBLT, rec, LONG, cySrc, cvt);
    } // TreeDecode::DecodeStretchBlt

/****************************************************************************
*                        TreeDecode::DecodeStretchDIBits
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(EMRSTRETCHDIBITS*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEM) item to add to
* Result: void
*       
* Effect: 
*       Handles EMRSTRETCHDIBITS
* Notes:
*    typedef struct {
*      EMR   emr;
*      RECTL rclBounds; 
*      LONG  xDest; 
*      LONG  yDest; 
*      LONG  xSrc; 
*      LONG  ySrc; 
*      LONG  cxSrc; 
*      LONG  cySrc;
*      DWORD offBmiSrc;
*      DWORD cbBmiSrc; 
*      DWORD offBitsSrc; 
*      DWORD cbBitsSrc; 
*      DWORD iUsageSrc; 
*      DWORD dwRop; 
*      LONG  cxDest; 
*      LONG  cyDest; 
*    } EMRSTRETCHDIBITS;
****************************************************************************/

void TreeDecode::DecodeStretchDIBits(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();

     SetState(ctl, item, StateIcon::Drawing);

     AddFormattedItem(ctl, item, EMRSTRETCHDIBITS, rec, RECTL, rclBounds, cvt);
     AddFormattedItem(ctl, item, EMRSTRETCHDIBITS, rec, LONG, xDest, cvt);
     AddFormattedItem(ctl, item, EMRSTRETCHDIBITS, rec, LONG, yDest, cvt);
     AddFormattedItem(ctl, item, EMRSTRETCHDIBITS, rec, LONG, xSrc, cvt);
     AddFormattedItem(ctl, item, EMRSTRETCHDIBITS, rec, LONG, ySrc, cvt);
     AddFormattedItem(ctl, item, EMRSTRETCHDIBITS, rec, LONG, cxSrc, cvt);
     AddFormattedItem(ctl, item, EMRSTRETCHDIBITS, rec, LONG, cySrc, cvt);
     HTREEITEM bmi;
     AddFormattedItemEx(ctl, item, bmi, EMRSTRETCHDIBITS, rec, DWORD, offBmiSrc, cvt);
     if(((EMRSTRETCHDIBITS *)rec)->offBmiSrc != 0)
        FormatBITMAPINFO((LPBITMAPINFO)((LPBYTE)rec + ((EMRSTRETCHDIBITS *)rec)->offBmiSrc), p1, (LPVOID)bmi);
     AddFormattedItem(ctl, item, EMRSTRETCHDIBITS, rec, DWORD, cbBmiSrc, cvt);
     AddFormattedItem(ctl, item, EMRSTRETCHDIBITS, rec, DWORD, offBitsSrc, cvt);
     AddFormattedItem(ctl, item, EMRSTRETCHDIBITS, rec, DWORD, cbBitsSrc, cvt);
     // TODO: decode bitmap
     AddFormattedItem(ctl, item, EMRSTRETCHDIBITS, rec, DWORD, iUsageSrc, cvtDBC);
     AddFormattedItem(ctl, item, EMRSTRETCHDIBITS, rec, DWORD, dwRop, cvtROP);
     AddFormattedItem(ctl, item, EMRSTRETCHDIBITS, rec, LONG, cxDest, cvt);
     AddFormattedItem(ctl, item, EMRSTRETCHDIBITS, rec, LONG, cyDest, cvt);
    } // TreeDecode::DecodeStretchDIBits

/****************************************************************************
*                         TreeDecode::DecodeStrokeAndFillPath
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(EMRSTROKEANDFILLPATH*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEM) item to add to
* Result: void
*       
* Effect: 
*       Handles EMRSTROKEANDFILLPATH
* Notes:
*    typedef struct tagEMRFILLPATH {
*      EMR   emr; 
*      RECTL rclBounds; 
*    } EMRSTROKEANDFILLPATH;
****************************************************************************/

void TreeDecode::DecodeStrokeAndFillPath(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();

     SetState(ctl, item, StateIcon::Drawing);
     AddFormattedItem(ctl, item, EMRSTROKEANDFILLPATH, rec, RECTL, rclBounds, cvt);
    } // TreeDecode::DecodeStrokeAndFillPath

/****************************************************************************
*                         TreeDecode::DecodeStrokePath
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(EMRSTROKEPATH*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEM) item to add to
* Result: void
*       
* Effect: 
*       Handles EMRSTROKEANDFILLPATH
* Notes:
*    typedef struct tagEMRFILLPATH {
*      EMR   emr; 
*      RECTL rclBounds; 
*    } EMRSTROKEPATH;
****************************************************************************/

void TreeDecode::DecodeStrokePath(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();

     SetState(ctl, item, StateIcon::Drawing);
     AddFormattedItem(ctl, item, EMRSTROKEPATH, rec, RECTL, rclBounds, cvt);
    } // TreeDecode::DecodeStrokePath

/****************************************************************************
*                      TreeDecode::DecodeTransparentBlt
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(EMRTRANSPARENTBLT*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEM) item to add to
* Result: void
*       
* Effect: 
*       Handles EMRTRANSPARENTBLT
* Notes:
*    typedef struct { 
*      EMR     emr; 
*      RECTL   rclBounds; 
*      LONG    xDest;
*      LONG    yDest;
*      LONG    cxDest;
*      LONG    cyDest;
*      DWORD   dwRop;
*      LONG    xSrc;
*      LONG    ySrc;
*      XFORM   xformSrc; 
*      COLORREF crBkColorSrc; 
*      DWORD   iUsageSrc; 
*      DWORD   offBmiSrc;
*      DWORD   cbBmiSrc;
*      DWORD   offBitsSrc;
*      DWORD   cbBitsSrc; 
*      LONG    cxSrc; 
*      LONG    cySrc; 
*    } EMREMRTRANSPARENTBLT
****************************************************************************/

void TreeDecode::DecodeTransparentBlt(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();

     SetState(ctl, item, StateIcon::Drawing);
     
     AddFormattedItem(ctl, item, EMRTRANSPARENTBLT, rec, RECTL, rclBounds, cvt);
     AddFormattedItem(ctl, item, EMRTRANSPARENTBLT, rec, LONG, xDest, cvt);
     AddFormattedItem(ctl, item, EMRTRANSPARENTBLT, rec, LONG, yDest, cvt);
     AddFormattedItem(ctl, item, EMRTRANSPARENTBLT, rec, LONG, cxDest, cvt);
     AddFormattedItem(ctl, item, EMRTRANSPARENTBLT, rec, LONG, cyDest, cvt);
     AddFormattedItem(ctl, item, EMRTRANSPARENTBLT, rec, DWORD, dwRop, cvtROP);
     AddFormattedItem(ctl, item, EMRTRANSPARENTBLT, rec, LONG, xSrc, cvt);
     AddFormattedItem(ctl, item, EMRTRANSPARENTBLT, rec, LONG, ySrc, cvt);
     AddFormattedItem(ctl, item, EMRTRANSPARENTBLT, rec, XFORM, xformSrc, cvt);
     AddFormattedItem(ctl, item, EMRTRANSPARENTBLT, rec, COLORREF, crBkColorSrc, cvtCR);
     AddFormattedItem(ctl, item, EMRTRANSPARENTBLT, rec, DWORD, iUsageSrc, cvtDBC);
     HTREEITEM bmi;
     AddFormattedItemEx(ctl, item, bmi, EMRTRANSPARENTBLT, rec, DWORD, offBmiSrc, cvt);
     if(((EMRTRANSPARENTBLT *)rec)->offBmiSrc != 0)
        FormatBITMAPINFO((LPBITMAPINFO)((LPBYTE)rec + ((EMRTRANSPARENTBLT *)rec)->offBmiSrc), p1, (LPVOID)bmi);
     AddFormattedItem(ctl, item, EMRTRANSPARENTBLT, rec, DWORD, cbBmiSrc, cvt);
     AddFormattedItem(ctl, item, EMRTRANSPARENTBLT, rec, DWORD, offBitsSrc, cvt);
     AddFormattedItem(ctl, item, EMRTRANSPARENTBLT, rec, DWORD, cbBitsSrc, cvt);
    } // TreeDecode::DecodeTransparentBlt

/****************************************************************************
*                           TreeDecode::DecodeWidenPath
* Inputs:
*       const ENHMETARECORD * rec: (ENHMETARECORD*)(EMRWIDENPATH*)
*       LPVOID p1: (LPVOID)(CTreeCtrl *) Control to add it to
*       LPVOID p2: (LPVOID)(HTREEITEM) item to add to
* Result: void
*       
* Effect: 
*       Handles EMRWIDENPATH
* Notes:
*    typedef struct { 
*      EMR emr; 
*    } EMRWIDENPATH;
****************************************************************************/

void TreeDecode::DecodeWidenPath(DECODE_PARAMETERS)
    {
     GET_CTL_ITEM();
     
     SetState(ctl, item, StateIcon::State);
    } // TreeDecode::DecodeWidenPath
